<?php
/* A Proximite - Visualization of third parties around a third party
 * Copyright (C) 2018       Inovea-conseil.com     <info@inovea-conseil.com>
 */

/**
 * \defgroup aproximite Visualization of third parties around a third party
 * \file    core/modules/modAproximite.class.php
 * \ingroup aproximite
 * \brief   ActionsAproximite
 *
 * Visualization of third parties around a third party
 */

include_once DOL_DOCUMENT_ROOT .'/core/modules/DolibarrModules.class.php';
include_once DOL_DOCUMENT_ROOT .'/core/lib/admin.lib.php';
require_once DOL_DOCUMENT_ROOT.'/core/class/extrafields.class.php';

/**
 *  Description and activation class for module MyModule
 */
class modAproximite extends DolibarrModules
{
    /**
     * Constructor. Define names, constants, directories, boxes, permissions
     *
     * @param DoliDB $db Database handler
     */
    public function __construct($db) {
        global $langs,$conf;

        $this->db = $db;

        $this->numero = 432441;

        $this->rights_class = 'Aproximite';

        $this->family = "Inovea Conseil";
        $this->special = 0;

        $this->module_position = 500;

        $this->name = "aproximite";
        $this->url_last_version = "https://www.dolibiz.com/wp-content/uploads/lastversion/last_version-aproximite.txt";
        // Module description, used if translation string 'ModuleXXXDesc' not found (where XXX is value of numeric property 'numero' of module)
        $this->description = "Module432441Desc";
        $this->descriptionlong = "Visualisation sur une map de tiers autour d'un tiers";
        $this->editor_name = 'Inovea Conseil';
        $this->version = '1.5.3';

        $this->const_name = 'MAIN_MODULE_'.strtoupper($this->name);

        $this->picto='inoveaconseil@aproximite';

        $this->module_parts = array(
            'css' => array('/aproximite/css/aproximite.css'),
            'js' => array('/aproximite/js/aproximite.js.php'),
            'hooks' => array(
                'globalcard',
                'thirdpartycard',
                'projectcard',
                'infothirdparty',
                'propalcard',
                'invoicecard',
                'ordercard',
            ),
            'triggers' => 1,
        );

        $this->dirs = array();

        // Config pages. Put here list of php page, stored into dolitest/admin directory, to use to setup module.
        $this->config_page_url = array("google_gmaps.php@aproximite");

        // Dependencies
        $this->hidden = false;
        $this->depends = array();
        $this->requiredby = array();
        $this->conflictwith = array();
        $this->phpmin = array(5,0);
        $this->need_dolibarr_version = array(3,0);
        $this->langfiles = array("aproximite@aproximite");

        $this->const = array();
        $country = explode(":", $conf->global->MAIN_INFO_SOCIETE_COUNTRY);
        if ($country[0] == $conf->entity && $country[2] == "France")
            $this->editor_url = "<a target='_blank' href='https://www.inovea-conseil.com/'>www.inovea-conseil.com</a> (<a target='_blank' href='https://www.dolibiz.com/wp-content/uploads/attestation/attestation-" . strtolower($this->name) . "-" . $this->version . ".pdf'>Attestation NF525</a>)";
        else
            $this->editor_url = 'https://www.inovea-conseil.com';

        $this->tabs = array();

        if (! isset($conf->aproximite) || ! isset($conf->aproximite->enabled)) {
            $conf->aproximite=new stdClass();
            $conf->aproximite->enabled=0;
        }

        // Dictionaries
        $this->dictionaries=array();
        $this->boxes = array();

        // Cronjobs
        $this->cronjobs = array();

        // Permissions
        $this->rights = array();		// Permission array used by this module

        $this->menu = array();
        $r=0;
        $r=1;
    }

    /**
     * Init function
     *
     * @param      string	$options    Options when enabling module ('', 'noboxes')
     * @return     int             	1 if OK, 0 if KO
     */
    public function init($options='')
    {

        global $conf, $db;
        $sql = array();
        //$this->_load_tables('/aproximite/sql/');

        //VERIF EXTRAFIELD COMMANDE
        $extrafields = new ExtraFields($this->db);
        $extrafields_societe = $extrafields->fetch_name_optionals_label('societe');
        $pos = count($extrafields_societe);

        // N° à prévenir
        if (!isset($extrafields_societe['lat_prox'])) {
            $extrafields->addExtraField('lat_prox', 'Latitude', 'float', $pos++, null, 'societe', 0, 0, '', 0, true, '', 0, 0);
        }

        // Statut
        if (!isset($extrafields_societe['long_prox'])) {
            $extrafields->addExtraField('long_prox', 'Longitude', 'float', $pos++, null, 'societe', 0, 0, '', 0, true, '', 0, 0);
        }

        //écriture du fichier de config js
        $file = __DIR__.'/../../js/conf_aproximite.js';
        $content = "var confUrlAproximite = '" .dol_buildpath('/aproximite/aproximite.php?rowid=',1). "';" . PHP_EOL;
        if((int)DOL_VERSION>7){
            $content .= 'var confImgAproximite = "<i class=\'fa fa-eye\' style=\'color: #444;\'></i>";' . PHP_EOL;
        }else{
            $content .= 'var confImgAproximite = "<img class=\'valigntextbottom\' src=\''.DOL_URL_ROOT.'/theme/common/octicons/lib/svg/eye.svg\'>";' . PHP_EOL;
        }

        file_put_contents($file, $content);

        dolibarr_set_const($db, "CHECKLASTVERSION_EXTERNALMODULE", '1', 'int', 0, '', $conf->entity);

        return $this->_init($sql, $options);
    }

    /**
     * Function called when module is disabled.
     * Remove from database constants, boxes and permissions from Dolibarr database.
     * Data directories are not deleted
     *
     * @param      string	$options    Options when enabling module ('', 'noboxes')
     * @return     int             	1 if OK, 0 if KO
     */
    public function remove($options = '')
    {
        $sql = array();

        return $this->_remove($sql, $options);
    }

}

