<?php

/**
 *      \file       /aproximite/core/triggers/interface_99_modAproximite_AproximiteWorkflow.class.php
 *      \ingroup    ouvrage
 *      \brief      Trigger file for create ouvrage data
 */


/**
 *      \class      InterfaceMilestoneWorkflow
 *      \brief      Classe des fonctions triggers des actions personnalisees
 */

class InterfaceAproximiteWorkflow
{
    private $db;

    /**
     *   Constructor
     *
     *   @param      DoliDB		$db		Database handler
     */
    public function __construct($db)
    {
        $this->db = $db;

        $this->name = preg_replace('/^Interface/i','',get_class($this));
        $this->family = "ouvrage";
        $this->description = "Triggers of this module allows to update latitude / longitude data";
        $this->version = '1.0.0';            // 'development', 'experimental', 'dolibarr' or version
        $this->picto = 'aproximite@aproximite';
    }


    /**
     * Trigger name
     *
     * 	@return		string	Name of trigger file
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Trigger description
     *
     * 	@return		string	Description of trigger file
     */
    public function getDesc()
    {
        return $this->description;
    }

    /**
     * Trigger version
     *
     * 	@return		string	Version of trigger file
     */
    public function getVersion()
    {
        global $langs;
        $langs->load("admin");

        if ($this->version == 'development') return $langs->trans("Development");
        elseif ($this->version == 'experimental') return $langs->trans("Experimental");
        elseif ($this->version == 'dolibarr') return DOL_VERSION;
        elseif ($this->version) return $this->version;
        else return $langs->trans("Unknown");
    }

     /**
     * Function called when a Dolibarrr business event is done.
     * All functions "run_trigger" are triggered if file
     * is inside directory core/triggers
     *
     * 	@param		string		$action		Event action code
     * 	@param		Object		$object		Object
     * 	@param		User		$user		Object user
     * 	@param		Translate	$langs		Object langs
     * 	@param		conf		$conf		Object conf
     * 	@return		int						<0 if KO, 0 if no triggered ran, >0 if OK
     */
    public function run_trigger($action, $object, $user, $langs, $conf)
    {
                        //commande
        $langs->load("aproximite@aproximite");
        // Mettre ici le code a executer en reaction de l'action
        // Les donnees de l'action sont stockees dans $object
        
        $actions = array('COMPANY_MODIFY','COMPANY_CREATE');
        
        // Update Line
        if (in_array($action, $actions)) {
            dol_syslog("Trigger '".$this->name."' for action '$action' launched by ". __FILE__ .". id=".$object->rowid);
            
            require_once DOL_DOCUMENT_ROOT.'/core/class/extrafields.class.php';
            require_once DOL_DOCUMENT_ROOT . '/societe/class/societe.class.php';
            include __DIR__ . '/../../include/gmaps_all.php';
            
            $extrafields = new ExtraFields($this->db);
            $extralabels=$extrafields->fetch_name_optionals_label('societe', true);
            
            $object->fetch_optionals($object->rowid,$extralabels);
            
            if (empty($object->country_code)) $object->country_code=$mysoc->country_code;
            
            $addresstosearch=dol_format_address($object,1," ");
            
            if (strlen(trim($addresstosearch)) > 2) {
                $geo = geocoding($addresstosearch);
                if (is_array($geo)) {
                    $sql = "INSERT INTO ".MAIN_DB_PREFIX."societe_extrafields (fk_object, lat_prox, long_prox) VALUES (".$object->id.", ".$geo['lat'].", ".$geo['lng'].") ON DUPLICATE KEY UPDATE lat_prox=".$geo['lat'].", long_prox=".$geo['lng'];
                    $resql=$this->db->query($sql);

                    $object->array_options['options_lat_prox'] = $geo['lat'];
                    $object->array_options['options_long_prox'] = $geo['lng'];

                } else {
                    setEventMessages($langs->trans("ERRORGEOCODING") . ' : ' . $geo, null, 'errors');
                }
            }
            return 1;
        }
        
	return 0;
    }

}
?>
