<?php
/* Copyright (C) 2005      Rodolphe Quiedeville <rodolphe@quiedeville.org>
 * Copyright (C) 2005-2012 Laurent Destailleur	<eldy@users.sourceforge.net>
 * Copyright (C) 2005-2012 Regis Houssin		<regis.houssin@capnetworks.com>
 * Copyright (C) 2014-2015 Marcos García        <marcosgdf@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * or see http://www.gnu.org/
 */

/**
 *	\file       htdocs/core/modules/expedition/doc/pdf_rouget.modules.php
 *	\ingroup    expedition
 *	\brief      Fichier de la classe permettant de generer les bordereaux envoi au modele Rouget
 */

require_once DOL_DOCUMENT_ROOT.'/core/modules/expedition/modules_expedition.php';
require_once DOL_DOCUMENT_ROOT.'/core/lib/company.lib.php';
require_once DOL_DOCUMENT_ROOT.'/core/lib/pdf.lib.php';


/**
 *	Classe permettant de generer les borderaux envoi au modele Rouget
 */
class pdf_ETI9_colis extends ModelePdfExpedition
{
	var $emetteur;	// Objet societe qui emet


	/**
	 *	Constructor
	 *
	 *	@param	DoliDB	$db		Database handler
	 */
	function __construct($db=0)
	{
		global $conf,$langs,$mysoc;

		$this->db = $db;
		$this->name = "etiquette ETI9 colis";
		$this->description = "Modèle d'etiquette de transport";

		$this->type = 'pdf';
		$formatarray=pdf_getFormat();
		$this->page_largeur = 210;//$formatarray['height'];
		$this->page_hauteur = 74;//$formatarray['width'];
		$this->format = array($this->page_largeur,$this->page_hauteur);
		$this->marge_gauche=0;//isset($conf->global->MAIN_PDF_MARGIN_LEFT)?$conf->global->MAIN_PDF_MARGIN_LEFT:10;
		$this->marge_droite=0;//isset($conf->global->MAIN_PDF_MARGIN_RIGHT)?$conf->global->MAIN_PDF_MARGIN_RIGHT:10;
		$this->marge_haute=0;//isset($conf->global->MAIN_PDF_MARGIN_TOP)?$conf->global->MAIN_PDF_MARGIN_TOP:10;
		$this->marge_basse=0;//isset($conf->global->MAIN_PDF_MARGIN_BOTTOM)?$conf->global->MAIN_PDF_MARGIN_BOTTOM:10;
		$this->hauteur_label = 74;//82;

		$this->option_logo = 1;

		// Get source company
		$this->emetteur=$mysoc;
		if (! $this->emetteur->country_code) $this->emetteur->country_code=substr($langs->defaultlang,-2);    // By default if not defined

		// Define position of columns		
		$this->posxdesc=$this->marge_gauche+1;
		$this->posxfirsttier = 70;
		$this->posxmiddle = 105;
		$this->posxpds = 125;
		$this->posxsecondtier = 130;
		$this->posxindice = 145;
		$this->posxqty = 158;
		
		$this->posxweightvol=$this->page_largeur - $this->marge_droite - 78;
		$this->posxqtyordered=$this->page_largeur - $this->marge_droite - 56;
		$this->posxqtytoship=$this->page_largeur - $this->marge_droite - 28;
		$this->posxpuht=$this->page_largeur - $this->marge_droite;

		if (!empty($conf->global->MAIN_PDF_SHIPPING_DISPLAY_AMOUNT_HT)) {

			$this->posxweightvol=$this->page_largeur - $this->marge_droite - 118;
			$this->posxqtyordered=$this->page_largeur - $this->marge_droite - 96;
			$this->posxqtytoship=$this->page_largeur - $this->marge_droite - 68;
			$this->posxpuht=$this->page_largeur - $this->marge_droite - 40;
			$this->posxtotalht=$this->page_largeur - $this->marge_droite - 20;
		}

		$this->posxpicture=$this->posxweightvol - (empty($conf->global->MAIN_DOCUMENTS_WITH_PICTURE_WIDTH)?20:$conf->global->MAIN_DOCUMENTS_WITH_PICTURE_WIDTH);	// width of images

		if ($this->page_largeur < 210) // To work with US executive format
		{
		    $this->posxweightvol-=20;
		    $this->posxpicture-=20;
		    $this->posxqtyordered-=20;
		    $this->posxqtytoship-=20;
		}

		if (! empty($conf->global->SHIPPING_PDF_HIDE_ORDERED))
		{
		    $this->posxweightvol += ($this->posxqtytoship - $this->posxqtyordered);
		    $this->posxpicture += ($this->posxqtytoship - $this->posxqtyordered);
		    $this->posxqtyordered = $this->posxqtytoship;
		}
	}

	/**
	 *	Function to build pdf onto disk
	 *
	 *	@param		Object		$object			Object expedition to generate (or id if old method)
	 *	@param		Translate	$outputlangs		Lang output object
     *  @param		string		$srctemplatepath	Full path of source filename for generator using a template file
     *  @param		int			$hidedetails		Do not show line details
     *  @param		int			$hidedesc			Do not show desc
     *  @param		int			$hideref			Do not show ref
     *  @return     int         	    			1=OK, 0=KO
	 */
	function write_file($object,$outputlangs,$srctemplatepath='',$hidedetails=0,$hidedesc=0,$hideref=0)
	{
		global $user,$conf,$langs,$hookmanager;

		$object->fetch_thirdparty();

		if (! is_object($outputlangs)) $outputlangs=$langs;
		// For backward compatibility with FPDF, force output charset to ISO, because FPDF expect text to be encoded in ISO
		if (! empty($conf->global->MAIN_USE_FPDF)) $outputlangs->charset_output='ISO-8859-1';

		$outputlangs->load("main");
		$outputlangs->load("dict");
		$outputlangs->load("companies");
		$outputlangs->load("bills");
		$outputlangs->load("products");
		$outputlangs->load("propal");
		$outputlangs->load("deliveries");
        $outputlangs->load("sendings");
		$outputlangs->load("productbatch");

		$nblignes = count($object->lines);

		if ($conf->expedition->dir_output)
		{
			// Definition de $dir et $file
			if ($object->specimen)
			{
				$dir = $conf->expedition->dir_output."/sending";
				$file = $dir . "/SPECIMEN.pdf";
			}
			else
			{
			    $sql = "SELECT liv.ref FROM " . MAIN_DB_PREFIX . "element_element as ee";
			    $sql.= " LEFT JOIN " . MAIN_DB_PREFIX . "livraison as liv on ee.fk_target = liv.rowid";
			    $sql.= " WHERE ee.targettype = 'delivery'";
			    $sql.= " AND ee.sourcetype = 'shipping'";
			    $sql.= " AND ee.fk_source = ".$object->id;
			    
			    $res = $this->db->query($sql);
			    
			    $refbl = $object->ref;
			    if($res && $this->db->num_rows($res))
			    {
			        $obj = $this->db->fetch_object($res);
			        $refbl = $obj->ref;
			    }
			    
			    $expref = dol_sanitizeFileName($object->ref);
			    $dir = $conf->expedition->dir_output."/sending/" . $expref;
			    $file = $dir . "/ETI9_colis_" . $refbl . ".pdf";
			    if(file_exists($dir . "/ETI9_colis_" . $expref . ".pdf")) unlink($dir . "/ETI9_colis_" . $expref . ".pdf");
				
				$prod = new Product($this->db);
				
			}

			if (! file_exists($dir))
			{
				if (dol_mkdir($dir) < 0)
				{
					$this->error=$langs->transnoentities("ErrorCanNotCreateDir",$dir);
					return 0;
				}
			}

			if (file_exists($dir))
			{
				// Add pdfgeneration hook
				if (! is_object($hookmanager))
				{
					include_once DOL_DOCUMENT_ROOT.'/core/class/hookmanager.class.php';
					$hookmanager=new HookManager($this->db);
				}
				$hookmanager->initHooks(array('pdfgeneration'));
				$parameters=array('file'=>$file,'object'=>$object,'outputlangs'=>$outputlangs);
				global $action;
				$reshook=$hookmanager->executeHooks('beforePDFCreation',$parameters,$object,$action);    // Note that $action and $object may have been modified by some hooks

				// Set nblignes with the new facture lines content after hook
				$nblignes = count($object->lines);

				$pdf=pdf_getInstance($this->format);
				$default_font_size = pdf_getPDFFontSize($outputlangs);
				$heightforinfotot = 8;	// Height reserved to output the info and total part
		        $heightforfreetext= (isset($conf->global->MAIN_PDF_FREETEXT_HEIGHT)?$conf->global->MAIN_PDF_FREETEXT_HEIGHT:5);	// Height reserved to output the free text on last page
	            $heightforfooter = $this->marge_basse + 8;	// Height reserved to output the footer (value include bottom margin)
                $pdf->SetAutoPageBreak(0,0);

                if (class_exists('TCPDF'))
                {
                    $pdf->setPrintHeader(false);
                    $pdf->setPrintFooter(false);
                }
                $pdf->SetFont(pdf_getPDFFont($outputlangs));
                // Set path to the background PDF File
                if (empty($conf->global->MAIN_DISABLE_FPDI) && ! empty($conf->global->MAIN_ADD_PDF_BACKGROUND))
                {
                    $pagecount = $pdf->setSourceFile($conf->mycompany->dir_output.'/'.$conf->global->MAIN_ADD_PDF_BACKGROUND);
                    $tplidx = $pdf->importPage(1);
                }

				$pdf->Open();
				$pagenb=0;
				$pdf->SetDrawColor(128,128,128);

				if (method_exists($pdf,'AliasNbPages')) $pdf->AliasNbPages();

				$pdf->SetTitle($outputlangs->convToOutputCharset($object->ref));
				$pdf->SetSubject($outputlangs->transnoentities("Shipment"));
				$pdf->SetCreator("Dolibarr ".DOL_VERSION);
				$pdf->SetAuthor($outputlangs->convToOutputCharset($user->getFullName($outputlangs)));
				$pdf->SetKeyWords($outputlangs->convToOutputCharset($object->ref)." ".$outputlangs->transnoentities("Shipment"));
				if (! empty($conf->global->MAIN_DISABLE_PDF_COMPRESSION)) $pdf->SetCompression(false);

				$pdf->SetMargins($this->marge_gauche, $this->marge_haute, $this->marge_droite);   // Left, Top, Right

				// New page
				$pdf->AddPage('L', array(0 => $this->page_largeur, 1 => $this->page_hauteur, 'Rotate' => -90));
				if (! empty($tplidx)) $pdf->useTemplate($tplidx);
				$pagenb++;
				//$this->_print_label($pdf, $object, $outputlangs, $this->marge_haute);
				$pdf->SetFont('','', $default_font_size - 1);
				$pdf->MultiCell(0, 3, '');		// Set interline to 3
				$pdf->SetTextColor(0,0,0);

				$tab_top = 90;
				$tab_top_newpage = (empty($conf->global->MAIN_PDF_DONOTREPEAT_HEAD)?42:10);
				$tab_height = 130;
				$tab_height_newpage = 150;

				$iniY = $curY = $nexY = $this->marge_haute;
				$j = 0;
				
				$arrLabels = array();
				// Loop on each lines
				for ($i = 0; $i < $nblignes; $i++)
				{
				    // on récupère le détail des lots
				    $detail_batch = $object->lines[$i]->detail_batch;
				    $object->lines[$i]->detail_batch = array();
				    
				    if ($object->lines[$i]->fk_product)
				    {
				        $prod->fetch($object->lines[$i]->fk_product);
				        
				        $refclient = $prod->ref;
				        
				        $sqlref = "SELECT ref FROM ".MAIN_DB_PREFIX."custom_product_ref WHERE fk_soc='".$object->thirdparty->id."' AND fk_product='".$prod->id."'";
				        $res = $this->db->query($sqlref);
				        if($res && $this->db->num_rows($res))
				        {
				            $obj = $this->db->fetch_object($res);
				            
				            $refclient = $obj->ref;
				        }
				        
				        // TODO Modifier la requête pour récupérer les données de la table UC
				        // SELECT uc.number, um.qty, eb.batch FROM llx_labels_uc as uc LEFT JOIN llx_labels_um as um ON um.rowid = uc.fk_um LEFT JOIN llx_expeditiondet_batch as eb ON eb.rowid = um.fk_lot WHERE 1 AND um.fk_expedition = 67
				        
				        // Les enregistrements sont créés sur le trigger de création de l'expédition
				        $sql_labels = "SELECT um.rowid, uc.number, um.qty, eb.batch";
				        $sql_labels.= " FROM llx_labels_uc as uc";
				        $sql_labels.= " LEFT JOIN llx_labels_um as um ON um.rowid = uc.fk_um";
				        $sql_labels.= " LEFT JOIN llx_expeditiondet_batch as eb ON eb.rowid = um.fk_lot";
				        $sql_labels.= " WHERE um.fk_expeditiondet = ".$object->lines[$i]->id;
				        $reslabel = $this->db->query($sql_labels);
				        
				        $um_id = '';
				        if ($reslabel && $this->db->num_rows($reslabel))
				        {
				            while ($obj = $this->db->fetch_object($reslabel))
				            {
				                if ($obj->rowid !== $um_id)
				                {
				                    $um_id = $obj->rowid;
				                    $qte_restante = $obj->qty;
				                }
				                
				                $qty = ($qte_restante > $prod->array_options['options_prod_per_col']) ? $prod->array_options['options_prod_per_col'] : $qte_restante;
				                
				                $tmparr = array();
				                $tmparr['desc'] = dol_trunc(html_entity_decode(pdf_getlinedesc($object,$i,$outputlangs,1,1)), 70);
				                $tmparr['nb'] = 1;
				                $tmparr['pds_net'] = $prod->weight * $qty;
				                switch ($prod->weight_units)
				                {
				                    case -6:
				                        $poids = ceil($tmparr['pds_net'] / 10000)/100;
				                        break;
				                        
				                    case -3:
				                        $poids = ceil($tmparr['pds_net'] / 10)/100;
				                        break;
				                        
				                    case 3:
				                        $poids = $tmparr['pds_net'] * 1000;
				                        break;
				                        
				                    Default:
				                        $poids = $tmparr['pds_net'];
				                }
				                $tmparr['pds_net'] = round($poids, 2);
				                $tmparr['pds_brut'] = $tmparr['pds_net'] + $conf->global->ACOBAL_COL_WEIGHT;
				                $tmparr['qty'] = round($qty);
				                $tmparr['qtyBarcode'] = $this->_getBarCode('Q'.$tmparr['qty']);
				                $tmparr['ref'] = $prod->ref;
				                $tmparr['refBarcode'] = $this->_getBarCode('30S'.$prod->ref);
				                $tmparr['refclient'] = $refclient;
				                $tmparr['refclientBarcode'] = $this->_getBarCode('P'.$refclient);
				                $tmparr['lot'] = $obj->batch;
				                $tmparr['date'] = "D".date("ymd");
				                $tmparr['num'] = $obj->number;
				                $tmparr['numBarcode'] = $this->_getBarCode('M'.$tmparr['num']);
				                $tmparr['code_va'] = $object->thirdparty->array_options['options_code_va'];
				                $tmparr['code_vaBarcode'] = (!empty($tmparr['code_va'])) ? $this->_getBarCode('V'.$tmparr['code_va']) : '';
				                
				                $qte_restante-=$qty;
				                
				                $arrLabels[] = $tmparr;
				            }
				            
				        }
				        
				    }
				}
				
// 				var_dump(count($arrLabels));
				$i = 0;
				$num = count($arrLabels);
				foreach ($arrLabels as $label)
				{
				    $curY = $this->marge_haute;
				    $start_label = $curY;
				    $nexY = $start_label + $this->hauteur_label + 5;
				    
				    $pdf->SetFont('','', $default_font_size - 1);   // Into loop to work with multipage
				    $pdf->SetTextColor(0,0,0);
				    
				    $pdf->setTopMargin($tab_top_newpage);
				    $pdf->setPageOrientation('L', 0, $heightforfooter+$heightforfreetext+$heightforinfotot);	// The only function to edit the bottom margin of current page to set it.
				    
				    $this->_print_label($pdf, $object, $outputlangs, $start_label);
				    
// 				    $pdf->SetXY($this->posxdesc, $start_label + 5);
// 				    $pdf->MultiCell(20, 3, $j,0,'R');
				    
				    // ref client
				    $pdf->SetFont('','B', $default_font_size + 8);
				    $pdf->SetXY($this->posxdesc + 30, $start_label + 21);
				    $text = $label['refclient'];
				    $pdf->MultiCell($this->posxqty - $this->posxmiddle, 2, $outputlangs->convToOutputCharset($text), 0, 'L');
				    
				    // codebarre refclient
				    $text = $label['refclientBarcode'];
				    $imglinesize=pdf_getSizeForImage($text);
				    if (isset($imglinesize['width']) && isset($imglinesize['height']))
				    {
				        $pdf->Image($text, $this->posxdesc + 30, $start_label +30, $imglinesize['width']*3, 7, '', '', '', 2, 300);	// Use 300 dpi
				    }
				    
				    $pdf->SetFont('','B', $default_font_size - 1);
				    
				    // Poids net
				    $pdf->SetXY($this->posxmiddle,$start_label +24);
				    $text= $label['pds_net']. " KG";
				    $pdf->MultiCell($this->posxqty - $this->posxmiddle-2, 2, $outputlangs->convToOutputCharset($text), 0, 'R');
				    
				    // Poids brut
				    $pdf->SetXY($this->posxmiddle,$start_label +31);
				    $text = $label['pds_brut']. " KG";
				    $pdf->MultiCell($this->posxqty - $this->posxmiddle-2, 2, $outputlangs->convToOutputCharset($text), 0, 'R');
				    
				    // Date
				    $pdf->SetXY($this->posxmiddle,$start_label +37.5);
				    $text = $label['date'];
				    $pdf->MultiCell($this->posxqty - $this->posxmiddle-2, 2, $outputlangs->convToOutputCharset($text), 0, 'R');
				    
				    // Qty
				    $pdf->SetFont('','B', $default_font_size + 3);
				    $pdf->SetXY($this->posxqty,$start_label +29);
				    $text= $label['qty'];
				    $pdf->MultiCell($this->posxqty - $this->posxmiddle-2, 2, $outputlangs->convToOutputCharset($text), 0, 'L');
				    
				    $pdf->SetFont('','B', $default_font_size - 1);
				    // Qty Barcode
				    $text = $label['qtyBarcode'];
				    $imglinesize=pdf_getSizeForImage($text);
				    if (isset($imglinesize['width']) && isset($imglinesize['height']))
				    {
				        $pdf->Image($text, $this->posxqty + 2, $start_label +35.5, $imglinesize['width']*2, 5, '', '', '', 2, 300);	// Use 300 dpi
				    }
				    
				    // Num etiquette
				    $pdf->SetFont('','B', $default_font_size + 8);
				    $pdf->SetXY($this->posxdesc + 30, $start_label + 47);
				    $text = $label['num'];
				    $pdf->MultiCell($this->posxqty - $this->posxmiddle, 2, $outputlangs->convToOutputCharset($text), 0, 'L');
				    
				    // codebarre Num etiquette
				    $text = $label['numBarcode'];
				    $imglinesize=pdf_getSizeForImage($text);
				    if (isset($imglinesize['width']) && isset($imglinesize['height']))
				    {
				        $pdf->Image($text, $this->posxdesc + 30, $start_label +58, $imglinesize['width']*3, 7, '', '', '', 2, 300);	// Use 300 dpi
				    }
				    
				    // nb
				    $pdf->SetXY($this->posxmiddle, $start_label + 47);
				    $pdf->SetFont('','B', $default_font_size - 1);
				    $pdf->MultiCell(20, 3, $label['nb'],0,'C');
				    
				    // label
				    $pdf->SetXY($this->posxmiddle +25,$start_label + 47);
				    $pdf->SetFont('','B', $default_font_size - 3);
				    $text = $tmparr['desc'];
				    $pdf->MultiCell($this->posxmiddle +25 - $this->marge_droite, 2, $outputlangs->convToOutputCharset($text), 0, 'L');
				    
				    // ref
				    $pdf->SetXY($this->posxmiddle + 4, $start_label + 54);
				    $pdf->MultiCell($this->posxmiddle - $this->marge_droite - 4, 3, $label['ref'],0,'L');
				    
				    // ref Barcode
				    $text = $label['refBarcode'];
				    $imglinesize=pdf_getSizeForImage($text);
				    if (isset($imglinesize['width']) && isset($imglinesize['height']))
				    {
				        $pdf->Image($text, $this->posxqty + 2, $start_label +52, $imglinesize['width']*2, 5, '', '', '', 2, 300);	// Use 300 dpi
				    }
				    
				    // lot
				    $pdf->SetXY($this->posxmiddle + 4, $start_label + 62);
				    $pdf->MultiCell($this->posxmiddle - $this->marge_droite - 4, 3, $label['lot'],0,'L');
				    
				    // code vendeur
				    $posy=$top + 69;
				    $pdf->SetXY($this->posxmiddle + 4,$posy);
				    $text= $label['code_va'];
				    $pdf->MultiCell($this->posxmiddle - $this->marge_droite - 4, 2, $outputlangs->convToOutputCharset($text), 0, 'L');
				    
				    // code vendeur Barcode
				    $text = $label['code_vaBarcode'];
				    $imglinesize=pdf_getSizeForImage($text);
				    if (isset($imglinesize['width']) && isset($imglinesize['height']))
				    {
				        $pdf->Image($text, $this->posxqty + 2, $start_label +67.5, $imglinesize['width']*2, 5, '', '', '', 2, 300);	// Use 300 dpi
				    }
				    
				    $i++;
				    
				    if($i < $num)
				    {
				        $pdf->AddPage('L', array(0 => $this->page_largeur, 1 => $this->page_hauteur, 'Rotate' => -90));
				        if (! empty($tplidx)) $pdf->useTemplate($tplidx);
				        $pagenb++;
				        $pdf->setPage($pagenb);
				        $pdf->setPageOrientation('L', 0, 0);	// The only function to edit the bottom margin of current page to set it.
				        $start_label = $curY = $this->marge_haute;
				    }
				}

				if (method_exists($pdf,'AliasNbPages')) $pdf->AliasNbPages();

				$pdf->Close();

				$pdf->Output($file,'F');

				// Add pdfgeneration hook
				$hookmanager->initHooks(array('pdfgeneration'));
				$parameters=array('file'=>$file,'object'=>$object,'outputlangs'=>$outputlangs);
				global $action;
				$reshook=$hookmanager->executeHooks('afterPDFCreation',$parameters,$this,$action);    // Note that $action and $object may have been modified by some hooks

				if (! empty($conf->global->MAIN_UMASK))
				@chmod($file, octdec($conf->global->MAIN_UMASK));

				$this->result = array('fullpath'=>$file);

				return 1;	// No error
			}
			else
			{
				$this->error=$langs->transnoentities("ErrorCanNotCreateDir",$dir);
				return 0;
			}
		}
		else
		{
			$this->error=$langs->transnoentities("ErrorConstantNotDefined","EXP_OUTPUTDIR");
			return 0;
		}
	}
	
	/**
	 *   Show table for labels
	 *
	 *   @param		PDF			$pdf     		Object PDF
	 *   @param		string		$tab_top		Top position of table
	 *   @param		string		$tab_height		Height of table (rectangle)
	 *   @param		int			$nexY			Y
	 *   @param		Translate	$outputlangs	Langs object
	 *   @param		int			$hidetop		Hide top bar of array
	 *   @param		int			$hidebottom		Hide bottom bar of array
	 *   @return	void
	 */
	function _tableau_label(&$pdf, $tab_top, $tab_height)
	{
	    $this->printRect($pdf,$this->marge_gauche, $tab_top, $this->page_largeur-$this->marge_gauche-$this->marge_droite, $tab_height, 0,0);
	    
	    // lignes horizontales
	    $pdf->line($this->marge_gauche, $tab_top+7, $this->page_largeur-$this->marge_droite, $tab_top+7);
	    $pdf->line($this->marge_gauche, $tab_top+20, $this->page_largeur-$this->marge_droite, $tab_top+20);
	    $pdf->line($this->posxpds, $tab_top+28, $this->posxqty, $tab_top+28);
	    $pdf->line($this->posxpds, $tab_top+35.5, $this->posxqty, $tab_top+35.5);
	    $pdf->line($this->marge_gauche, $tab_top+43, $this->page_largeur-$this->marge_droite, $tab_top+43);
	    $pdf->line($this->posxmiddle, $tab_top+51, $this->page_largeur-$this->marge_droite, $tab_top+51);
	    $pdf->line($this->posxmiddle, $tab_top+58.5, $this->page_largeur-$this->marge_droite, $tab_top+58.5);
	    $pdf->line($this->posxmiddle, $tab_top+66, $this->page_largeur-$this->marge_droite, $tab_top+66);
	    
	    // verticales
	    $pdf->line($this->posxfirsttier, $tab_top, $this->posxfirsttier, $tab_top+7);
	    $pdf->line($this->posxsecondtier, $tab_top, $this->posxsecondtier, $tab_top+7);
	    $pdf->line($this->posxmiddle, $tab_top+7, $this->posxmiddle, $tab_top + 20);
	    $pdf->line($this->posxpds, $tab_top+20, $this->posxpds, $tab_top + 51);
	    $pdf->line($this->posxmiddle, $tab_top+43, $this->posxmiddle, $tab_top + $tab_height);
	    $pdf->line($this->posxqty, $tab_top+20, $this->posxqty, $tab_top+43);
	    $pdf->line($this->posxindice, $tab_top+58.5, $this->posxindice, $tab_top+66);
	}

	/**
	 *  Print Label structure.
	 *
	 *  @param	PDF			$pdf     		Object PDF
	 *  @param  Object		$object     	Object to show
	 *  @param  Translate	$outputlangs	Object lang for output
	 *  @param  int         $top            top position to start to print label
	 *  @return	void
	 */
	function _print_label(&$pdf, $object, $outputlangs, $top)
	{
	    global $conf,$langs,$mysoc;
	    
	    $langs->load("orders");
	    
	    $default_font_size = pdf_getPDFFontSize($outputlangs);
	    
	    // Show Draft Watermark
	    if($object->statut==0 && (! empty($conf->global->SHIPPING_DRAFT_WATERMARK)) )
	    {
	        pdf_watermark($pdf,$outputlangs,$this->page_hauteur,$this->page_largeur,'mm',$conf->global->SHIPPING_DRAFT_WATERMARK);
	    }
	    
	    $this->_tableau_label($pdf, $top, $this->hauteur_label);
	    //Prepare la suite
	    //$pdf->SetTextColor(0,0,60);
	    $pdf->SetFont('','B', $default_font_size -2);
	    
	    $w = ($this->page_largeur-$this->posxsecondtier);
	    
	    // destinataire
	    $posy=$top + 1;
	    $posx=$this->posxdesc;
	    $pdf->SetXY($posx,$posy);
	    $text= $outputlangs->transnoentities("Recipient");
	    $pdf->MultiCell(70-2, 2, $outputlangs->convToOutputCharset($text), 0, 'L');
	    
	    $posy += 1;
	    $pdf->SetXY($posx,$posy);
	    $pdf->SetFont('','B', $default_font_size);
	    $text= $this->_getContactLiv($object);
	    $pdf->MultiCell(70-2, 2, $outputlangs->convToOutputCharset($text), 0, 'R');
	    
	    // point de chargement
	    $posy=$top;
	    $posx = $this->posxfirsttier;
	    $pdf->SetXY($posx,$posy+5);
	    $pdf->SetFont('','B', $default_font_size -2);
	    $text= "POINT DE<br>DECHARGEMENT";
	    $pdf->writeHTMLCell($w-2, 10, $posx, $posy, $text);
	    
	    $posy=$top + 1;
	    $posx = $this->posxfirsttier;
	    $pdf->SetXY($posx,$posy);
	    $pdf->SetFont('','B', $default_font_size);
	    $text= $this->_getContactLiv2($object, true);
	    $pdf->MultiCell(60-2, 2, $outputlangs->convToOutputCharset($text), 0, 'R');
	    
	    // expediteur
	    $posy=$top + 1;
	    $pdf->SetFont('','B', $default_font_size -2);
	    $pdf->SetXY($this->posxsecondtier,$posy);
	    $text= $outputlangs->transnoentities("Sender");
	    $pdf->MultiCell($w, 2, $outputlangs->convToOutputCharset($text), 0, 'L');
	    
	    $posy += 1;
	    $pdf->SetXY($this->posxsecondtier,$posy);
	    $pdf->SetFont('','B', $default_font_size);
	    $text= "ACOBAL Saint Chamond Cedex";
	    $pdf->MultiCell($w, 2, $outputlangs->convToOutputCharset($text), 0, 'R');
	    
	    // code routage
	    $posy=$top + 8;
	    $posx=$this->posxdesc;
	    $pdf->SetFont('','B', $default_font_size - 2);
	    $text= "CODE<br>ROUTAGE";
	    $pdf->writeHTMLCell($w, 10, $posx, $posy, $text);
	    
	    // Point destination
	    $posx=$this->posxmiddle;
	    $text= "POINT DE<br>DESTINATION";
	    $pdf->writeHTMLCell($w, 10, $posx, $posy, $text);
	    
	    // Produit
	    $posy=$top + 21;
	    $posx=$this->posxdesc;
	    $text= "CODE<br>PRODUIT (P)";
	    $pdf->writeHTMLCell($w, 10, $posx, $posy, $text);
	    
	    $pdf->SetXY($this->posxpds,$posy);
	    $text= "PDS NET";
	    $pdf->MultiCell($w, 2, $outputlangs->convToOutputCharset($text), 0, 'L');
	    
	    $pdf->SetXY($this->posxqty,$posy);
	    $text= "QUANTITE (Q)";
	    $pdf->MultiCell($w, 2, $outputlangs->convToOutputCharset($text), 0, 'L');
	    
	    $posy=$top + 28;
	    $pdf->SetXY($this->posxpds,$posy);
	    $text= "PDS BRUT";
	    $pdf->MultiCell($w, 2, $outputlangs->convToOutputCharset($text), 0, 'L');
	    
	    $posy=$top + 35.5;
	    $pdf->SetXY($this->posxpds,$posy);
	    $text= "DATE";
	    $pdf->MultiCell($w, 2, $outputlangs->convToOutputCharset($text), 0, 'L');
	    
	    // Num etiquette
	    $posy=$top + 44;
	    $pdf->SetXY($this->posxdesc,$posy);
	    $text= "N° ETIQUETTE (M)";
	    $pdf->MultiCell($w, 2, $outputlangs->convToOutputCharset($text), 0, 'L');
	    
	    // NB
	    $pdf->SetXY($this->posxmiddle,$posy);
	    $text= "NB.";
	    $pdf->MultiCell($w, 2, $outputlangs->convToOutputCharset($text), 0, 'L');
	    
	    // description produit
	    $pdf->SetXY($this->posxpds,$posy);
	    $text= "DESCRIPTION";
	    $pdf->MultiCell($w, 2, $outputlangs->convToOutputCharset($text), 0, 'L');

	    // Ref fourn
	    $posy=$top + 51;
	    $pdf->SetXY($this->posxmiddle,$posy);
	    $text= "REFFOURNISSEUR (30S)";
	    $pdf->MultiCell($w, 2, $outputlangs->convToOutputCharset($text), 0, 'L');

	    // Lot
	    $posy=$top + 58.5;
	    $pdf->SetXY($this->posxmiddle,$posy);
	    $text= "N° LOT";
	    $pdf->MultiCell($w, 2, $outputlangs->convToOutputCharset($text), 0, 'L');
	    
	    // INDICE
	    $pdf->SetXY($this->posxindice,$posy);
	    $text= "INDICE MODIF";
	    $pdf->MultiCell($w, 2, $outputlangs->convToOutputCharset($text), 0, 'L');
	    
	    // Code vendeur
	    $posy=$top + 66;
	    $pdf->SetXY($this->posxmiddle,$posy);
	    $text= "CODE VENDEUR (V)";
	    $pdf->MultiCell($w, 2, $outputlangs->convToOutputCharset($text), 0, 'L');
	}

	/**
	 * 
	 * @param  int     $commande_id    id of the origin command
	 * @return string                  return name of the thirdparty or name of the delivery contact
	 * 
	 */
	function _getContactLiv($object, $onlycontact = false)
	{
	    $arrayidcontact=$object->commande->getIdContact('external','SHIPPING');
	    if (count($arrayidcontact) > 0)
	    {
	        $result=$object->fetch_contact($arrayidcontact[0]);
	        return $object->contact->lastname;
	    }
	    elseif (!$onlycontact) return $object->thirdparty->name;
	    else return '';
	    
	}

/**
         * 
         * @param  int     $commande_id    id of the origin command
         * @return string                  return name of the thirdparty or name of the delivery contact
         * 
         */
        function _getContactLiv2($object, $onlycontact = false)
        {
            $arrayidcontact=$object->commande->getIdContact('external','SHIPPING');
            if (count($arrayidcontact) > 0)
            {
                $result=$object->fetch_contact($arrayidcontact[0]);
//              echo '<pre>';var_dump($object->contact);exit;
//              return $object->contact->getFullName($arrayidcontact[0]);
                return $object->contact->firstname;
            }
            elseif (!$onlycontact) return $object->thirdparty->name;
            else return '';

        }	

	/**
	 * 
	 * @param  string  $code           String to convert
	 * @param  string  $encoding       type of encoding
	 * @param  string  $generator      barcode generator
	 * 
	 * @return string                  path of the generated image
	 */
	function _getBarCode($code='', $encoding='C128', $generator='tcpdfbarcode') {
	    global $conf,$db;
	    $modulepart = 'barcode';
	    
	    $readable="Y";
	    
	    if(empty($code)) return '';
	    
	    $dirbarcode=array_merge(array("/core/modules/barcode/doc/"),$conf->modules_parts['barcode']);
	    
	    $result='';
	    
	    foreach($dirbarcode as $reldir)
	    {
	        $dir=dol_buildpath($reldir,0);
	        $newdir=dol_osencode($dir);
	        
	        // Check if directory exists (we do not use dol_is_dir to avoid loading files.lib.php)
	        if (! is_dir($newdir)) continue;
	        
	        $result=@include_once $newdir.$generator.'.modules.php';
	        if ($result) break;
	    }
	    
	    // Load barcode class
	    $classname = "mod".ucfirst($generator);
	    $module = new $classname($db);
	    if ($module->encodingIsSupported($encoding))
	    {
	        if($module->writeBarCode($code,$encoding,$readable)>0)
	        {
	            return $conf->barcode->dir_temp.'/barcode_'.$code.'_'.$encoding.'.png';
	        }
	    }
	    
	    return '';
	}

}

