<?php
/* Copyright (C) 2004-2014 Laurent Destailleur   <eldy@users.sourceforge.net>
 * Copyright (C) 2005-2014 Regis Houssin         <regis.houssin@capnetworks.com>
 * Copyright (C) 2007      Franky Van Liedekerke <franky.van.liedekerke@telenet.be>
 * Copyright (C) 2008      Chiptronik
 * Copyright (C) 2011-2012 Philippe Grand        <philippe.grand@atoo-net.com>
 * Copyright (C) 2015      Marcos García         <marcosgdf@gmail.com>

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * or see http://www.gnu.org/
 */

/**
 *	\file       htdocs/core/modules/livraison/doc/pdf_typhon.modules.php
 *	\ingroup    livraison
 *	\brief      File of class to manage receving receipts with template Typhon
 *	\author	    Laurent Destailleur
 */

require_once DOL_DOCUMENT_ROOT.'/core/modules/livraison/modules_livraison.php';
require_once DOL_DOCUMENT_ROOT.'/livraison/class/livraison.class.php';
require_once DOL_DOCUMENT_ROOT.'/core/lib/company.lib.php';
require_once DOL_DOCUMENT_ROOT.'/core/lib/pdf.lib.php';
ini_set('display_errors', 1);
error_reporting(E_ALL & ~E_NOTICE);

/**
 *	Classe permettant de generer les bons de livraison au modele Typho
 */
class pdf_avisexp 
{
    var $db;
    var $name;
    var $description;
    var $type;

    var $phpmin = array(4,3,0); // Minimum version of PHP required by module
    var $version = 'dolibarr';

    var $page_largeur;
    var $page_hauteur;
    var $format;
	var $marge_gauche;
	var	$marge_droite;
	var	$marge_haute;
	var	$marge_basse;

	var $emetteur;	// Objet societe qui emet

	/**
	 *	Constructor
	 *
	 *  @param		DoliDB		$db      Database handler
	 */
	function __construct($db)
	{
		global $conf,$langs,$mysoc;

		$langs->load("main");
		$langs->load("bills");
		$langs->load("sendings");
		$langs->load("companies");

		$this->db = $db;
		$this->name = "AVISEXP";
		$this->description = $langs->trans("DocumentModelTyphon") . "AVISEXP";

		// Dimension page pour format A4
		$this->type = 'pdf';
		$formatarray=pdf_getFormat();
		$this->page_largeur = $formatarray['width'];
		$this->page_hauteur = $formatarray['height'];
		$this->format = array($this->page_largeur,$this->page_hauteur);
		$this->marge_gauche=isset($conf->global->MAIN_PDF_MARGIN_LEFT)?$conf->global->MAIN_PDF_MARGIN_LEFT:10;
		$this->marge_droite=isset($conf->global->MAIN_PDF_MARGIN_RIGHT)?$conf->global->MAIN_PDF_MARGIN_RIGHT:10;
		$this->marge_haute =isset($conf->global->MAIN_PDF_MARGIN_TOP)?$conf->global->MAIN_PDF_MARGIN_TOP:10;
		$this->marge_basse =isset($conf->global->MAIN_PDF_MARGIN_BOTTOM)?$conf->global->MAIN_PDF_MARGIN_BOTTOM:10;

		$this->option_logo = 1;                    // Affiche logo FAC_PDF_LOGO
		$this->option_tva = 1;                     // Gere option tva FACTURE_TVAOPTION
		$this->option_codeproduitservice = 1;      // Affiche code produit-service

		$this->franchise=!$mysoc->tva_assuj;

		// Get source company
		$this->emetteur=$mysoc;
		if (empty($this->emetteur->country_code)) $this->emetteur->country_code=substr($langs->defaultlang,-2);    // By default, if was not defined

		// Define position of columns
		$this->posxdesc=$this->marge_gauche+1;
		//$this->posxcomm=112;
		//$this->posxtva=112;
		//$this->posxup=126;
		$this->posxqty=165;
		$this->posxunit=185;
		//$this->posxdiscount=162;
		//$this->postotalht=174;
		if ($this->page_largeur < 210) // To work with US executive format
		{
			//$this->posxcomm-=20;
			//$this->posxtva-=20;
			//$this->posxup-=20;
			$this->posxqty-=20;
			//$this->posxdiscount-=20;
			//$this->postotalht-=20;
		}

		$this->tva=array();
		$this->atleastoneratenotnull=0;
		$this->atleastonediscount=0;
	}

	/**
     *  Function to build pdf onto disk
     *
     *  @param		Object		$object				Object to generate
     *  @param		Translate	$outputlangs		Lang output object
     *  @param		string		$srctemplatepath	Full path of source filename for generator using a template file
     *  @param		int			$hidedetails		Do not show line details
     *  @param		int			$hidedesc			Do not show desc
     *  @param		int			$hideref			Do not show ref
     *  @return     int             			1=OK, 0=KO
	 */
	function write_file($object,$outputlangs,$srctemplatepath='',$hidedetails=0,$hidedesc=0,$hideref=0)
	{
		global $user,$langs,$conf,$mysoc,$hookmanager;
		$version="dolibarr";
		$fichier=fopen ("/var/www/$version/documents/expedition/receipt/$object->ref/$object->ref.txt","w+");
		
		$idmessage=substr($object->ref,2,4);
		$idmessage.=substr($object->ref,7,4);
		$idmessage=$object->ref;
		$datemessage=date("YmdHi", filemtime("/var/www/$version/documents/expedition/receipt/$object->ref/$object->ref.txt"));
		$exp = new Expedition($this->db);
		$object->fetch_thirdparty();
		$exp->fetch($object->origin_id);
		//$exp->fetch_lines($exp->rowid);
		//print_r($exp);
		
		//$object->expedition=$exp;
		
		//print_r($objet->expedition);
		$object->fetchObjectLinked();
		//echo "<pre>";
		//var_dump($exp);
		//$object->expedition->LoadOrders();
		
		//Entete message
		$texte="SER   ACOBAL                                               CEGIDPMI                                                            DESADVD  96AUNA01051\r\n";
		//Identification et date message
		$texte.="T0100 $idmessage                                        $datemessage                \r\n";
		// Invariant puis poids total en Kg
		$nb_um = $nb_uc = $total_weight = 0;
		if (count($object->lines))
		{
		    foreach ($object->lines as $line)
		    {
		        $prod = new Product($this->db);
		        
		        if ($line->fk_product)
		        {
		            $prod->fetch($line->fk_product);
		            
		             
		            
		            $nb_uc += ceil(floatval($line->qty_shipped) / floatval($prod->array_options['options_prod_per_col']));
		            
		            switch ($prod->weight_units)
		            {
		                case -6:
		                    $poids = $line->qty_shipped * $prod->weight / 1000000;
		                    break;
		                    
		                case -3:
		                    $poids = $line->qty_shipped * $prod->weight / 1000;
		                    break;
		                    
		                case 3:
		                    $poids = $line->qty_shipped * $prod->weight * 1000;
		                    break;
		                    
		                Default:
		                    $poids = $prod->weight;
		            }
		            
		            $total_weight += $poids;
		            
		        }
		    }
			$nb_um += ceil($nb_uc / floatval($prod->array_options['options_prod_per_pal']));
		}
		$poids_edi=$total_weight*1000;
		
		$poids_edi=str_pad($poids_edi,13,"0",STR_PAD_LEFT);
		$poids_edi=$poids_edi."AADKGM";
		$poids_edi=str_pad($poids_edi,79);
		
		$date_edi=date(YmdHi,$object->date_delivery);
		$texte.="D0300                  351      67                                            +".$poids_edi.$date_edi."00\r\n";
		$commande = new Commande($this->db);
		
				if ($exp->origin == 'commande')
				{
					$commande->fetch($exp->origin_id);
				}
				$object->commande=$commande;	// We set order of shipment onto delivery.
				$object->commande->loadExpeditions();
				
		$usecontact=false;
		
			$arrayidcontact=$object->commande->getIdContact('external','SHIPPING');
			
			if (count($arrayidcontact) > 0)
			{
				
				$usecontact=true;
				$result=$object->fetch_contact($arrayidcontact[0]);
			}
		
		
		
		$nom_livraison_edi=$object->thirdparty->name;
		$adresse_edi=$object->contact->address;
		$cp_edi=$object->contact->zip;
		$ville_edi=$object->contact->town;
		$pays_edi=$object->thirdparty->country_code;
		$clef_edi=$object->thirdparty->array_options['options_clefedi'];
		$numvendeur=$object->thirdparty->array_options['options_code_va'];
		//clef edi et adresse de livraison
		$clef_edi=str_pad($clef_edi,23);
		//Il faut strpader nom livraison
		$nom_livraison_edi=str_pad($nom_livraison_edi,178);
		$tab_adresse_edi=explode("\r\n", $adresse_edi);
		$tab_adresse_edi[0]=str_pad($tab_adresse_edi[0],35);
		$tab_adresse_edi[1]=str_pad($tab_adresse_edi[1],35);
		$ville_edi=str_pad($ville_edi,29);
		$pays_edi=str_pad("FRANCE",52);
		//$adresse_edi=str_replace("\n","",$adresse_edi); 
		$texte.="C0500 ".$clef_edi."10 ".$nom_livraison_edi.$tab_adresse_edi[0].$tab_adresse_edi[1].$cp_edi." ".$ville_edi;
		$texte.=$pays_edi."FR\r\n";
		//Porte complete
		$porte_edi=$object->contact->firstname;
		$texte.="H0200 $porte_edi                   \r\n";
		//Expéditeur (on change rien)
		$numvendeur=str_pad($numvendeur,17);
		$texte.="C0600 093140876867900016     92 ACOBAL                             Z.I. du Clos Marquet               CS 10 177                          42403 Saint Chamond Cedex          FRANCE                                              FR \r\n";
		//Vendeur (on change rien)
		$texte.="C1800 0931408768679          10 ACOBAL                                                                                                                                                                            Z.I. du Clos Marquet               CS 10 177                          42403 Saint Chamond Cedex          FRANCE                             ".$numvendeur."FR39408768679                     FR \r\n";
		
		// plaque d'immat	
		$result = $exp->fetch($object->origin_id);
		if (!$exp->tracking_number) $immat_edi=str_pad("9999ZZZ98",16);
		else $immat_edi=str_pad($exp->tracking_number, 16);
		$texte.="D0400 $immat_edi TE \r\n";
		
		//Transporteur
		if (!empty($exp->shipping_method_id)){
				    $sql = "SELECT libelle FROM ".MAIN_DB_PREFIX."c_shipment_mode WHERE rowid = ".$exp->shipping_method_id;
				    $res = $this->db->query($sql);
				    if($res)
				    {
				        $obj = $this->db->fetch_object($res);
				        $transporteur = $obj->libelle;
				    }
				    //$object->note_public = "<b>Transporté par ".$transporteur."</b>";
				} else {
				    $transporteur = "";
				}
		$transporteur=str_pad($transporteur,23);
		$texte.="C0300 ".$transporteur."10 $transporteur                              \r\n";
		
		//parcours des lignes de BL
		$nblines = count($object->lines);
		//Select des Numéro de carte 
		
		
		for ($i = 0 ; $i < $nblines ; $i++) { 
			$qte_edi=$object->lines[$i]->qty_shipped*1000;
			//echo "<pre>";
			//var_dump ($exp->lines[$i]);
			
			$lignecmd=$object->lines[$i]->fk_origin_line;
			$sql="SELECT num_carte from llx_commandedet,llx_commandedet_extrafields WHERE llx_commandedet_extrafields.fk_object=llx_commandedet.rowid and llx_commandedet.rowid=".$object->lines[$i]->fk_origin_line;
			
			$res = $this->db->query($sql);
			if($res) {
				$obj = $this->db->fetch_object($res);
			}
			
			$ligne_edi=$i+1;
			//Qte Totale expédiée ligne i
			$texte.="A0200 00000$ligne_edi                 00+0000000".$qte_edi."PCE\r\n";
			
			//poids d'un colis nb colis palette i et nb piece par colis
			$uc_edi=str_pad($prod->array_options['options_prod_per_col']*1000,13,"0",STR_PAD_LEFT);
			$um_edi=str_pad(ceil($prod->array_options['options_prod_per_pal']),6,"0",STR_PAD_LEFT);
			$poids_colis_edi=round(($prod->weight / 1000*$prod->array_options['options_prod_per_col'])*1000);
			$poids_colis_edi=str_pad($poids_colis_edi,13,"0",STR_PAD_LEFT);
			$texte.="B0200       +".$poids_colis_edi."KGM   +".$poids_colis_edi."KGM   ".$um_edi."+".$uc_edi."PCE\r\n";
			
			//print_r($prod);echo"<BR>";
			
			//code palette
			$texte.="B0300 F0002            92                                          \r\n";
			
			//une carte par ligne de commande, souvent une carte par palette
			$carte_edi=$obj->num_carte;
			$nb_pal_ligne=ceil($object->lines[$i]->qty_shipped)/ceil($prod->array_options['options_prod_per_col'])/ceil($prod->array_options['options_prod_per_pal']);
			$nb_col_ligne=ceil($prod->array_options['options_prod_per_pal']);
			$sql="SELECT rowid,number FROM `llx_labels_um` where fk_expeditiondet=".$exp->lines[$i]->id;
			$res = $this->db->query($sql);
			
			//echo $Tobj_um[1]['number'];
			while ($Tobj_um = $this->db->fetch_array($res)){
			//UM il faut une boucle en comptant le nb de label_um pour une expé 
				//echo "number ".$Tobj_um['number'];
				//echo "<pre>";print_r($Tobj_um);
				$label_um_edi=$Tobj_um['number'];
				$texte.="B0400 $label_um_edi\r\n";
			
				//UC il faut une boucle imbriquée en comptant le nb de label_uc pour un label_um 
				$sql="SELECT number FROM `llx_labels_uc` where fk_um=".$Tobj_um['rowid'];
				$res2 = $this->db->query($sql);
				
			
				while ($Tobj_uc=$this->db->fetch_array($res2)) { 
					
					$label_uc_edi=str_pad($Tobj_uc["number"],19);
			
					$texte.="B0500 $label_uc_edi $carte_edi\r\n";
				}
			}
			
				
			
			//Num lot de l'article
			$sql="SELECT batch FROM `llx_expeditiondet_batch`,llx_expeditiondet WHERE llx_expeditiondet_batch.fk_expeditiondet=llx_expeditiondet.rowid and llx_expeditiondet.fk_origin_line=".$object->lines[$i]->fk_origin_line;
			$res = $this->db->query($sql);
			if($res) {
				$obj= $this->db->fetch_object($res);
			}
		
			$num_lot_edi=$obj->batch;
			$texte.="A0500 $num_lot_edi\r\n";
			
			//Notre reférence article
			$sql= "SELECT ref FROM `llx_custom_product_ref` WHERE fk_product=".$prod->id." and fk_soc=".$object->socid;
			
			$res = $this->db->query($sql);
			if($res) {
				$obj = $this->db->fetch_object($res);
			}
			
			//print_r($prod->id);
			$ref_art_edi=str_pad($object->lines[$i]->ref,114);
			$libelle_art_edi=str_pad($object->lines[$i]->libelle,258);
			$texte.=str_pad("A0400",23).str_pad($obj->ref,53)."00\r\n";
			
			$texte.=str_pad("A0100",55).$ref_art_edi.$libelle_art_edi." FR                      +".$poids_edi."   KGM                                                                                                                                          \r\n";
			$texte.="A0900                                                                         $object->ref_customer         \r\n";

		
		}
		
		
		
		
		fwrite($fichier,$texte);
		fclose($fichier);
		
		if (!copy("/var/www/$version/documents/expedition/receipt/$object->ref/$object->ref.txt","/home/acobal/EDI/OUT/$object->ref.txt")) {
			echo "La copie du fichier $filename dans OUT a échoué...\n";
		}
		chmod ("/home/acobal/EDI/OUT/$object->ref.txt",0666);
		return 1;
	}

	/**
	 *   Show miscellaneous information (payment mode, payment term, ...)
	 *
	 *   @param		PDF			$pdf     		Object PDF
	 *   @param		Object		$object			Object to show
	 *   @param		int			$posy			Y
	 *   @param		Translate	$outputlangs	Langs object
	 *   @return	void
	 */
	

	/**
	 *   Show table for lines
	 *
	 *   @param		PDF			$pdf     		Object PDF
	 *   @param		string		$tab_top		Top position of table
	 *   @param		string		$tab_height		Height of table (rectangle)
	 *   @param		int			$nexY			Y (not used)
	 *   @param		Translate	$outputlangs	Langs object
	 *   @param		int			$hidetop		1=Hide top bar of array and title, 0=Hide nothing, -1=Hide only title
	 *   @param		int			$hidebottom		Hide bottom bar of array
	 *   @return	void
	 */
	
	/**
	 *  Show top header of page.
	 *
	 *  @param	PDF			$pdf     		Object PDF
	 *  @param  Object		$object     	Object to show
	 *  @param  int	    	$showaddress    0=no, 1=yes
	 *  @param  Translate	$outputlangs	Object lang for output
	 *  @return	void
	 */
	

	/**
	 *   	Show footer of page. Need this->emetteur object
     *
	 *   	@param	PDF			$pdf     			PDF
	 * 		@param	Object		$object				Object to show
	 *      @param	Translate	$outputlangs		Object lang for output
	 *      @param	int			$hidefreetext		1=Hide free text
	 *      @return	int								Return height of bottom margin including footer text
	 */
	

}

