<?php
/* <one line to give the program's name and a brief idea of what it does.>
 * Copyright (C) 2015 ATM Consulting <support@atm-consulting.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * 	\file		core/triggers/interface_99_modMyodule_CliAcobaltrigger.class.php
 * 	\ingroup	cliacobal
 * 	\brief		Sample trigger
 * 	\remarks	You can create other triggers by copying this one
 * 				- File name should be either:
 * 					interface_99_modCliacobal_Mytrigger.class.php
 * 					interface_99_all_Mytrigger.class.php
 * 				- The file must stay in core/triggers
 * 				- The class name must be InterfaceMytrigger
 * 				- The constructor method must be named InterfaceMytrigger
 * 				- The name property name must be Mytrigger
 */

/**
 * Trigger class
 */
class InterfaceCliAcobaltrigger
{

    private $db;

    /**
     * Constructor
     *
     * 	@param		DoliDB		$db		Database handler
     */
    public function __construct($db)
    {
        $this->db = $db;

        $this->name = preg_replace('/^Interface/i', '', get_class($this));
        $this->family = "demo";
        $this->description = "Triggers of this module are empty functions."
            . "They have no effect."
            . "They are provided for tutorial purpose only.";
        // 'development', 'experimental', 'dolibarr' or version
        $this->version = 'development';
        $this->picto = 'cliacobal@cliacobal';
    }

    /**
     * Trigger name
     *
     * 	@return		string	Name of trigger file
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Trigger description
     *
     * 	@return		string	Description of trigger file
     */
    public function getDesc()
    {
        return $this->description;
    }

    /**
     * Trigger version
     *
     * 	@return		string	Version of trigger file
     */
    public function getVersion()
    {
        global $langs;
        $langs->load("admin");

        if ($this->version == 'development') {
            return $langs->trans("Development");
        } elseif ($this->version == 'experimental')

                return $langs->trans("Experimental");
        elseif ($this->version == 'dolibarr') return DOL_VERSION;
        elseif ($this->version) return $this->version;
        else {
            return $langs->trans("Unknown");
        }
    }

    /**
     * Function called when a Dolibarrr business event is done.
     * All functions "run_trigger" are triggered if file
     * is inside directory core/triggers
     *
     * 	@param		string		$action		Event action code
     * 	@param		Object		$object		Object
     * 	@param		User		$user		Object user
     * 	@param		Translate	$langs		Object langs
     * 	@param		conf		$conf		Object conf
     * 	@return		int						<0 if KO, 0 if no triggered ran, >0 if OK
     */
    public function run_trigger($action, $object, $user, $langs, $conf)
    {
        global $user;
    	$db = &$object->db;
    	
        if ($action === 'PROPAL_VALIDATE') {

        	
        	$a = $this->getEventPropal($object);
        		
        	if($a === false ) {
        	
        		$nb_day = empty($conf->global->ACOBAL_NB_DAY_EVENT) ? 7 : (int)$conf->global->ACOBAL_NB_DAY_EVENT;
	        	$title = empty($conf->global->ACOBAL_TITLE_EVENT) ? 'définir ACOBAL_TITLE_EVENT' : $conf->global->ACOBAL_TITLE_EVENT;
	        	$content = empty($conf->global->ACOBAL_CONTENT_EVENT) ? 'définir ACOBAL_CONTENT_EVENT' : $conf->global->ACOBAL_CONTENT_EVENT;
	        	
	        	$actioncomm=new ActionComm($db);
	        	$actioncomm->datep = strtotime('+'.$nb_day.'day');
	        	//$a->datef = $t_end;
	        	
	        	// Association au contact suivi proposition de la propal
	        	$TCommercialPropo = $object->liste_contact(-1, 'internal', 1, 'SALESREPFOLL');
	        	if(!empty($TCommercialPropo)) $actioncomm->userownerid = $TCommercialPropo[0];
	        	else $actioncomm->userownerid = $user->id;
	        	$actioncomm->type_code='AC_REL_PROP';
	        	$actioncomm->label = $title ;
	        	
	        	$actioncomm->elementtype=$object->element;
	        	$actioncomm->fk_element = $object->id;
	        	$actioncomm->fk_project = $object->fk_project;
	        	
	        	$actioncomm->progress = 0;
	        	
	        	$actioncomm->socid = !empty($object->socid) ? $object->socid : $object->fk_soc;
	        	
	        	$actioncomm->note = $content;
	        	
	        	
	        	$actioncomm->create($user);
	        	
	        	
	            dol_syslog(
	                "Trigger '" . $this->name . "' for action '$action' launched by " . __FILE__ . ". id=" . $object->id
	            );
	            
        	}
        
        } elseif ($action === 'PROPAL_CLOSE_SIGNED' || $action==='PROPAL_CLOSE_REFUSED') {
        	
        	$a = $this->getEventPropal($object);
        	if($a !== false && $a->percentage<=0) {
        	    if ($a->datec !== $a->datem)
        	    {
        	        $a->percentage = 100;
        	        $a->update($user);
        	    }
        	    else
        	    {
        	        $res = $a->delete();
        	    }
        		
        	}
        	
            dol_syslog(
                "Trigger '" . $this->name . "' for action '$action' launched by " . __FILE__ . ". id=" . $object->id
            );
        } elseif ($action == 'SHIPPING_CLOSED') {
            
            if(! empty($conf->expedition->enabled) && !empty($conf->global->MAIN_SUBMODULE_LIVRAISON) && ! empty($conf->global->ACOBAL_CREATE_DELIVERY_FROM_CLOSED_SHIPPING))
            {
                $sql = "SELECT * FROM `llx_element_element` WHERE sourcetype = 'shipping' AND targettype = 'delivery' AND fk_source = ".$object->id;
                $resql = $this->db->query($sql);
                if ($resql && !$this->db->num_rows($resql))
                {
                    include_once DOL_DOCUMENT_ROOT.'/delivery/class/delivery.class.php';
                    $delivery = new Delivery($this->db);
                    $result=$delivery->create_from_sending($user, $object->id);
                    if ($result < 0)
                    {
                        setEventMessages($delivery->error, $delivery->errors, 'errors');
                    }
                    
                    $delivery->valid($user);
                    
                    $res=$delivery->generateDocument('typhon_acobal', $langs);
                }
            }
            
            $sql = "SELECT rowid FROM " . MAIN_DB_PREFIX . "labels_um WHERE fk_expedition = ".$object->id;
            $resql2 = $this->db->query($sql);
            if ($resql2 && !$this->db->num_rows($resql2))
            {
                $this->_create_labels($object);
            }
            
            $res = $object->generateDocument('ETI1', $langs);
            $res = $object->generateDocument('ETI9', $langs);
            
            $object->date_expedition = dol_now();
//             $object->date_shipping = dol_now();
            $object->update($user);

    	    $sql = "SELECT fk_entrepot FROM ".MAIN_DB_PREFIX."expeditiondet WHERE fk_expedition = ".$object->id." AND fk_entrepot = 4"; // 4 = entrepot XPO
    	    $resql_temp=$this->db->query($sql);
    	    //$restemp=$this->db->fetch_object($resql_temp);

            if(!empty($conf->global->ACOBAL_XPO_EXPORT_FROM_CLOSED_SHIPPING) && $this->db->num_rows($resql_temp) > 0)
            {
                
                global $put, $exp_id, $file;
                $put = "export";
                $exp_id = $object->id;
    
                $filename = 'CDECLI'.date("YmdHis").'.txt';
                $file = DOL_DATA_ROOT.'/cliacobal/export/'.$filename;
                
                dol_include_once('/cliacobal/script/interface.php');
    //             var_dump(is_file(DOL_DATA_ROOT.'/cliacobal/export/CDECLI'.date("YmdHis").'.txt'));
                
                // extrafield caché sur expé pour fichier transmis
                $transmis = $object->array_options['options_ftp_xpo'];
                if (!empty($conf->global->ACOBAL_XPO_FTP_FROM_CLOSED_SHIPPING) && empty($transmis) && is_file($file))
                {
                    $ftp_host = (empty($conf->global->ACOBAL_XPO_FTP_HOST)) ? "business.xpo.com" : $conf->global->ACOBAL_XPO_FTP_HOST;
                    $ftp_port = (empty($conf->global->ACOBAL_XPO_FTP_PORT)) ? 21 : $conf->global->ACOBAL_XPO_FTP_PORT;
                    $ftp_user = (empty($conf->global->ACOBAL_XPO_FTP_USER)) ? "FTP_ACOBAL" : $conf->global->ACOBAL_XPO_FTP_USER;
                    $ftp_pass = (empty($conf->global->ACOBAL_XPO_FTP_PASS)) ? "F7P_4C0B4L" : $conf->global->ACOBAL_XPO_FTP_PASS;
                    $ftp_folder = (empty($conf->global->ACOBAL_XPO_FTP_DIR)) ? "/" : $conf->global->ACOBAL_XPO_FTP_DIR;
                    $timeout = 120;
//                     var_dump($_SERVER['HTTP_HOST']); exit;
                    if (!empty($conf->global->ACOBAL_HOST_SENDING_FTP) && in_array($_SERVER['HTTP_HOST'], explode(';', $conf->global->ACOBAL_HOST_SENDING_FTP)))
                    { // valider qu'on est bien sur le serveur de prod, lui seul doit pouvoir envoyer les fichiers en ftp
//                         var_dump($_SERVER['HTTP_HOST']); exit;
                        $ftpc = ftp_connect($ftp_host, $ftp_port, $timeout );
                        if ($ftpc)
                        {
                            $res_log = ftp_login($ftpc, $ftp_user, $ftp_pass);
                            if($res_log)
                            {
                                $fp = fopen($file, 'r');
                                
                                if (ftp_fput($ftpc, $filename, $fp, FTP_BINARY)) {
                                    setEventMessage("Chargement avec succès du fichier $filename\n", 'mesgs');
                                    $object->array_options['options_ftp_xpo'] = time();
                                    $object->insertExtraFields();
                                } else {
                                    setEventMessage("Il y a eu un problème lors du chargement du fichier $filename\n", 'errors');
                                    $errors = error_get_last();
                                    var_dump($errors);
                                }
                            }
                            else setEventMessage("Erreur de login ftp\n", 'errors');
                        }
                        else setEventMessage("Erreur de connection ftp\n", 'errors');
                        
    //                     var_dump($res_log);
                        ftp_close($ftpc);
                    }
                    else setEventMessage("Erreur : vous êtes sur un serveur ('".$_SERVER['HTTP_HOST']."')qui n'est pas autorisé à faire des envois ftp \n", 'warnings');
//                     exit;
                }
                //exit;
            }
        }
        elseif ($action == "SHIPPING_CREATE")
        {
            
            return $this->_create_labels($object);
            
        }
        elseif ($action == 'SHIPPING_DELETE')
        {
            $sql = "SELECT rowid FROM ".MAIN_DB_PREFIX."labels_um WHERE fk_expedition =".$object->id;
            $res = $this->db->query($sql);
            
            if($res)
            {
                $Tum = array();
                while ($obj = $this->db->fetch_object($res)) $Tum[] = $obj->rowid;
                
                $sql_uc = "DELETE FROM ".MAIN_DB_PREFIX."labels_uc WHERE fk_um IN (".implode(",", $Tum).")";
                $res = $this->db->query($sql_uc);
                
                $sql_um = "DELETE FROM ".MAIN_DB_PREFIX."labels_um WHERE rowid IN (".implode(",", $Tum).")";
                $res = $this->db->query($sql_um);
            }
        }
        elseif ($action == 'LINEORDER_SUPPLIER_CREATE')
        {
            $object->desc = $this->_concatProdDesc($object);
            $object->update();
//             var_dump($object); exit;
        }
        elseif ($action == 'LINEBILL_SUPPLIER_CREATE')
        {
            $object->desc = $this->_concatProdDesc($object);
            $object->update();
            //             var_dump($object); exit;
        }
        
        return 0;
    }
    
    private function getEventPropal(&$object) {
    	
    	$db = &$object->db;
    	
    	$res = $db->query("SELECT id
						FROM ".MAIN_DB_PREFIX."actioncomm WHERE fk_element=".$object->id."
								AND elementtype='".$object->element."' AND code='AC_REL_PROP' "
    			);
    	
    	if($res === false){
    		var_dump($db);
    		exit;
    		
    	}
    	
    	if($db->num_rows($res) == 0) {
    		return false;	
    	}
    	
    	$obj = $db->fetch_object($res);
    	
    	$a=new ActionComm($db);
    	$a->fetch($obj->id);
    	
    	return $a;
    	
    }

    function concatBatchs($BatchsForProd, $lineDetails)
	{
		foreach ($lineDetails as $detail)
		{
			$last_fk_commande = '';
			$found = false;
			foreach ($BatchsForProd as $bp)
			{
				$last_fk_commande = $bp->fk_commande;
				if ($bp->batch == $detail->batch && $bp->sellby == $detail->sellby && $bp->eatby == $detail->eatby && $bp->entrepot_id == $detail->entrepot_id)
				{
					$found = true;
					$bp->dluo_qty = floatval($bp->dluo_qty) + floatval($detail->dluo_qty);
				}
			}
			if (!$found)
			{
				$detail->dluo_qty = floatval($detail->dluo_qty);
				$detail->fk_commande = $last_fk_commande;
				$BatchsForProd[] = $detail;
			}
		}

		return $BatchsForProd;
	}
    
    function _create_labels(&$object)
    {
        $nblignes = count($object->lines);
        $prod = new Product($this->db);

        $TProds = array();

        if ($nblignes)
		{
			foreach ($object->lines as $line)
			{
				dol_include_once('/commande/class/commande.class.php');
				$commandedet = new OrderLine($this->db);
				$commandedet->fetch($line->origin_line_id);

//				var_dump($line->detail_batch);
				if ($commandedet->fk_product)
				{
					if (empty($TProds[$commandedet->fk_product]))
					{
						$TProds[$commandedet->fk_product] = $line->detail_batch;
						foreach ($TProds[$commandedet->fk_product] as &$det)
						{
							$det->dluo_qty = floatval($det->dluo_qty);
							$det->fk_commande = $commandedet->fk_commande;
						}

					}
					else
					{
						$TProds[$commandedet->fk_product] = $this->concatBatchs($TProds[$commandedet->fk_product], $line->detail_batch);
					}
				}
			}
			//var_dump($TProds);

		}
//exit;
        if (!empty($TProds))
		{
			foreach ($TProds as $prod_id => $detail_batch)
			{
				$prod->fetch($prod_id);

				$colis_par_palette = $prod->array_options['options_prod_per_pal'];
				$prod_par_colis = $prod->array_options['options_prod_per_col'];

				if(empty($colis_par_palette)) setEventMessage("Le champ \"Nb colis par palette (UM)\" du produit ".$prod->ref." n'est pas renseigné.", 'errors');
				if(empty($prod_par_colis)) setEventMessage("Le champ \"Nb produits par colis (UC)\" du produit ".$prod->ref." n'est pas renseigné.", 'errors');

				if(empty($colis_par_palette) || empty($prod_par_colis))
				{
					setEventMessage("Impossible de créer l'expédition.", 'errors');
					return -1;
				}

				$prod_par_palette = $colis_par_palette * $prod_par_colis;
				$label_number = '0000001';

				foreach ($detail_batch as $batch)
				{
					$qty_restante = floatval($batch->dluo_qty);

					while ($qty_restante > $prod_par_palette)
					{

						$sql = "SELECT number FROM ".MAIN_DB_PREFIX."labels_um WHERE 1 ORDER BY rowid DESC limit 1";
						$res = $this->db->query($sql);

						if ($res)
						{
							if($this->db->num_rows($res))
							{
								$obj = $this->db->fetch_object($res);
								$num = intval($obj->number) +1;
								$label_number = ''.$num;

								while (strlen($label_number) < 7) $label_number = '0'.$label_number;
							}
						}

						// insertion des données UM
						$sql = "INSERT INTO ".MAIN_DB_PREFIX."labels_um (";
						$sql.= "number";
						$sql.= ", fk_commande";
						$sql.= ", fk_expedition";
						$sql.= ", fk_expeditiondet";
						$sql.= ", fk_lot";
						$sql.= ", qty";
						$sql.= ") VALUES (";
						$sql.= "'".$label_number."'";
						$sql.= ", ".$batch->fk_commande;
						$sql.= ", ".$object->id;
						$sql.= ", ".$batch->fk_expeditiondet;
						$sql.= ", ".$batch->id;
						$sql.= ", ".$prod_par_palette;
						$sql.= ")";

						$res = $this->db->query($sql);
						if ($res)
						{
							// insertion des données UC
							$fk_um = $this->db->last_insert_id(MAIN_DB_PREFIX."labels_um");
							for($j = 1; $j <= $colis_par_palette; $j++)
							{
								if($j < 10) $label_number_uc = $label_number.'0'.$j;
								else $label_number_uc = $label_number.$j;
								$sql = "INSERT INTO ".MAIN_DB_PREFIX."labels_uc (";
								$sql.= "number";
								$sql.= ", fk_um";
								$sql.= ") VALUES (";
								$sql.= "'".$label_number_uc."'";
								$sql.= ", ".$fk_um;
								$sql.= ")";

								$res = $this->db->query($sql);

							}
						}

						$qty_restante -= $prod_par_palette;
					}

					if($qty_restante)
					{
						$sql = "SELECT number FROM ".MAIN_DB_PREFIX."labels_um WHERE 1 ORDER BY rowid DESC limit 1";
						$res = $this->db->query($sql);

						if ($res)
						{
							if($this->db->num_rows($res))
							{
								$obj = $this->db->fetch_object($res);
								$num = intval($obj->number) +1;
								$label_number = ''.$num;

								while (strlen($label_number) < 7) $label_number = '0'.$label_number;
							}
						}

						$sql = "INSERT INTO ".MAIN_DB_PREFIX."labels_um (";
						$sql.= "number";
						$sql.= ", fk_commande";
						$sql.= ", fk_expedition";
						$sql.= ", fk_expeditiondet";
						$sql.= ", fk_lot";
						$sql.= ", qty";
						$sql.= ") VALUES (";
						$sql.= "'".$label_number."'";
						$sql.= ", ".$batch->fk_commande;
						$sql.= ", ".$object->id;
						$sql.= ", ".$batch->fk_expeditiondet;
						$sql.= ", ".$batch->id;
						$sql.= ", ".$qty_restante;
						$sql.= ")";

						$res = $this->db->query($sql);
						if ($res)
						{
							$uc = ceil($qty_restante/$prod_par_colis);

							// insertion des données UC
							$fk_um = $this->db->last_insert_id(MAIN_DB_PREFIX."labels_um");
							for($j = 1; $j <= $uc; $j++)
							{
								if($j < 10) $label_number_uc = $label_number.'0'.$j;
								else $label_number_uc = $label_number.$j;

								$sql = "INSERT INTO ".MAIN_DB_PREFIX."labels_uc (";
								$sql.= "number";
								$sql.= ", fk_um";
								$sql.= ") VALUES (";
								$sql.= "'".$label_number_uc."'";
								$sql.= ", ".$fk_um;
								$sql.= ")";

								$res = $this->db->query($sql);

							}
						}
					}
				}
			}
		}
        
        return 0;
    }
    
    // concatène la description produit à la description fournisseur
    function _concatProdDesc(&$object)
    {
        global $db;
        
        $prodid = $object->fk_product;
        $desc = $object->desc;
        
        $prod = new Product($this->db);
        $out = '';
        
        if (!empty($prodid))
        {
            $prod->fetch($prodid);
            if (!empty($prod->description))
            {
                $pos = strpos($desc, '<br>');
                $out = substr($desc, 0, $pos).$prod->description."<br><br>".substr($desc, $pos);
                // 	            var_dump($pos, $desc, $out); exit;
            }
        }
        
        return (!empty($out)) ? $out : $desc;
        
    }
}
