<?php
/* Copyright (C) 2005-2015 Laurent Destailleur  <eldy@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *       \file       htdocs/exports/index.php
 *       \ingroup    export
 *       \brief      Home page of export wizard
 */

require_once 'config.php';
dol_include_once('/core/lib/files.lib.php');
dol_include_once('/core/class/html.form.class.php');
dol_include_once('/core/class/html.formfile.class.php');

$langs->load("exports");
$url = GETPOST('urlfile');
$action = GETPOST('action');

if ($action == 'delete')
{
    $filename = DOL_DATA_ROOT.'/cliacobal'.$url;
    unlink($filename);
}

/*
 * View
 */

$form=new Form($db);

llxHeader('',$langs->trans("ExportsArea"),'EN:Module_Exports_En|FR:Module_Exports|ES:M&oacute;dulo_Exportaciones');

print load_fiche_titre($langs->trans("ExportsArea"));

print "Cet espace permet de réaliser l'export des expéditions cloturés de date à date pour le transporteur XPO.<br>";
print "Cet export est basé sur la date de clôture de l'expédition.<br>";
print '<br>';

print '<div>';

if ($user->rights->export->creer)
{
    print '<form name="exporter" method="POST">';
    print "Exporter les expéditions du&nbsp;";
    print $form->select_date('', 'date_from')."&nbsp;au&nbsp;";
    print $form->select_date('', 'date_to');
    print '<a id="export" class="butAction" href="'.DOL_URL_ROOT.'/exports/export.php?leftmenu=export">'.$langs->trans("Export").'</a>';
    print '</form>';
    
    $exportfiles=dol_dir_list(DOL_DATA_ROOT.'/cliacobal/export',"files",0,'','(\.pdf)$','date',SORT_DESC,1);
    
    $modulepart = 'cliacobal';
    $permissions = 1;
    $formfile=new FormFile($db);
    
    $formfile->list_of_documents(
        $exportfiles,
        $object,
        $modulepart,
        $param,
        0,
        '/export/',		// relative path with no file. For example "moduledir/0/1"
        $permissions
    );
    
    ?>
    
<script type="text/javascript">
$(document).ready(function(){
	console.log('echo');
	$('#export').click(function(e){
		e.preventDefault();
		if($(this).hasClass("butAction")) {
			console.log("GO");
            
            if($('#progress').length==0) {
                $('body').append('<div id="progress" title="titre du modal"></div>');
            }

            var html = '<div style="text-align:center">';
            html += "<p>L'export de la liste est en cours. Merci de patienter et de ne pas rafraichir la page pour ne pas bloquer l'export !</p>";
            html += '<p id="final"></p>'
            html += '</div>';
            $('#progress').html(html);

            $('#progress').dialog({
                modal:true,
                width:'80%'
            });

            $('.ui-dialog-titlebar-close').hide(); // cache le bouton de fermeture

            var from = $('#date_fromyear').val()+"-"+$('#date_frommonth').val()+"-"+$('#date_fromday').val();
            var to = $('#date_toyear').val()+"-"+$('#date_tomonth').val()+"-"+$('#date_today').val();
            printfile(from, to);
            
		} else {
			console.log("Déjà en cours !");
		}
		
	});

    function printfile(from, to)
    {
        
        $.ajax({
            url : "<?php echo dol_buildpath('/cliacobal/script/interface.php',1).'?put=export' ; ?>"
            ,data : {
            	date_from : from
                ,date_to : to
            }
            ,method:"post"
            ,dataType:'json'
        }).done(function(data) {
            console.log(data);
			$('#export').toggleClass("butAction").toggleClass("butActionRefused");
            $('.ui-dialog-titlebar-close').show();
            $('#export').toggleClass("butAction").toggleClass("butActionRefused");
            location.reload();
        });
    }
});

</script>
    
    <?php
}
else
{
    print "Vous n'avez pas les droits pour créer des exports";
}
    
print '</div>';
print '<br>';

llxFooter();

$db->close();
