<?php
/* Copyright (C) 2017 	oscss-shop 					<support@oscss-shop.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * 	\file       htdocs/edi/core/tabs/thirdparty.tabs.class.php
 * 	\ingroup    edi
 * 	\brief      Fiche d'information sur une proposition commerciale
 */
use \Form as From;
use \Formother as Formother;
use \FormFile as FormFile;
use \UserGroup as UserGroup;
use \User as User;
use \ExtraFields as ExtraFields;
use \CORE\FRAMEWORK\Societe as Societe;
use \CORE\FRAMEWORK\AutoTabs as AutoTabs;
use \CORE\FRAMEWORK\AutoTabsRequired as AutoTabsRequired;

require_once(DOL_DOCUMENT_ROOT."/core/lib/date.lib.php");
require_once(DOL_DOCUMENT_ROOT."/core/lib/functions2.lib.php");
require_once(DOL_DOCUMENT_ROOT."/core/lib/company.lib.php");


dol_include_once('/edi/class/edi.class.php');
dol_include_once('/edi/class/societe.edi.class.php');
dol_include_once('/edi/class/setup_loader.class.php');
dol_include_once('/edi/core/lib/edi.lib.php');

$langs->load('propal');
$langs->load('compta');
$langs->load('bills');
$langs->load("companies");
$langs->load("edi@edi");


// Security check
// if ($user->societe_id) $socid=$user->societe_id;
// $result = restrictedArea($user, 'propale', $id, 'propal');

/**
  @class TabsThirdparty

 */
Class TabsThirdparty extends AutoTabs implements
AutoTabsRequired
{
    public
    /**
      @var
     */
        $name
        ,
        /**
          @var array
         */
        $refparam = array(
            'socid' => 'int'
            , 'ref' => 'alpha'
            , 'action' => 'alpha'
            , 'mod' => 'alpha'
            , 'tab' => 'alpha'
            )
        /**
          @var
         */
        , $lines    = array()

    ;

    /**
      @fn Init()
      @brief
      @param
      @return  treu/ false/ 2
      if return false: error
      if return true;
      if return 2,  No AutoTabs load class by type
     */
    public function _Init()
    {
        global $langs;

        $file = basename(__FILE__);

        $this->type = substr($file, 0, strpos($file, '.'));

        $class = $this->FV->GetClassByType($this->type);
        if (empty($class)) return false;


        // load Specific context
        $class .= 'Edi';
        if (!class_exists($class)) return 2;

        $this->object = new $class($this->db);

        $this->TitlePage = $langs->trans("EdiCompanyTabTitle");
        $this->TitleTab  = $langs->trans("EdiCompanyTabName");

        return true;
    }

    /**
      @fn Init()
      @brief
      @param
      @return
     */
    public function _Process()
    {
        global $conf, $langs;

        if ($this->GetParams('socid') != null || $this->GetParams('ref') != null) {

            $this->object->fetch($this->GetParams('socid'), $this->GetParams('ref'));
            $this->object->fetch_thirdparty();

            if (!empty($conf->global->MAIN_HTML_TITLE) && preg_match('/thirdpartynameonly/', $conf->global->MAIN_HTML_TITLE) && $this->object->name)
                    $this->TitlePage = $this->object->name." - ".$langs->trans('EdiCompanyTabCard');
// 			$this->TitlePage = $langs->trans("EdiCompanyTabC");


            $this->lines = printline(new Edi($this->db), array(), 0, 0, $this->GetParams('socid'));
        }

        return true;
    }

    /**
      @fn Display()
      @brief
      @param
      @return
     */
    public function _Display()
    {
        global $langs, $conf, $user, $hookmanager;

        global $formfile;

        $db = $this->db;

        $hookmanager->initHooks(array('ediexternaltabs', 'edithirdpartycard'));     // Note that conf->hooks_modules contains array
// 			$formfile = new FormFile($this->db);
        // Country
        print '<div class="fichecenter">';
        print '<div class="underbanner clearboth"></div>';
        print '<table width="100%">';
        print '<tr><td>'.$langs->trans("Config").'</td><td colspan="3">';

        //if(!empty($this->object->client) )
        echo ' <a class="button" href="'.dol_buildpath('edi/tab/edi_societe_config.php?type=client&socid='.$this->GetParams('socid'), 2).'" >'.$langs->trans('EdiSetupButtonSend').'</a>';

        //if(!empty($this->object->fournisseur) )
        echo ' <a class="button" href="'.dol_buildpath('edi/tab/edi_societe_config.php?type=vendor&socid='.$this->GetParams('socid'), 2).'" >'.$langs->trans('EdiSetupButtonReceive').'</a>';


        print '</td></tr>';
        print '</table>';
        print '</div>';


        print '<div class="fichecenter">';
        print '<div class="underbanner clearboth"></div>';
        ?>
        <table summary="" class="centpercent notopnoleftnoright" style="margin-bottom: 2px;">
            <tbody>
                <tr>
                    <td class="nobordernopadding widthpictotitle" valign="middle">
                        <img src="/htdocs/theme/eldy/img/title_generic.png" alt="" title="" id="pictotitle" border="0">
                    </td>
                    <td class="nobordernopadding" valign="middle">
                        <div class="titre"><?= $langs->trans('EdiOutput') ?>
                        </div>
                    </td>
                </tr>
            </tbody>
        </table>
        <p><?= $langs->trans('EdiOutputMore') ?></p>



        <table class="border" width="100%" style="table-layout: fixed;">
            <thead>
                <tr>
        <?php
        // Fields from hook
        $parameters = array();
        $reshook    = $hookmanager->executeHooks('printFieldListTitle', $parameters);    // Note that $action and $object may have been modified by hook
        print $hookmanager->resPrint;
        ?>
                </tr>
            </thead>
            <tbody>
        <?php foreach ($this->lines as $line): ?>
                    <tr style="background-color: <?= $line->color ?>">
            <?php
            // Fields from hook
            $parameters = array('obj' => $line);
            $reshook    = $hookmanager->executeHooks('printFieldListValue', $parameters);    // Note that $action and $object may have been modified by hook
            print $hookmanager->resPrint;
            ?>
                    </tr>
        <?php endforeach; ?>
            </tbody>
        </table>
                    <?php
                    print '</div>';
                    return false;
                }
            }
