<?php
/* Copyright (C) 2005-2009 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2009      Regis Houssin        <regis.houssin@capnetworks.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       htdocs/core/triggers/interface_20_all_Logevents.class.php
 *  \ingroup    core
 *  \brief      Trigger file for
 */
/**
  @brief Call Specific Class provider for OScssShop Exts
  This class garanted process for dolibarr evolution
 */
dol_include_once('/framework/main.inc.php');
dol_include_once('/framework/core/lib/framework.lib.php');

/**
 *  Class of triggers for security events
 */
class InterfaceGenerate
{
    var $db;
    var $error;
    var $date;
    var $duree;
    var $texte;
    var $desc;

    /**
     *   Constructor
     *
     *   @param		DoliDB		$db      Database handler
     */
    function __construct($db)
    {
        $this->db = $db;

        $this->name        = preg_replace('/^Interface/i', '', get_class($this));
        $this->family      = "module";
        $this->description = "Triggers of this module allows to add security event records inside Dolibarr.";
        $this->version     = 'dolibarr';                        // 'experimental' or 'dolibarr' or version
        $this->picto       = 'technic';
    }

    /**
     *   Return name of trigger file
     *
     *   @return     string      Name of trigger file
     */
    function getName()
    {
        return $this->name;
    }

    /**
     *   Return description of trigger file
     *
     *   @return     string      Description of trigger file
     */
    function getDesc()
    {
        return $this->description;
    }

    /**
     *   Return version of trigger file
     *
     *   @return     string      Version of trigger file
     */
    function getVersion()
    {
        global $langs;
        $langs->load("admin");

        if ($this->version == 'experimental') return $langs->trans("Experimental");
        elseif ($this->version == 'dolibarr') return DOL_VERSION;
        elseif ($this->version) return $this->version;
        else return $langs->trans("Unknown");
    }

    /**
     *      Function called when a Dolibarrr business event is done.
     *      All functions "run_trigger" are triggered if file is inside directory htdocs/core/triggers
     *
     *      @param	string		$action		Event action code
     *      @param  Object		$object     Object
     *      @param  User		$user       Object user
     *      @param  Translate	$langs      Object langs
     *      @param  conf		$conf       Object conf
     *      @param  string		$entity     Value for instance of data (Always 1 except if module MultiCompany is installed)
     *      @return int         			<0 if KO, 0 if no triggered ran, >0 if OK
     */
    function run_trigger($action, $object, $user, $langs, $conf, $entity = 1)
    {

        dol_include_once('/edi/class/setup_loader.class.php');
        dol_include_once('/edi/class/actioncomm.class.php');
        global $mysoc;

        $setup = new setup_loader($this->db);


        if (
            is_object($object) && property_exists($object, 'thirdparty') && is_object($object->thirdparty) && property_exists($object->thirdparty, 'id')
// 						
// /// TODO FIXEME
// 						|| !in_array(strtolower(get_class($object)), array('commande', 'facture', 'commandefournisseur', 'facturefournisseur') )
        ) {
// var_dump(property_exists($object, 'socid'), $object->socid); 

            if (in_array($object->element, array('order_supplier', 'invoice_supplier'))) $mode = 'vendor';
            else $mode = 'client';


            if (property_exists($object->thirdparty, 'parent') && !empty($object->thirdparty->parent)) $setup->load_user($object->thirdparty->parent, $mode);
            elseif (property_exists($object, 'socid') && !empty($object->socid)) $setup->load_user($object->socid, $mode);
            elseif (property_exists($object, 'fk_soc')) $setup->load_user($object->fk_soc, $mode);
            else $setup->load_user($object->thirdparty->id, $mode);
// print_r($setup->user); 
// exit; 
            foreach ($setup->descriptor as $norme => $array) {
// var_dump($action); 
// 
//             print_r($array->trigger->$action); 
// print_r($setup->user->setup->$norme->message->output); 

// exit; 

				if (isset($array->trigger->$action)) { // there are trigger(s) to run for this action
                    foreach ($array->trigger->$action as $trigger_conf) {

                        if ($setup->user->setup->$norme->message->output->{$trigger_conf->message}) {  // user is allowed to run this trigger
                            dol_include_once("/edi/plugin/$norme".$trigger_conf->file);


							$setup->current_norme = $norme; 
							$setup->current_message = $trigger_conf->message; 
							
                            $classtrigger = 'EdiTrigger'.$trigger_conf->message;

                            $trigger = new $classtrigger($action, $object, $user, $langs, $conf, $entity, $this->db, $mysoc, $setup);
                            $trigger->set_my_soc_id($object->thirdparty->id);

                            $res = $trigger->run_trigger();

                            dol_syslog("Trigger '".$this->name."' for action '$action' launched by ".__FILE__.". id=".$object->id);

                            $callable = 'send_edi_'.$setup->user->trans_mode;

                            if (method_exists($trigger, $callable)) {

								$sql = "INSERT INTO ".MAIN_DB_PREFIX."edi_send ( method, file,  fk_soc, fk_ref_uniq, status ) VALUES ( '".$callable."', '".$trigger->output_file."', '".$trigger->setup->fk_soc."', '".$res->ref_uniq."', '".$res->count_message."')";

                                $this->db->query($sql);

                                if (!empty($conf->global->EDIDEBUGLEVEL) && $conf->global->EDIDEBUGLEVEL >= 2) trigger_error('[EDI][CRON] '.$sql, E_USER_NOTICE);


// 										if(! empty( $conf->global->EDIDEBUGLEVEL ) && $conf->global->EDIDEBUGLEVEL >= 5 )
// 												trigger_error('[EDI][TRIGGER] '."EDI action '$action' launched 'send_" . $setup->user->trans_mode . "' to '" . $setup->user->trans_destination . "'" , E_USER_NOTICE );

								$trigger->setMessageType($trigger_conf->message);
								$trigger->setNorme($norme);
								$trigger->setTransMode($setup->user->trans_mode);



								$trigger->ref_uniq      = $res->ref_uniq;
								$trigger->ref_parent    = $res->ref_parent;
								$trigger->count_message = $res->count_message;
								$trigger->dol_type      = $res->dol_type;
								$trigger->dol_type_id   = $res->dol_type_id;
								$trigger->accept_message(0);

// 								trigger_error('[EDI][TRIGGER] '."Accept method" , E_USER_NOTICE );
								$trigger->trace_message();

								/**
								SEGMENT FOR PROCESS APERAK
								*/
// 								$ActionComm = new ActionCommEdi($this->db);
// 								$ActionComm->SetEvent(
// 									$user
// 									, $object->thirdparty
// 									, $object
// 									, $trigger->coreclass
// 								);
                            
                            }
//                             exit;
                        }
                        // Maintenance : triggers define this action, but is not event EDI 
                        // close, delete .. 
                        else {
                        
// 							$sql = "INSERT INTO ".MAIN_DB_PREFIX."edi_send ( method, file,  fk_soc, fk_ref_uniq, status ) VALUES ( '".$callable."', '".$trigger->output_file."', '".$trigger->setup->fk_soc."', '".$res->ref_uniq."', '".$res->count_message."')";
// 
// 							$this->db->query($sql);
                        }
                    }
                }
            }
// exit;
            return 1;
        } elseif (get_class($object) == "ActionsEdi") {
            $mode = 'client';
            foreach ($setup->descriptor as $norme => $array) {
                if (isset($array->trigger->$action)) { // there are trigger(s) to run for this action
                    foreach ($array->trigger->$action as $trigger_conf) {
                        foreach ($setup->find_users($norme, $trigger_conf->message) as $thirdparty) {
                            $setup->load_user($thirdparty, $mode);
                            //trigger_error("TRIGGER EDI : soc ".$thirdparty.' - '.$trigger_conf->message, E_USER_NOTICE);
                            if ($setup->user->setup->$norme->message->output->{$trigger_conf->message}) {  // user is allowed to run this trigger
                                dol_include_once("/edi/plugin/$norme".$trigger_conf->file);
                                dol_include_once("/edi/plugin/$norme".$setup->descriptor->$norme->message->output->{$trigger_conf->message}->file);
                                $classtrigger = 'EdiTriggerGeneric';

                                $trigger = new $classtrigger($action, $thirdparty, $user, $langs, $conf, $entity, $this->db, $mysoc, $setup, $trigger_conf->message,$norme);
                                $trigger->set_my_soc_id($thirdparty);
                                $res = $trigger->run_trigger();

                                dol_syslog("Trigger '".$this->name."' for action '$action' launched by ".__FILE__.". id=".$soc->id);

                                $callable = 'send_edi_'.$setup->user->trans_mode;
                                if (method_exists($trigger, $callable)) {
                                    // according to user settings, transmission this way
                                    //$trigger->$callable();
// 										INSERT INTO pfortierdolibarr505.llx_edi_send (rowid, method, file, trans_destination, response, status, sendcount) VALUES (NULL, 'test', 'file', 'dest', '', '0', '0');
                                    $sql = "INSERT INTO ".MAIN_DB_PREFIX."edi_send ( method, file,  fk_soc, fk_ref_uniq ) VALUES ( '".$callable."', '".$trigger->output_file."', '".$trigger->setup->fk_soc."', '".$res->ref_uniq."')";

                                    $this->db->query($sql);


                                    if (!empty($conf->global->EDIDEBUGLEVEL) && $conf->global->EDIDEBUGLEVEL >= 2) trigger_error('[EDI][CRON] '.$sql, E_USER_NOTICE);


// 										if(! empty( $conf->global->EDIDEBUGLEVEL ) && $conf->global->EDIDEBUGLEVEL >= 5 )
// 												trigger_error('[EDI][TRIGGER] '."EDI action '$action' launched 'send_" . $setup->user->trans_mode . "' to '" . $setup->user->trans_destination . "'" , E_USER_NOTICE );
                                }
                                $trigger->setMessageType($trigger_conf->message);
                                $trigger->setNorme($norme);
                                $trigger->setTransMode($setup->user->trans_mode);



                                $trigger->ref_uniq      = $res->ref_uniq;
                                $trigger->ref_parent    = $res->ref_parent;
                                $trigger->count_message = $res->count_message;
                                $trigger->dol_type      = $res->dol_type;
                                $trigger->dol_type_id   = $res->dol_type_id;
                                $trigger->accept_message();

                                $trigger->trace_message();

                                /**
                                  SEGMENT FOR PROCESS APERAK
                                 */
                                $ActionComm = new ActionCommEdi($this->db);
                                $ActionComm->SetEvent(
                                    $user
                                    , $object->thirdparty
                                    , $object
                                    , $trigger->coreclass
                                );
                            }
                        }
                    }
                }
            }
        }

        return 0;
    }
}
