<?php
/* Copyright (C) 2016		 Oscss-Shop       <support@oscss-shop.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * or see http://www.gnu.org/
 */

namespace EDI;

// on va utiliser le datetime plus bas donc on le déclare, pareil pour l'object Facture
use \DateTime;
use \CommandeFournisseur;
use \Commande;
use \Societe;

// use \CORE\FRAMEWORK\ProductFournisseur;
// use \CORE\FRAMEWORK\Product;

class D96aCommon extends Encoder
{
    public $header,
        $message = [],
        $trailer = [],
        $arr;
    public $heck_digit;

    /**
      @var uniq ref message
     */
    public $ref_uniq = '';

    /**
      @var count_message
     */
    public $count_message = '';

    /**
      @var ref_parent
     */
    public $ref_parent = '';

    /**
      @var  Syntax version number of message
     */
    public $unbCharset = "UNOC";

    /**
      @var  Syntax version number of message
     */
    public $version = "2";

    /**
      @var custom
     */
    public $custom;

    /**
     */
    public $db;

    /**
     */
    public function SetDB($db)
    {
        $this->db = $db;
        return $this->db;
    }

    /**
      @brief To start, identify and specify an interchange.
      Segment of message
      @param $version int version of message
      @return void
     */
    public function loadCustom()
    {
        $this->custom = new \stdClass();
        if (isset($this->obj_ref) && isset($this->obj_ref->thirdparty)) { // i ve got a third party
            $path = DOL_DATA_ROOT.DIRECTORY_SEPARATOR.'societe'.DIRECTORY_SEPARATOR.$this->obj_ref->thirdparty->id.DIRECTORY_SEPARATOR.'edi'.DIRECTORY_SEPARATOR;
            if (is_dir($path)) {
                // search a custom called global in thirdparty dir
                if (file_exists($path.'global.json')) {
                    $f = json_decode(file_get_contents($path.'global.json'), true);
                    if (json_last_error() == JSON_ERROR_NONE) {
                        $this->custom = (object) array_merge((array) $this->custom, $f);
                    }
                }
                if (file_exists($path.$this->type.'.json')) {
                    $f = json_decode(file_get_contents($path.$this->type.'.json'), true);
                    if (json_last_error() == JSON_ERROR_NONE) {
                        $this->custom = (object) array_merge((array) $this->custom, $f);
                    }
                }
            }
        }
    }

    /**
      @brief To start, identify and specify an interchange.
      Segment of message
      @param $version int version of message
      @return Array UNB Section
     */
    protected function GetUNB()
    {
        global $conf;

        return array
            (
            "UNB"
            , array(
                "{$this->unbCharset}",
                "{$this->version}"
            )
            , array(
                "{$conf->global->EDI_UNIQNUM}",
                (!empty($conf->global->EDI_GLNFULLQUALIFIED) ? "14" : '01' )
            )
            , array(
                "{$this->setup->user->edi_num}",
                "14"
            )
            , make_UNB_time()
            , "{$this->ref_unid}"
        );
    }

    /**
      @return Array UNH Section
     */
    protected function GetUNH()
    {
        global $conf;

        // MESSAGE HEADER
        // Mandatory Max repeat: 1
        // A service segment starting and uniquely identifying a message.

        $return = array(
            "UNH"
            // 0062 MESSAGE REFERENCE NUMBER
            // Mandatory Type: anLength: 0..14
            , "{$this->ref_unid}"

            // S009 MESSAGE IDENTIFIER
            // Mandatory
            // Identification of the type, version etc. of the message being interchanged.
            , array(
                // 0065 Message type identifier
                // MandatoryType: anLength: 0..6
                strtoupper($this->type)

                // 0052 Message type version number
                // MandatoryType: anLength: 0..3
                , "D"

                // 0054 Message type release number
                // MandatoryType: anLength: 0..3
                , "96A" // normes
                // 0051 Controlling agency
                // MandatoryType: anLength: 0..2
                , "UN"

                // 0057 Association assigned code
                // ConditionalType: anLength: 0..6
                , "{$this->AssociationAssignedCode}"
            )
        );

        if (empty($this->AssociationAssignedCode)) {
            // 0068 COMMON ACCESS REFERENCE
            // ConditionalType: anLength: 0..35
            $return[] = "{$this->ref_parent}";

            // S010 STATUS OF THE TRANSFER
            // Conditional
            $return[] = array(
                // 0070 Sequence message transfer number
                // MandatoryType: n Length: 0..2
                "{$this->count_message}"
                // 0073 First/last sequence message transfer indication
                // ConditionalType: nLength: 1
                // FIRST / LAST
                , (($this->count_message <= 1) ? 'FIRST' : 'LAST' )
            );
        }


        return $return;
    }

    /**
      @brief Construct NAD (Address) and LOC (complement) Section, Array
      @param string (lengt 2) $type
      @param int gln identify lenght 14
      @param object $object
      @param bool $short
      @return Array NAD Section
     */
    protected function GetNAD_LOC($type, $gln, $object, $short = true)
    {

        $return = array($this->GetNAD($type, $gln, $object, $short));

        if ($type == 'DP' && !empty($object->tel)) $return[] = $this->GetLOC(7, $object);

        return $return;
    }

    /**
      @brief Construct NAD (Address) Section, Array
      @param string (lengt 2) $type
      @param int gln identify lenght 14
      @param object $object
      @param bool $short
      @return Array NAD Section
     */
    protected function GetNAD($type, $gln, $object, $short = true)
    {
        $return = array(
            // NAME AND ADRESSE	M1
            "NAD",
            // 3035 PARTY QUALIFIER
            // Mandatory
            // Type: an
            // Length: 0..3
            //
            // Code giving specific meaning to a party.
            "{$type}", // Invoice
            // C082 PARTY IDENTIFICATION DETAILS
            // 	Conditional
            // Identification of a transaction party by code.
            array(
                "{$gln}",
                "",
                "9"
            ),
        );

        // Not data for internal address
        if ($type != 'SU' || !$short) {
            // C058  NAME AND ADDRESS
            //
            //     Conditional
            //
            // Unstructured name and address: one to five lines.
            $return[] = array();

            if (empty($gln) || !$short) {

                // C080 PARTY NAME
                //
                //     Conditional
                //
                // Identification of a transaction party by name, one to five lines. Party name may be formatted.
                $return[] = array(
                    $object->name,
                    $object->address
                );


                // 3164 CITY NAME
                //
                //     Conditional
                //     Type: an
                //     Length: 0..35
                //
                // Name of a city (a town, a village) for addressing purposes.
                $return[] = $object->town;


                // 3229  COUNTRY SUB-ENTITY IDENTIFICATION
                //
                //     Conditional
                //     Type: an
                //     Length: 0..9
                //
                // Identification of the name of sub-entities (state, province) defined by appropriate governmental agencies.
                $return[] = $object->state_code;


                // 3251 POSTCODE IDENTIFICATION
                //     Conditional
                //     Type: an
                //     Length: 0..9
                // Code defining postal zones or addresses.
                $return[] = $object->zip;
            } else {

                // C080 PARTY NAME
                $return[] = array();

                // 3164 CITY NAME
                $return[] = '';

                // 3229  COUNTRY SUB-ENTITY IDENTIFICATION
                $return[] = '';

                // 3251 POSTCODE IDENTIFICATION
                $return[] = '';
            }


            // 3207  COUNTRY, CODED
            //     Conditional
            //     Type: an
            //     Length: 0..3
            // Identification of the name of a country or other geographical entity as specified in ISO 3166.
            $return[] = $object->country_code;
        }
        return $return;
    }

    /**
      @brief Construct NAD (Address) Section, Array
      @param string (lengt 2) $type
      @param int gln identify lenght 14
      @param object $object
      @return Array NAD Section
     */
    protected function GetLOC($type, $object)
    {
        return array(
            // PLACE LOCATION C25
            "LOC",
            //  3227
            // PLACE/LOCATION QUALIFIER
            //    Mandatory
            //    Type: an
            //    Length: 0..3
            //
				// Code identifying the function of a location.
            "{$type}",
            // C517  LOCATION IDENTIFICATION
            // 	Conditional
            // Identification of a location by code or name.
            array(
                // 3225 Place/location identification
                //     Conditional
                //     Type: an
                //     Length: 0..25
                // Identification of the name of place/location, other than 3164 City name.
                "{$object->tel}",
                //  1131 Code list qualifier
                //     Conditional
                //     Type: an
                //     Length: 0..3
                // Identification of a code list.
                "12"
            )
        );
    }

    /**
      @return Array UNH Section
     */
    protected function GetBGM($RESPONSETYPE = false)
    {

        //BGM M 1
        // BEGINNING OF MESSAGE
        // Mandatory Max repeat: 1
        // A segment by which the sender must uniquely identify the order by
        $r = array(
            "BGM"

            // C002 DOCUMENT/MESSAGE NAME
            // Conditional
            , array(
                // 1001 Document/message name, coded
                // ConditionalType: anLength: 0..3
                "{$this->bgmtype}"

            // 1131 Code list qualifier
            // ConditionalType: anLength: 0..3
//                     , ""
            // 3055 Code list responsible agency, coded
            // ConditionalType: anLength: 0..3
//                     , ""
            // 1000 Document/message name
            // ConditionalType: anLength: 0..35
//                     , "{$this->ref}"
            )

            // 1004 DOCUMENT/MESSAGE NUMBER
            // ConditionalType: anLength: 0..35
            // Reference number assigned to the document/message by the issuer.
            , (!empty($this->ref) ? "{$this->ref}" : "{$this->id}") // numero de message
            // 1225 MESSAGE FUNCTION, CODED
            // ConditionalType: anLength: 0..3
            , "{$this->MessageFunctionCoded}"
        );

        if ($RESPONSETYPE)
        // 4343 RESPONSE TYPE, CODED
        // ConditionalType: anLength: 0..3
        //Indicates that an acknowledge relating to receipt is required.
            $r[] = "AB";

        return $r;
    }

    /**
      @param string $refinternal
      @param string $code
      @param bool $short
     */
    protected function GetExternalAddress($refinternal, $code, $short = true)
    {
        $usecontact     = false;
        $arrayidcontact = $this->obj_ref->getIdContact('external', strtoupper($refinternal));
        if (count($arrayidcontact) > 0) {
            $usecontact = true;
            $result     = $this->obj_ref->fetch_contact($arrayidcontact[0]);
        }
        //Recipient name
        // On peut utiliser le nom de la societe du contact
        if ($usecontact /* && !empty($conf->global->MAIN_USE_COMPANY_NAME_OF_CONTACT) */) {
            $shipping = $this->obj_ref->contact;
            $shipping->fetch_thirdparty();

            if (empty($shipping->name)) $shipping->name = $shipping->lastname;

            if (empty($shipping->phone_pro) && empty($shipping->phone_mobile)) $shipping->phone = $shipping->thirdparty->phone;
            if (!empty($shipping->phone_pro)) $shipping->phone = $shipping->phone_pro;
            elseif (!empty($shipping->phone_mobile)) $shipping->phone = $shipping->phone_mobile;

            $gln = $shipping->array_options['options_edirum'];
        } else {
            $shipping = $this->obj_ref->thirdparty;
            $gln      = $this->setup->user->edi_num;
        }

        return $this->GetNAD_LOC(strtoupper($code), $gln, $shipping, $short);
    }

    /**
     */
    protected function combine_array()
    {
        $this->arr    = $this->header;
        $this->length = count($this->arr);
        foreach ($this->message as $arr) {
            $this->length += count($arr); // count length of message
            $this->arr    = array_merge($this->arr, $arr);
        }
        $this->length += count($this->trailer); // count length of message
        $this->arr    = array_merge($this->arr, $this->trailer);

        $this->arr = array_merge($this->arr,
            array(
            array(//Compteur de segments
                "UNT",
                "$this->length", //  NUMBER OF SEGMENTS IN A MESSAGE ???????? from where to where ?
                "{$this->ref_unid}" // MESSAGE REFERENCE NUMBER ???
            ),
// 						array(//Control Total Optionnal
//                 "CNT",
//                 "$this->length", //  NUMBER OF SEGMENTS IN A MESSAGE ???????? from where to where ?
//                 "{$this->order->id}" // MESSAGE REFERENCE NUMBER ???
//             ),
            array(//Fin de l’interchang
                "UNZ",
                "1", // a garder à 1
                $this->ref_unid // ref to UNB password 
            )
        ));
        return $this->arr;
    }

    /**
     */
    protected function make_ean13_ILN($nom_societe, $etablissement_interne = 1)
    {
        $md5              = str_replace(".", "", "".hexdec(md5($nom_societe)));
        $digits           = substr($md5, 0, 6).str_pad($etablissement_interne, 6, "0", STR_PAD_LEFT);
        //first change digits to a string so that we can access individual numbers
        $digits           = "".$digits;
        $this->heck_digit = $this->get_ean13_key($digits);
        return $digits.$this->heck_digit;
    }

    //protected function get_ean13_key(string $digits)
    protected function get_ean13_key($digits)
    {
        if (strlen($digits) != 12) {
            return FALSE;
        }
        // 1. Add the values of the digits in the even-numbered positions: 2, 4, 6, etc.
        $even_sum       = $digits{1} + $digits{3} + $digits{5} + $digits{7} + $digits{9} + $digits{11};
        // 2. Multiply this result by 3.
        $even_sum_three = $even_sum * 3;
        // 3. Add the values of the digits in the odd-numbered positions: 1, 3, 5, etc.
        $odd_sum        = $digits{0} + $digits{2} + $digits{4} + $digits{6} + $digits{8} + $digits{10};
        // 4. Sum the results of steps 2 and 3.
        $total_sum      = $even_sum_three + $odd_sum;
        // 5. The check character is the smallest number which, when added to the result in step 4,  produces a multiple of 10.
        $next_ten       = (ceil($total_sum / 10)) * 10;
        return $next_ten - $total_sum;
    }

    function findDateShippingInLinkedObj()
    {
        $date = FALSE;
        //look for a shipping first
        if (isset($this->obj_ref->linkedObjects['shipping'])) {
            foreach ($this->obj_ref->linkedObjects['shipping'] as $id => $obj) {
                if ($obj->statut > 0) {
                    if (isset($obj->date_delivery) && !empty($obj->date_delivery)) {
                        return $obj->date_delivery;
                    }
                    if (isset($obj->date_livraison) && !empty($obj->date_livraison)) {
                        return $obj->date_livraison;
                    }
                }
            }
        }
        //look all linked
        foreach ($this->obj_ref->linkedObjects as $type => $arr) {
            foreach ($arr as $id => $obj) {
                if ($obj->statut > 0) {
                    if (isset($obj->date_delivery) && !empty($obj->date_delivery)) {
                        return $obj->date_delivery;
                    }
                    if (isset($obj->date_livraison) && !empty($obj->date_livraison)) {
                        return $obj->date_livraison;
                    }
                }
            }
        }
        return $date;
    }
}