<?php
/* Copyright (C) 2016		 Oscss-Shop       <support@oscss-shop.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * or see http://www.gnu.org/
 */

namespace CORE\FRAMEWORK;

namespace EDI;

use \DateTime;
use \Contact;
use \CORE\FRAMEWORK\Facture;
use \CORE\FRAMEWORK\Societe as Societe;

dol_include_once('/core/lib/product.lib.php');
global $langs;
$langs->loadLangs(array('other')); // for kg units

class INVOIC extends D96aCommon
{
    var $edi
        , $output
        , $length = 0
        , $uniq_id
        , $invoice
        , $company
        , $tax    = []
        , $setup;

    /**
      @var  Syntax version number of message
     */
    public $version = "2";

    /**
      @var Type of message
     */
    public $type = 'INVOIC';

    /**
      @var Code, assigned by the association responsible for the design and maintenance of the message type concerned, which further identifies the message.
     */
    public $AssociationAssignedCode = 'EAN008';

    /**
      @var   1225 MESSAGE FUNCTION, CODED
     */
    public $MessageFunctionCoded = "53"; //( code TEST)

    /**
      @fn __construct()
     */

    public function __construct($obj_ref, /* Societe */ $mysoc, $setup)
    {

// 				if(get_class($obj_ref) =='Facture' )
        $this->InitInvoice($obj_ref);

        //find and load json for the associated thirdparty
        $this->loadCustom();
        // bypass default values
        if (isset($this->custom->unbCharset)) $this->unbCharset = $this->custom->unbCharset;
        if (isset($this->custom->version)) $this->version    = $this->custom->version;
        $this->mysoc      = $mysoc;
        $this->setup      = $setup;
    }

    /**
      @fn Generate($allref)
      @brief surclass function to had UNA delimiter
      @param $allref object
      @return none
     */
    public function Generate($allref)
    {

        /**
          identification message
         */
        $this->ref_unid      = $allref->ref_uniq;
        $this->count_message = $allref->count_message;
        $this->ref_parent    = $allref->ref_parent;


        $this->prepare();



        // build arrays
        $this->header();
        $this->message();
        $this->trailer();
        parent::__construct();

        //debug
//        header("Content-Type: text/plain; charset=utf-8");
//        print $this->encodeUNA(false);
//        die();
    }

    /**
      @fn InitInvoice
      @param $obj_ref Object Dolibarr Object
      @return none
     */
    protected function InitInvoice(/* Facture */ $obj_ref)
    {

        $this->obj_ref = $obj_ref;
        $this->obj_ref->fetchObjectLinked();

        $this->ref  = (isset($this->obj_ref->newref) ? $this->obj_ref->newref : $this->obj_ref->ref );
        $this->id   = $this->obj_ref->id;
        $this->path = DOL_DATA_ROOT.DIRECTORY_SEPARATOR.'facture'.DIRECTORY_SEPARATOR.'%1$s'.DIRECTORY_SEPARATOR.'%1$s'.".edi.txt";



        // CODE LIST
        // 325 ProFormat Invoice
        // 326 Partial Invoice
        // 380 Commercia invoice
        // 384 Correect Invoice
        // 385 Consolaidated invoic e
        // 386 Prepayment invoice
        // 387 Hire invocice (location bien / equipement)
        $this->bgmtype = "380";

        if ($this->obj_ref->type == 1) $this->bgmtype = "384";
        if ($this->obj_ref->type == 2) $this->bgmtype = "384";
        if ($this->obj_ref->type == 3) $this->bgmtype = "386";
        if ($this->obj_ref->type == 4) $this->bgmtype = "325";
        if ($this->obj_ref->situationinvoice) $this->bgmtype = "326";
    }

    /**
      @brief surclass function to had UNA delimiter
     */
    public function encodeUNA($wrap = true)
    {
        $this->combine_array(); // output is in $this->arr
        $this->enableUNA();
//        $this->encode($this->arr, $wrap);
        $this->encode($this->arr, false);
        $this->output = $this->get();
        // unoa
        if ($this->unbCharset == 'UNOA') {
            $this->output = $this->remove_accents($this->output);
            if ($this->version == 1) { // not sure, to be clarified
                $this->output = strtoupper($this->output);
            }
        }
        return $this->output;
    }

    /**
      @fn GetPath()
      @brief
      @return string path
     */
    public function GetPath()
    {
        return $this->path;
    }

    /**
      @fn prepare()
      @brief prepare and calculate data
      @return none
     */
    private function prepare()
    {

        // confition regelement
        if ($this->obj_ref->cond_reglement_id == '1') { // a reception
            $this->paycond = 1; // Paiement direct
        } elseif ($this->obj_ref->cond_reglement_id == '6') { // proformat
            $this->paycond = 'P'; // Paiement direct
        } elseif ($this->obj_ref->cond_reglement_id == '7') { // proformat
            $this->paycond = "J12"; // Paiement direct
        } elseif ($this->obj_ref->cond_reglement_id == '2') { // proformat
            $this->paycond = "J30"; // Paiement direct
        } elseif ($this->obj_ref->cond_reglement_id == '3') { // proformat
            $this->paycond = "E30"; // Paiement direct
        } elseif ($this->obj_ref->cond_reglement_id == '4') { // proformat
            $this->paycond = "J60"; // Paiement direct
        } elseif ($this->obj_ref->cond_reglement_id == '5') { // proformat
            $this->paycond = "E60"; // Paiement direct
        }


        // mode payement
        if ($this->obj_ref->mode_reglement_id == '7') { // cheque
            $this->payid   = 23;
            $this->paychan = 1; // Ordinary post
        } elseif ($this->obj_ref->mode_reglement_id == '4') { // cash
            $this->payid   = 10;
            $this->paychan = 9; // Personal (face-to-face)
        } elseif ($this->obj_ref->mode_reglement_id == '3') { // prelevement
            $this->payid   = 31;
            $this->paychan = 5; // S.W.I.F.T.
        } elseif ($this->obj_ref->mode_reglement_id == '2') { // virment
            $this->payid   = 30;
            $this->paychan = 5; // S.W.I.F.T.
        } elseif ($this->obj_ref->mode_reglement_id == '2') { // paiement interbancaire
            $this->payid   = 42;
            $this->paychan = 5; // S.W.I.F.T.
        }


        // TODO MAnage DELETE Object by action trigger
        switch ($this->obj_ref->statut) {
            case Facture::STATUS_ABANDONED:
                $this->MessageFunctionCoded = '1';
                break;

            case Facture::STATUS_VALIDATED:
                $this->MessageFunctionCoded = '9';
                break;

// 			case Facture::TYPE_REPLACEMENT:
// 				$this->MessageFunctionCoded = '7'; 
// 			break;
        }
    }

    /**
      @fn header()
      @brief
      @return
     */
    function header()
    {
        global $conf;
        $this->header = array
            (
            //
            $this->GetUNB(),
            // MESSAGE HEADER
            // Mandatory Max repeat: 1
            // A service segment starting and uniquely identifying a message.
            $this->GetUNH(),
            //BGM M 1
            // BEGINNING OF MESSAGE
            // Mandatory Max repeat: 1
            // A segment by which the sender must uniquely identify the order by
            $this->GetBGM(),
            array(
                "DTM",
                make_DTM_time($this->obj_ref->date) // created date
        ));
        // not used by PSA
        if (!isset($this->custom->bypass_shipping_date) || isset($this->custom->bypass_shipping_date) && $this->custom->bypass_shipping_date == FALSE) {
            $shipping = $this->findDateShippingInLinkedObj();
            if ($shipping != FALSE) {
                $this->header = array_merge($this->header,
                    array(array(
                        "DTM",
                        make_DTM_time($shipping, "35") // date transmission OF PRODUCTS !!
                )));
            }
        }
        // not used by PSA
        if (!isset($this->custom->bypass_paiement_terms) || isset($this->custom->bypass_paiement_terms) && $this->custom->bypass_paiement_terms == FALSE) {
            $this->header = array_merge($this->header,
                array
                (array(
                    "PAI",
                    array(
                        // 4439 : Payment conditions, coded
                        //ConditionalType: anLength: 0..3
                        "{$this->paycond}" //
                        // 4431 Payment guarantee, coded
                        // ConditionalType: anLength: 0..3
                        , "20"
                        // 4461 Payment means, coded
                        // ConditionalType: anLength: 0..3
                        , "{$this->payid}" //
                        // 1131 Code list qualifier
                        // ConditionalType: anLength: 0..3
                        // 3055 Code list responsible agency, coded
                        // ConditionalType: anLength: 0..3
                        // 4435 Payment channel, coded
                        // ConditionalType: anLength: 0..3
                        , "{$this->paychan}"
                    )
            )));
        }
        // used by PSA
        if (isset($this->custom->{'FTX+AAI'})) {
            $this->header = array_merge($this->header,
                array
                (array(
                    "FTX",
                    "AAI",
                    "",
                    "",
                    array(
                        substr($this->custom->{'FTX+AAI'}['vat_cond'], 0, 69),
                        substr($this->custom->{'FTX+AAI'}['disc_cond'], 0, 69),
                        substr($this->custom->{'FTX+AAI'}['penal_cond'], 0, 69),
                        substr($this->custom->{'FTX+AAI'}['penal_cond'], 70, 139)
            ))));
        }
        // used by PSA
        if (isset($this->custom->{'FTX+REG'}) && $this->custom->{'FTX+REG'}) {
            $this->header = array_merge($this->header,
                array
                (array(
                    "FTX",
                    "REG",
                    "",
                    "",
                    array(
                        $this->mysoc->nom, // nom de la boite
                        ($this->mysoc->forme_juridique_code == 54) ? "SARL" : "Societe", // forme juridique
                        $this->mysoc->capital." EUR", //capital
                        $this->mysoc->siret //capital
                    )
            )));
        }

        $this->header = array_merge($this->header, $this->GetExternalAddress('BILLING', "BY", false, $this->obj_ref) );
        //$this->header = array_merge($this->header, array($this->GetNAD("BY", $this->setup->user->edi_num, $this->obj_ref->thirdparty, false)));


        $contactFacturation = $this->obj_ref->getIdBillingContact();
        foreach ($contactFacturation as $idContact) {
            $contact = new Contact($this->db);
            $contact->fetch($idContact);
            $thirdParty = new Societe($this->db);
            $thirdParty->fetch($contact->socid);
            $this->header = array_merge($this->header,
                array(array(
                    "RFF", array(
                        "VA",
                        remove_space($thirdParty->tva_intra) // my num tva
                    ))
            ));
        }

//        $this->header = array_merge($this->header,
//            array(array(
//                "RFF", array(
//                    "VA",
//                    remove_space($this->obj_ref->thirdparty->tva_intra) // my num tva
//                )),
////             array(
////                 "RFF", array(
////                     "GN",
////                     $this->obj_ref->thirdparty->idprof1 // SIREN client
////                 )
////             ),
////             array(
////                 "RFF", array(
////                     "VA",
////                     remove_space($this->mysoc->tva_intra) // mon num tva
////                 )
////             ),
////             array(
////                 "RFF", array(
////                     "GN",
////                     remove_space($this->mysoc->idprof1) // mon SIREN
////                 )
////             ),
////             array(
////                 "RFF", array(
////                     "XA",
////                     $this->mysoc->idprof4 // mon RCS
////                 )
////             ),
//        ));


        $seller_type  = (isset($this->custom->seller_SE) && $this->custom->seller_SE) ? 'SE' : 'SU';
        $this->header = array_merge($this->header, array($this->GetNAD($seller_type, $conf->global->EDI_UNIQNUM, $this->mysoc, false)));
        


        if (isset($this->custom->supplier_code)) {
            $this->header = array_merge($this->header,
                array(
                array(
                    "RFF", array(
                        "ADE",
                        $this->custom->supplier_code
                    )
                )
                )
            );
        }

        $this->header = array_merge($this->header,
            array(
            array(
                "RFF",
                array(
                    "VA",
                    remove_space($this->mysoc->tva_intra) // num tva perso
                ),
            ),
//            array(
//                "RFF", array(
//                    "FC",
//                    remove_space($this->mysoc->idprof1) // siret
//                )
//            )
            )
        );
        if (isset($this->custom->headquarter)) {
            $this->header = array_merge($this->header,
                array(
                array(
                    "NAD",
                    "FH",
                    "", $this->custom->headquarter
                )
            ));
        }


        // if it s related to an order
//         if (false) {
//             $this->header = array_merge($this->header, array(
//                 "RFF",
//                 array(
//                     "ON", // order number
//                     "123456" // order number here
//                 )
//                     ), array(
//                 "DTM",
//                 make_DTM_time("2015-11-25 15:51:00", "171") // date order
//             ));
//         }

        $this->header = array_merge($this->header, $this->GetExternalAddress('SHIPPING', "PE", false));
        $this->header = array_merge($this->header, $this->GetExternalAddress('BILLING', "IV", false));


        $this->header = array_merge($this->header,
            array(
//             array(
//                 "NAD",
//                 "RE", // paied to
//                 array(
//                     $this->make_ean13_ILN($this->company->nom),
//                     "",
//                     "9"
//                 )
//             ),
//             array(
//                 "NAD",
//                 "DP", // payed by
//                 array(
//                     $this->setup->user->edi_num,
//                     "",
//                     "9"
//                 )
//             ),
            array(
                "CUX", //devise
                array(
                    "2",
                    $this->obj_ref->multicurrency_code,
                    "4" // home currencie
                )
            ),
//             array(
//                 "PAT",
//                 "3" // paiement at predefined date
//             ),
//             array(
//                 "DTM",
//                 make_DTM_time($this->obj_ref->date_lim_reglement, "13") // paiement limit date
//             ),
//             array(
//                 "PAT",
//                 "22" // discount/remise
//             ),
//             array(
//                 "PCD",
//                 array(
//                     "12", // remise/discount
//                     "0", // taux d'escompte
//                     "13" // apply to invoice
//                 )
//             ),
            array(
                "PAT",
                "1", // penality
//                '',
//                array(
//                    '5'
//                    , ''
//                    , 'D'
//                    , '60' // 60 jours
//                )
            ),
            array(
                "DTM",
                make_DTM_time($this->obj_ref->date_lim_reglement, 140) // limite de paiement
            )
//            array(
//                "PCD",
//                array(
//                    "7", // penality
//                    "3", // 3,5 x taux légal en vigueur
//                    "13" // apply to invoice
//                )
//            )
        ));
    }

    /**
      @fn message()
      @brief
      @return none
     */
    function message()
    {
        global $conf, $db;

        //load specific files acobal
        if (isset($conf->global->MAIN_MODULE_CLIACOBAL) && $conf->global->MAIN_MODULE_CLIACOBAL) {
            $sql                = "SELECT rowid, fk_product , ref FROM ".MAIN_DB_PREFIX."custom_product_ref WHERE fk_soc = ".$this->obj_ref->thirdparty->id;
            $custom_product_ref = [];
            $resql              = $db->query($sql);
            if ($resql) {
                while ($obj = $db->fetch_object($resql)) {
                    $custom_product_ref[$obj->fk_product] = $obj->ref;
                }
            }
        }


        for ($i = 0; $i < count($this->obj_ref->lines); $i++) { // not sure if first is 0 or 1
            $line = $this->obj_ref->lines[$i];
            if (isset($this->tax[$line->tva_tx])) { // we had totals by tva rate
                $this->tax[$line->tva_tx] += $line->total_tva;
            } else {
                $this->tax[$line->tva_tx] = $line->total_tva;
            }



            // Complet data
            if ($line->fk_product > 0) {
                $Prod                    = new \Product($this->db);
                $Prod->fetch($line->fk_product);
                $line->barcode_type      = $Prod->barcode_type;
                $line->barcode_type_code = $Prod->barcode;
            }


            if (isset($this->custom->use_client_ref) && $this->custom->use_client_ref) {// ref interne
                $tmpRef = $line->ref_ext;
                // specific acobal
                if (isset($conf->global->MAIN_MODULE_CLIACOBAL) && $conf->global->MAIN_MODULE_CLIACOBAL && $line->fk_product > 0 && isset($custom_product_ref[$line->fk_product])) {
                    $tmpRef = $custom_product_ref[$line->fk_product];
                }

                $ref = array(
                    $tmpRef, // code barre produit
                    "IN", // Internal system
                );
            } else {
                //Ref standard
                $ref = array(
                    $line->barcode_type_code, // code barre produit
                    "EN", // International Article Numbering Association
                    "", // code product 128 -> service
                    "9" // organism responsible of SN : 9 -> EAN (International Article Numbering association)
                );
            }


            $this->message[$i] = array(
                array(
                    "LIN", // product row
                    $i + 1, // line number !!!
                    "", // action 1-> add a line
                    $ref
                ),
                array(
                    "PIA", // product row
                    "1", // Product identification
                    "{$line->product_ref}", // 
                    array(
                        "SA" // Supplier's article number
                    )
                ),
//                 array(
//                     "PIA", // product row
//                     "5", // Product identification
//                     "{$line->product_ref}", // 
//                     array(
//                         "BP" // Buyer's part number
//                     )
//                 ),
                array(
                    "IMD",
//                    "E", //Free-form short description
//                    "ANM", // pas standard D96A ...
                    "",
                    "",
                    array(
                        "",
                        "",
                        "",
                        ($line->libelle) ? $line->libelle : $line->desc // description courte du produit
                    )
                )
            );




            if (isset($Prod->weight)) {
                $this->message[$i] = array_merge($this->message[$i],
                    array(
                    array(
                        "MEA",
                        "PD",
                        "AAL",
                        array(
                            measuring_units_string($Prod->weight_units, 'weight')),
                        $Prod->weight * $line->qty,
                    )
                ));
            }

            $qta  = array(
                "47", // Invoiced quantity 
                $line->qty // quantity
            );
            $unit = 'PCE';
            if ($conf->global->PRODUCT_USE_UNITS) {
                $unit  = $line->getLabelOfUnit('short');
                if (strlen($unit)) $qta[] = $unit;
            }
            $this->message[$i] = array_merge($this->message[$i], array(
                array(
                    "QTY",
                    $qta
            )));
            if ($line->product_type == 0) {
                $this->message[$i] = array_merge($this->message[$i],
                    array(
                    array(
                        "ALI",
                        isset($Prod->country_code) && !empty($Prod->country_code) ? $Prod->country_code : "FR" // origine du produit ????
                )));
            }
            $this->message[$i] = array_merge($this->message[$i],
                array(
                array(
                    "MOA", //monetary amount
                    array(
                        "203", // Line item amount
                        format_number($line->multicurrency_total_ht) // total price for this product (D3E include)
                    )
                ),
//                 array(
//                     "PRI", //brut price
//                     array(
//                         "AAB", // price without discount
//                         format_number($line->subprice), // price in catalog 12
//                         "", // Price type : PE -> per each
//                         "", // Price type qualifier recommand (AAA -> Reference price)
//                         "1", // unity quantity 1 (ex sold by 100ml here goes 100)
//                         "PCE" // units
//                     )
//                 ),
                array(
                    "PRI", //net price
                    array(
                        "AAA", // price WITH discount AND D3E tax
                        format_number($line->subprice), // price 10 + 25 D3E
                        "", // Price type : PE -> per each
                        "", // Price type qualifier recommand (AAA -> Reference price)
                        "1", // unity quantity 1 (ex sold by 100ml here goes 100)
                        $unit // units
                    )
                ),
            ));
//                  array(
//                     "PRI", //TTC
//                     array(
//                         "AAC", // price WITH discount AND D3E tax
//                         format_number($line->total_ttc /$line->qty ), // price 10 + 25 D3E
//                         "", // Price type : PE -> per each
//                         "", // Price type qualifier recommand (AAA -> Reference price)
//                         "1", // unity quantity 1 (ex sold by 100ml here goes 100)
//                         "PCE" // units
//                     )
//                 ),

            if (isset($this->obj_ref->linkedObjects['shipping'])) {
                foreach ($this->obj_ref->linkedObjects['shipping'] as $id => $obj) {
                    // find linked delivery
                    $obj->fetchObjectLinked($obj->id,'shipping',null,'delivery');
                    // add delivery to EDI doc
                    foreach ($obj->linkedObjects['delivery'] as $objsdelivery){
                    $this->message[$i] = array_merge($this->message[$i],
                        array(
                        array(
                            // Force put ref customer order
                            "RFF", array(
                                "AAK",
                                $objsdelivery->ref
                            )
                        ),
                        array(
                            // Force put ref customer order
                            "DTM", make_DTM_time($objsdelivery->date_creation, "171")
                        )
                    ));
                    }
                    
                }
            }
            if (isset($this->obj_ref->linkedObjects['commande'])) {
                foreach ($this->obj_ref->linkedObjects['commande'] as $id => $obj) {
                    $this->message[$i] = array_merge($this->message[$i],
                        array(
                        array(
                            // Force put ref customer order
                            "RFF", array(
                                "ON",
                                $obj->ref_client
                            )
                        ),
                        array(
                            // Force put ref customer order
                            "DTM", make_DTM_time($obj->date_commande, "171")
                        )
                    ));
                }
            } else {
                $this->message[$i] = array_merge($this->message[$i],
                    array(
                    array(
                        // Force put ref customer order
                        "RFF", array(
                            "ON",
                            $this->obj_ref->ref_client
                        )
                )));
            }
            $this->message[$i] = array_merge($this->message[$i],
                array(
                array(
                    "TAX",
                    "7", //indicate it s a tax, not a fee
                    "VAT", // it s a TVA
                    "", // not use for  VAT
                    "", // not use for VAT
                    array(
                        "", // not use for VAT
                        "", // not use for VAT
                        "", // not use for VAT
                        format_number($line->tva_tx) // taux de TVA
                    )
                )
                )
            );

            //NAD pour acobal
            if (isset($this->custom->precise_NAD_product) && $this->custom->precise_NAD_product) {
                if (isset($this->obj_ref->linkedObjects['shipping'])) {
                    foreach ($this->obj_ref->linkedObjects['shipping'] as $id => $obj) {
                        $this->message[$i] = array_merge($this->message[$i],
                            array(
                            $this->GetNAD('CN', $conf->global->EDI_UNIQNUM, $this->mysoc, false),
                            array('LOC', 11, $obj->thirdparty->array_options['options_clefedi'])
                        ));
                    }
                }
            }
            if ($line->remise_percent != 0) {
                $this->message[$i] = array_merge($this->message[$i],
                    array(
                    array(
                        "ALC", //ALLOWANCE OR CHARGE
                        "A", //Allowance / remise
                        "",
                        "2", // remise sur facture
                        "1", // computed on product raw
                        array(
                            "TD", //type de remise : Trade discount / remise négociée
                            "",
                            "",
                            "REMISE" // description
                        )
                    ),
                    array(
                        "MOA", // discount details 
                        array(
                            "204", // inform we will provide a discount/allowance
                            format_number($line->remise_percent) // discount per unit
                        )
                )));
            }


//                var_dump($line, $Prod);
//                die();
//
//        var_dump($obj_ref->linkedObjects['shipping'][5480]->lines,$obj_ref->lines);die();
//             if (false) { //DEE tax or other
//                 $this->message[$i] = array_merge($this->message[$i], array(
//                     array(// DEEE tax
//                         "ALC", //ALLOWANCE OR CHARGE
//                         "C", // charge
//                         "3001000002459", // code of D3E (tax mobilier is 3001000002671, tax ammeublement 3001000002046)
//                         "2", // apply on invoice
//                         "1", // computed on product raw
//                         array(
//                             "TX", // it s a tax
//                             "",
//                             "",
//                             "TAXE DEEE" // description
//                         )
//                     ),
//                     array(
//                         "MOA", // taxe DEEE details 
//                         array(
//                             "23", // inform we will provide a Charge amount
//                             "0.25" // D3E tax per unit
//                         )
//                     ),
//                     array(
//                         "TAX",
//                         "7", //indicate it s a tax, not a fee
//                         "VAT", // it s a TVA
//                         "", // not use for  VAT
//                         "", // not use for VAT
//                         array(
//                             "", // not use for VAT
//                             "", // not use for VAT
//                             "", // not use for VAT
//                             "20.0" // taux de TVA !!!!!!! la tva pour la tax mobilier est à 5.5%
//                         )
//                     )
//                 ));
//             }
        }
    }

    /**
      @fn trailer()
      @brief
      @return none
     */
    function trailer()
    {
        $this->trailer = array(
            array(
                "UNS", // fin de section
                "S" // Detail/summary section separation
            ),
            // C270 CONTROL
            //   Mandatory
            // Control total for checking integrity of a message or part of a message.
            array(
                "CNT", // fin de section
                array(
                    "2", // Detail/summary section separation
                    count($this->message)
                )
            ),
            ///////////////////////////////////// totals //////////////////////////////////////
            array(
                "MOA", //total HT
                array(
                    "125", // Taxable amount
                    format_number($this->obj_ref->total_ht) // amount excluding taxes
                )
            ),
            array(
                "MOA", //total VAT
                array(
                    "124", // Taxe amount
                    format_number($this->obj_ref->total_tva) // amount taxes
                )
            ),
//             array(
//                 "MOA", //total TTC
//                 array(
//                     "128", // Total amount
//                     format_number($this->obj_ref->total_ttc) // total amount including taxes
//                 )
        );
        foreach ($this->tax as $rate => $total) {
            $this->trailer = array_merge($this->trailer,
                array(
                array(// detail per tva rate : 20%
                    "TAX",
                    "7", //indicate it s a tax, not a fee
                    "VAT", // it s a TVA
                    "", // not use for  VAT
                    "", // not use for VAT
                    array(
                        "", // not use for VAT
                        "", // not use for VAT
                        "", // not use for VAT
                        format_number($rate) // taux de TVA !!!!!!! la tva pour la tax mobilier est à 5.5%
                    )
                ),
                array(
                    "MOA", // total tva 20%
                    array(
                        "124", // inform we will provide a tax
                        format_number($total) // TVA 20% total tax
                    )
            )));
        }
//         if (false) { // reserved for dEEE tax or other
//             $this->trailer = array_merge($this->trailer, array(
//                 array(// TOTAL DEEE tax
//                     "ALC", //ALLOWANCE OR CHARGE
//                     "C", // charge
//                     "3001000002459", // code of D3E (tax mobilier is 3001000002671, tax ammeublement 3001000002046)
//                     "2", // apply on invoice
//                     "1", // computed on product raw
//                     array(
//                         "TX", // it s a tax
//                         "",
//                         "",
//                         "TAXE DEEE" // description
//                     )
//                 ),
//                 array(
//                     "MOA", // taxe DEEE details 
//                     array(
//                         "23", // inform we will provide a Charge amount
//                         "2.50" // D3E tax per unit
//                     )
//                 )
//             ));
//         }
    }
    /* derivated from wordpress */

    function remove_accents($string)
    {
        if (!preg_match('/[\x80-\xff]/', $string)) {
            return $string;
        }

        if (mb_detect_encoding($string, 'UTF-8', true)) {
            $chars = array(
                // Decompositions for Latin-1 Supplement
                'ª' => 'a',
                'º' => 'o',
                'À' => 'A',
                'Á' => 'A',
                'Â' => 'A',
                'Ã' => 'A',
                'Ä' => 'A',
                'Å' => 'A',
                'Æ' => 'A',
                'Ç' => 'C',
                'È' => 'E',
                'É' => 'E',
                'Ê' => 'E',
                'Ë' => 'E',
                'Ì' => 'I',
                'Í' => 'I',
                'Î' => 'I',
                'Ï' => 'I',
                'Ð' => 'D',
                'Ñ' => 'N',
                'Ò' => 'O',
                'Ó' => 'O',
                'Ô' => 'O',
                'Õ' => 'O',
                'Ö' => 'O',
                'Ù' => 'U',
                'Ú' => 'U',
                'Û' => 'U',
                'Ü' => 'U',
                'Ý' => 'Y',
                'Þ' => 'T',
                'ß' => 's',
                'à' => 'a',
                'á' => 'a',
                'â' => 'a',
                'ã' => 'a',
                'ä' => 'a',
                'å' => 'a',
                'æ' => 'a',
                'ç' => 'c',
                'è' => 'e',
                'é' => 'e',
                'ê' => 'e',
                'ë' => 'e',
                'ì' => 'i',
                'í' => 'i',
                'î' => 'i',
                'ï' => 'i',
                'ð' => 'd',
                'ñ' => 'n',
                'ò' => 'o',
                'ó' => 'o',
                'ô' => 'o',
                'õ' => 'o',
                'ö' => 'o',
                'ø' => 'o',
                'ù' => 'u',
                'ú' => 'u',
                'û' => 'u',
                'ü' => 'u',
                'ý' => 'y',
                'þ' => 't',
                'ÿ' => 'y',
                'Ø' => 'O',
                // Decompositions for Latin Extended-A
                'Ā' => 'A',
                'ā' => 'a',
                'Ă' => 'A',
                'ă' => 'a',
                'Ą' => 'A',
                'ą' => 'a',
                'Ć' => 'C',
                'ć' => 'c',
                'Ĉ' => 'C',
                'ĉ' => 'c',
                'Ċ' => 'C',
                'ċ' => 'c',
                'Č' => 'C',
                'č' => 'c',
                'Ď' => 'D',
                'ď' => 'd',
                'Đ' => 'D',
                'đ' => 'd',
                'Ē' => 'E',
                'ē' => 'e',
                'Ĕ' => 'E',
                'ĕ' => 'e',
                'Ė' => 'E',
                'ė' => 'e',
                'Ę' => 'E',
                'ę' => 'e',
                'Ě' => 'E',
                'ě' => 'e',
                'Ĝ' => 'G',
                'ĝ' => 'g',
                'Ğ' => 'G',
                'ğ' => 'g',
                'Ġ' => 'G',
                'ġ' => 'g',
                'Ģ' => 'G',
                'ģ' => 'g',
                'Ĥ' => 'H',
                'ĥ' => 'h',
                'Ħ' => 'H',
                'ħ' => 'h',
                'Ĩ' => 'I',
                'ĩ' => 'i',
                'Ī' => 'I',
                'ī' => 'i',
                'Ĭ' => 'I',
                'ĭ' => 'i',
                'Į' => 'I',
                'į' => 'i',
                'İ' => 'I',
                'ı' => 'i',
                'Ĳ' => 'I',
                'ĳ' => 'i',
                'Ĵ' => 'J',
                'ĵ' => 'j',
                'Ķ' => 'K',
                'ķ' => 'k',
                'ĸ' => 'k',
                'Ĺ' => 'L',
                'ĺ' => 'l',
                'Ļ' => 'L',
                'ļ' => 'l',
                'Ľ' => 'L',
                'ľ' => 'l',
                'Ŀ' => 'L',
                'ŀ' => 'l',
                'Ł' => 'L',
                'ł' => 'l',
                'Ń' => 'N',
                'ń' => 'n',
                'Ņ' => 'N',
                'ņ' => 'n',
                'Ň' => 'N',
                'ň' => 'n',
                'ŉ' => 'n',
                'Ŋ' => 'N',
                'ŋ' => 'n',
                'Ō' => 'O',
                'ō' => 'o',
                'Ŏ' => 'O',
                'ŏ' => 'o',
                'Ő' => 'O',
                'ő' => 'o',
                'Œ' => 'O',
                'œ' => 'o',
                'Ŕ' => 'R',
                'ŕ' => 'r',
                'Ŗ' => 'R',
                'ŗ' => 'r',
                'Ř' => 'R',
                'ř' => 'r',
                'Ś' => 'S',
                'ś' => 's',
                'Ŝ' => 'S',
                'ŝ' => 's',
                'Ş' => 'S',
                'ş' => 's',
                'Š' => 'S',
                'š' => 's',
                'Ţ' => 'T',
                'ţ' => 't',
                'Ť' => 'T',
                'ť' => 't',
                'Ŧ' => 'T',
                'ŧ' => 't',
                'Ũ' => 'U',
                'ũ' => 'u',
                'Ū' => 'U',
                'ū' => 'u',
                'Ŭ' => 'U',
                'ŭ' => 'u',
                'Ů' => 'U',
                'ů' => 'u',
                'Ű' => 'U',
                'ű' => 'u',
                'Ų' => 'U',
                'ų' => 'u',
                'Ŵ' => 'W',
                'ŵ' => 'w',
                'Ŷ' => 'Y',
                'ŷ' => 'y',
                'Ÿ' => 'Y',
                'Ź' => 'Z',
                'ź' => 'z',
                'Ż' => 'Z',
                'ż' => 'z',
                'Ž' => 'Z',
                'ž' => 'z',
                'ſ' => 's',
                // Decompositions for Latin Extended-B
                'Ș' => 'S',
                'ș' => 's',
                'Ț' => 'T',
                'ț' => 't',
                // Euro Sign
                '€' => 'E',
                // GBP (Pound) Sign
                '£' => '',
                // Vowels with diacritic (Vietnamese)
                // unmarked
                'Ơ' => 'O',
                'ơ' => 'o',
                'Ư' => 'U',
                'ư' => 'u',
                // grave accent
                'Ầ' => 'A',
                'ầ' => 'a',
                'Ằ' => 'A',
                'ằ' => 'a',
                'Ề' => 'E',
                'ề' => 'e',
                'Ồ' => 'O',
                'ồ' => 'o',
                'Ờ' => 'O',
                'ờ' => 'o',
                'Ừ' => 'U',
                'ừ' => 'u',
                'Ỳ' => 'Y',
                'ỳ' => 'y',
                // hook
                'Ả' => 'A',
                'ả' => 'a',
                'Ẩ' => 'A',
                'ẩ' => 'a',
                'Ẳ' => 'A',
                'ẳ' => 'a',
                'Ẻ' => 'E',
                'ẻ' => 'e',
                'Ể' => 'E',
                'ể' => 'e',
                'Ỉ' => 'I',
                'ỉ' => 'i',
                'Ỏ' => 'O',
                'ỏ' => 'o',
                'Ổ' => 'O',
                'ổ' => 'o',
                'Ở' => 'O',
                'ở' => 'o',
                'Ủ' => 'U',
                'ủ' => 'u',
                'Ử' => 'U',
                'ử' => 'u',
                'Ỷ' => 'Y',
                'ỷ' => 'y',
                // tilde
                'Ẫ' => 'A',
                'ẫ' => 'a',
                'Ẵ' => 'A',
                'ẵ' => 'a',
                'Ẽ' => 'E',
                'ẽ' => 'e',
                'Ễ' => 'E',
                'ễ' => 'e',
                'Ỗ' => 'O',
                'ỗ' => 'o',
                'Ỡ' => 'O',
                'ỡ' => 'o',
                'Ữ' => 'U',
                'ữ' => 'u',
                'Ỹ' => 'Y',
                'ỹ' => 'y',
                // acute accent
                'Ấ' => 'A',
                'ấ' => 'a',
                'Ắ' => 'A',
                'ắ' => 'a',
                'Ế' => 'E',
                'ế' => 'e',
                'Ố' => 'O',
                'ố' => 'o',
                'Ớ' => 'O',
                'ớ' => 'o',
                'Ứ' => 'U',
                'ứ' => 'u',
                // dot below
                'Ạ' => 'A',
                'ạ' => 'a',
                'Ậ' => 'A',
                'ậ' => 'a',
                'Ặ' => 'A',
                'ặ' => 'a',
                'Ẹ' => 'E',
                'ẹ' => 'e',
                'Ệ' => 'E',
                'ệ' => 'e',
                'Ị' => 'I',
                'ị' => 'i',
                'Ọ' => 'O',
                'ọ' => 'o',
                'Ộ' => 'O',
                'ộ' => 'o',
                'Ợ' => 'O',
                'ợ' => 'o',
                'Ụ' => 'U',
                'ụ' => 'u',
                'Ự' => 'U',
                'ự' => 'u',
                'Ỵ' => 'Y',
                'ỵ' => 'y',
                // Vowels with diacritic (Chinese, Hanyu Pinyin)
                'ɑ' => 'a',
                // macron
                'Ǖ' => 'U',
                'ǖ' => 'u',
                // acute accent
                'Ǘ' => 'U',
                'ǘ' => 'u',
                // caron
                'Ǎ' => 'A',
                'ǎ' => 'a',
                'Ǐ' => 'I',
                'ǐ' => 'i',
                'Ǒ' => 'O',
                'ǒ' => 'o',
                'Ǔ' => 'U',
                'ǔ' => 'u',
                'Ǚ' => 'U',
                'ǚ' => 'u',
                // grave accent
                'Ǜ' => 'U',
                'ǜ' => 'u',
            );

            // Used for locale-specific rules
            $chars['Ä']   = 'A';
            $chars['ä']   = 'a';
            $chars['Ö']   = 'O';
            $chars['ö']   = 'o';
            $chars['Ü']   = 'U';
            $chars['ü']   = 'u';
            $chars['ß']   = 's';
            $chars['Æ']   = 'A';
            $chars['æ']   = 'a';
            $chars['Ø']   = 'O';
            $chars['ø']   = 'o';
            $chars['Å']   = 'A';
            $chars['å']   = 'a';
            $chars['l·l'] = 'l';
            $chars['Đ']   = 'D';
            $chars['đ']   = 'd';

            $string = strtr($string, $chars);
        } else {
            $chars       = array();
            // Assume ISO-8859-1 if not UTF-8
            $chars['in'] = "\x80\x83\x8a\x8e\x9a\x9e"
                ."\x9f\xa2\xa5\xb5\xc0\xc1\xc2"
                ."\xc3\xc4\xc5\xc7\xc8\xc9\xca"
                ."\xcb\xcc\xcd\xce\xcf\xd1\xd2"
                ."\xd3\xd4\xd5\xd6\xd8\xd9\xda"
                ."\xdb\xdc\xdd\xe0\xe1\xe2\xe3"
                ."\xe4\xe5\xe7\xe8\xe9\xea\xeb"
                ."\xec\xed\xee\xef\xf1\xf2\xf3"
                ."\xf4\xf5\xf6\xf8\xf9\xfa\xfb"
                ."\xfc\xfd\xff";

            $chars['out'] = 'EfSZszYcYuAAAAAACEEEEIIIINOOOOOOUUUUYaaaaaaceeeeiiiinoooooouuuuyy';

            $string              = strtr($string, $chars['in'], $chars['out']);
            $double_chars        = array();
            $double_chars['in']  = array("\x8c", "\x9c", "\xc6", "\xd0", "\xde", "\xdf", "\xe6", "\xf0", "\xfe");
            $double_chars['out'] = array('O', 'o', 'A', 'D', 'T', 's', 'a', 'd', 't');
            $string              = str_replace($double_chars['in'], $double_chars['out'], $string);
        }

        return $string;
    }
}