<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017 Mikael Carlavan <contact@mika-carl.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       htdocs/fraistransport/admin/setup.php
 *  \ingroup    fraistransport
 *  \brief      Admin page
 */


$res=@include("../../main.inc.php");                   // For root directory
if (! $res) $res=@include("../../../main.inc.php");    // For "custom" directory

// Libraries
require_once DOL_DOCUMENT_ROOT . "/core/lib/admin.lib.php";
dol_include_once("/fraistransport/lib/fraistransport.lib.php");

// Translations
$langs->load("fraistransport@fraistransport");

// Access control
if (! $user->admin) accessforbidden();

// Parameters
$action = GETPOST('action', 'alpha');


$form = new Form($db);

$formconfirm = '';

/*
 * Actions
 */

include DOL_DOCUMENT_ROOT.'/core/actions_setmoduleoptions.inc.php';

// Action mise a jour ou ajout d'une constante
if ($action == 'update')
{
    $constname=GETPOST('constname','alpha');
    $constvalue=(GETPOST('constvalue_'.$constname) ? GETPOST('constvalue_'.$constname) : GETPOST('constvalue'));


    $consttype=GETPOST('consttype','alpha');
    $constnote=GETPOST('constnote');
    $res = dolibarr_set_const($db,$constname,$constvalue,'chaine',0,$constnote,$conf->entity);

    if (! $res > 0) $error++;

    if (! $error)
    {
        if ($constname == 'TRANSPORTER_ZONES_MODE') {
            $formquestion = array(
                array('type' => 'other','name' => 'selectfk_pays','label' => $langs->trans("Country"),'value' => $form->select_country($mysoc->country_id, 'fk_pays', '', '', 'maxwidth200 maxwidthonsmartphone')
                )
            );
    
            if ($constvalue == 'departements') {
                $title = $langs->trans('DescUseDepartments');
                $message = $langs->trans('ConfirmUseDepartments');
                $confirm_action = 'confirm_insert_departments';
            } else if ($constvalue == 'regions') {
                $title = $langs->trans('DescUseRegions');
                $message = $langs->trans('ConfirmUseRegions');
                $confirm_action = 'confirm_insert_regions';
            } else {
                $title = $langs->trans('DescUsePays');
                $message = $langs->trans('ConfirmUsePays');
                $confirm_action = 'confirm_insert_pays';
    
                $formquestion = '';
            } 
    
            $formconfirm = $form->formconfirm($_SERVER["PHP_SELF"] . '?id=' . $object->id, $title, $message, $confirm_action, $formquestion, 'yes', 1);  
        }
        else
        {
            setEventMessages($langs->trans("SetupSaved"), null, 'mesgs');
        }

     }
    else
    {
        setEventMessages($langs->trans("Error"), null, 'errors');
    }
}

//MAJ Taxes sur le transport
if ($action == 'updatetransporttax')
{
    $constname=GETPOST('constname','alpha');
    $constvalue=(GETPOST('constvalue_'.$constname) ? GETPOST('constvalue_'.$constname) : GETPOST('constvalue'));


    $consttype=GETPOST('consttype','alpha');
    $constnote=GETPOST('constnote');
    $res = dolibarr_set_const($db,$constname,$constvalue,'chaine',0,$constnote,$conf->entity);

    if (! $res > 0) $error++;

    if (! $error)
    {
        setEventMessages($langs->trans("SetupSaved"), null, 'mesgs');
    }
    else
    {
        setEventMessages($langs->trans("Error"), null, 'errors');
    }
}

//MAJ Coef sur le transport
if ($action == 'updatetransportcoef')
{
    $constname=GETPOST('constname','alpha');
    $constvalue=(GETPOST('constvalue_'.$constname) ? GETPOST('constvalue_'.$constname) : GETPOST('constvalue'));


    $consttype=GETPOST('consttype','alpha');
    $constnote=GETPOST('constnote');
    $res = dolibarr_set_const($db,$constname,$constvalue,'chaine',0,$constnote,$conf->entity);

    if (! $res > 0) $error++;

    if (! $error)
    {
        setEventMessages($langs->trans("SetupSaved"), null, 'mesgs');
    }
    else
    {
        setEventMessages($langs->trans("Error"), null, 'errors');
    }
}

//MAJ hayon en sus
if ($action == 'updatehayon')
{
    $constname=GETPOST('constname','alpha');
    $constvalue=(GETPOST('constvalue_'.$constname) ? GETPOST('constvalue_'.$constname) : GETPOST('constvalue'));
    $consttype=GETPOST('consttype','alpha');
    $constnote=GETPOST('constnote');
    $res = dolibarr_set_const($db,$constname,$constvalue,'chaine',0,$constnote,$conf->entity);
    if (! $res > 0) $error++;

    if (! $error)
    {
        setEventMessages($langs->trans("SetupSaved"), null, 'mesgs');
    }
    else
    {
        setEventMessages($langs->trans("Error"), null, 'errors');
    }
}




if ($action == 'confirm_insert_departments' && GETPOST('confirm') == 'yes')
{
    $fk_pays = GETPOST('selectfk_pays', 'int');

    $now = dol_now();

    $sql = "SELECT * FROM ".MAIN_DB_PREFIX."c_zones_departements WHERE entity = ".$conf->entity;
    $result = $db->query($sql);

    if ($result > 0)
    {
        if ($db->num_rows($result) == 0)
        {
            $sql = "INSERT INTO ".MAIN_DB_PREFIX."c_zones_departements (fk_object, datec, tms, entity) SELECT rowid, '".$db->idate($now)."', '".$db->idate($now)."', ".$conf->entity." FROM ".MAIN_DB_PREFIX."c_departements WHERE fk_region IN (SELECT code_region FROM ".MAIN_DB_PREFIX."c_regions WHERE fk_pays = ".$fk_pays.")";
            $db->query($sql);  
        }
    }        
}

if ($action == 'confirm_insert_regions' && GETPOST('confirm') == 'yes')
{
    $fk_pays = GETPOST('selectfk_pays', 'int');
    $now = dol_now();

    $sql = "SELECT * FROM ".MAIN_DB_PREFIX."c_zones_regions WHERE entity = ".$conf->entity;
    $result = $db->query($sql);

    if ($result > 0)
    {
        if ($db->num_rows($result) == 0)
        {
            $sql = "INSERT INTO ".MAIN_DB_PREFIX."c_zones_regions (fk_object, datec, tms, entity) SELECT rowid, '".$db->idate($now)."', '".$db->idate($now)."', ".$conf->entity." FROM ".MAIN_DB_PREFIX."c_regions WHERE fk_pays = ".$fk_pays;
            $db->query($sql);  
        }
    }  
}

if ($action == 'confirm_insert_pays' && GETPOST('confirm') == 'yes')
{
    $now = dol_now();

    $sql = "SELECT * FROM ".MAIN_DB_PREFIX."c_zones_pays WHERE entity = ".$conf->entity;
    $result = $db->query($sql);

    if ($result > 0)
    {
        if ($db->num_rows($result) == 0)
        {
            $sql = "INSERT INTO ".MAIN_DB_PREFIX."c_zones_pays (fk_object, datec, tms, entity) SELECT rowid, '".$db->idate($now)."', '".$db->idate($now)."', ".$conf->entity." FROM ".MAIN_DB_PREFIX."c_country";
            $db->query($sql);  
        }
    }  
}

/*
 * View
 */

llxHeader('', $langs->trans('FraisTransportSetup'));

// Subheader
$linkback = '<a href="' . DOL_URL_ROOT . '/admin/modules.php">' . $langs->trans("BackToModuleList") . '</a>';

print load_fiche_titre($langs->trans('FraisTransportSetup'), $linkback);

// Configuration header
$head = fraistransport_prepare_admin_head();
dol_fiche_head(
	$head,
	'settings',
	$langs->trans("ModuleFraisTransportName"),
	0,
	"fraistransport@fraistransport"
);

// Setup page goes here
echo $langs->trans("FraisTransportSetupPage");


print load_fiche_titre($langs->trans("FraisTransportOptions"),'','');


echo $formconfirm;

$modes = array(
    'departements' => $langs->trans('DescUseDepartments'),
    'regions' => $langs->trans('DescUseRegions'),
    'pays' => $langs->trans('DescUsePays')
);

print '<table class="noborder" width="100%">';
print '<tr class="liste_titre">';
print '<td>'.$langs->trans("Description").'</td>';
print '<td colspan="2">';
$text = $langs->trans("Value");
print $form->textwithpicto($text, $helptext, 1, 'help', '', 0, 2, 'idhelptext');
print '</td>';
print "</tr>\n";

print '<form action="'.$_SERVER["PHP_SELF"].'" method="POST">';
print '<tr class="oddeven">';
print '<td>';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
print '<input type="hidden" name="action" value="update">';
print '<input type="hidden" name="constname" value="TRANSPORTER_ZONES_MODE">';
print '<input type="hidden" name="constnote" value="">';
print $langs->trans('DescTRANSPORTER_ZONES_MODE');
print '</td>';
print '<td>';
print $form->selectarray('constvalue', $modes, $conf->global->TRANSPORTER_ZONES_MODE, 1, 0, 0, '', 0, 0, 0, '', '', 1);
print '<input type="hidden" name="consttype" value="chaine">';
print '</td>';
print '<td align="center">';
print '<input type="submit" class="button" value="'.$langs->trans("Update").'" name="Button">';
print '</td>';
print '</tr>';
print '</form>';

print '<form action="'.$_SERVER["PHP_SELF"].'" method="POST">';
print '<tr class="oddeven">';
print '<td>';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
print '<input type="hidden" name="action" value="update">';
print '<input type="hidden" name="constname" value="TRANSPORTER_PRODUCT_LABEL">';
print '<input type="hidden" name="constnote" value="">';
print $langs->trans('DescTRANSPORTER_PRODUCT_LABEL');
print '</td>';
print '<td>';
print '<input type="text" class="flat" name="constvalue" value="'.$conf->global->TRANSPORTER_PRODUCT_LABEL.'" />';
print '<input type="hidden" name="consttype" value="chaine">';
print '</td>';
print '<td align="center">';
print '<input type="submit" class="button" value="'.$langs->trans("Update").'" name="Button">';
print '</td>';
print '</tr>';
print '</form>';


//Constant tax
print '<form action="'.$_SERVER["PHP_SELF"].'" method="POST">';
print '<tr class="oddeven">';
print '<td>';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
print '<input type="hidden" name="action" value="update">';
print '<input type="hidden" name="constname" value="TRANSPORTER_TX_TAX">';
print '<input type="hidden" name="constnote" value="Taxe sur les transports en %">';
print $langs->trans('DescTRANSPORTER_TX_TAX');
print '</td>';
print '<td>';
print '<input type="text" class="flat" name="constvalue" value="'.$conf->global->TRANSPORTER_TX_TAX.'" />';
print '<input type="hidden" name="consttype" value="chaine">';
print '</td>';
print '<td align="center">';
print '<input type="submit" class="button" value="'.$langs->trans("Update").'" name="Button">';
print '</td>';
print '</tr>';
print '</form>';

//Constant coef
print '<form action="'.$_SERVER["PHP_SELF"].'" method="POST">';
print '<tr class="oddeven">';
// Show constant
print '<td>';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
print '<input type="hidden" name="action" value="update">';
print '<input type="hidden" name="constname" value="TRANSPORTER_TX_COEF">';
print '<input type="hidden" name="constnote" value="Coefficient appliqué sur les frais de transport">';
print $langs->trans('DescTRANSPORTER_TX_COEF');
print '</td>';
print '<td>';
print '<input type="text" class="flat" name="constvalue" value="'.$conf->global->TRANSPORTER_TX_COEF.'" />';
print '<input type="hidden" name="consttype" value="chaine">';
print '</td>';
print '<td align="center">';
print '<input type="submit" class="button" value="'.$langs->trans("Update").'" name="Button">';
print '</td>';
print '</tr>';
print '</form>';

print '<tr class="oddeven">';
// Show constant
print '<td>';
print $langs->trans('DescUSE_REAL_WEIGHT');
print '</td>';
print '<td colspan="2">';
if ($conf->use_javascript_ajax)
{
    print ajax_constantonoff('USE_REAL_WEIGHT');
}
else
{
    if (empty($conf->global->USE_REAL_WEIGHT))
    {
        print '<a href="'.$_SERVER['PHP_SELF'].'?action=set_USE_REAL_WEIGHT">'.img_picto($langs->trans("Disabled"),'switch_off').'</a>';
    }
    else
    {
        print '<a href="'.$_SERVER['PHP_SELF'].'?action=del_USE_REAL_WEIGHT">'.img_picto($langs->trans("Enabled"),'switch_on').'</a>';
    }
}
print '</td>';
print '</tr>';


// Use hayon
print '<tr class="oddeven">';
print '<td>';
print $langs->trans('DescUSE_HAYON');
print '</td>';
print '<td colspan="2">';
if ($conf->use_javascript_ajax)
{
    print ajax_constantonoff('USE_HAYON','','','','',1,0,1);
}
else
{
    if (empty($conf->global->USE_HAYON))
    {
        print '<a href="'.$_SERVER['PHP_SELF'].'?action=set_USE_HAYON">'.img_picto($langs->trans("Disabled"),'switch_off').'</a>';
    }
    else
    {
        print '<a href="'.$_SERVER['PHP_SELF'].'?action=del_USE_HAYON">'.img_picto($langs->trans("Enabled"),'switch_on').'</a>';

        

    }
}
print '</td>';
print '</tr>';

//Prix du hayon à utiliser
print '<form action="'.$_SERVER["PHP_SELF"].'" method="POST">';
print '<tr class="oddeven">';
print '<td>';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
print '<input type="hidden" name="action" value="update">';
print '<input type="hidden" name="constname" value="HAYON_PRICE">';
print '<input type="hidden" name="constnote" value="Prix du hayon">';
print $langs->trans('DescHAYON_PRICE');
print '</td>';
print '<td>';
print '<input type="text" class="flat" name="constvalue" value="'.$conf->global->HAYON_PRICE.'" />';
print '<input type="hidden" name="consttype" value="chaine">';
print '</td>';
print '<td align="center">';
print '<input type="submit" class="button" value="'.$langs->trans("Update").'" name="Button">';
print '</td>';
print '</tr>';
print '</form>';

print '</table>';


// Page end
dol_fiche_end();
llxFooter();
