<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017 Mikael Carlavan <contact@mika-carl.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       htdocs/fraistransport/admin/setup.php
 *  \ingroup    fraistransport
 *  \brief      Admin page
 */


$res=@include("../../main.inc.php");                   // For root directory
if (! $res) $res=@include("../../../main.inc.php");    // For "custom" directory

// Libraries
require_once DOL_DOCUMENT_ROOT . "/core/lib/admin.lib.php";
require_once DOL_DOCUMENT_ROOT . "/product/class/product.class.php";

dol_include_once("/fraistransport/lib/fraistransport.lib.php");
dol_include_once("/fraistransport/class/transporter.class.php");


// Translations
$langs->load("admin");
$langs->load("other");

$langs->load("fraistransport@fraistransport");

// Access control
if (! $user->admin) accessforbidden();

// Parameters
$action = GETPOST('action', 'alpha');
$confirm = GETPOST('confirm', 'alpha');

$rowid = GETPOST('rowid', 'int');

$search_label = GETPOST('search_label', 'alpha');
$search_ref = GETPOST('search_ref', 'alpha');

$sortfield = GETPOST("sortfield",'alpha') ? GETPOST("sortfield",'alpha') : 'c.rowid';
$sortorder = GETPOST("sortorder",'alpha') ? GETPOST("sortorder",'alpha') : 'DESC';

$param = '';
if ($search_label != '')     $param.= '&search_label='.urlencode($search_label);
if ($search_ref != '')       $param.= '&search_ref='.urlencode($search_ref);


$paramwithsearch = $param;
$paramwithsearch.= '&sortorder='.$sortorder;
$paramwithsearch.= '&sortfield='.$sortfield;

/*
 * Actions
 */
$acts[0] = "activate";
$acts[1] = "disable";
$actl[0] = img_picto($langs->trans("Disabled"),'switch_off');
$actl[1] = img_picto($langs->trans("Activated"),'switch_on');

// activate
if ($action == $acts[0])
{
    $sql = "UPDATE ".MAIN_DB_PREFIX."c_transporteurs SET active = 1 WHERE rowid='".$rowid."'";

    $result = $db->query($sql);
    if (!$result)
    {
        dol_print_error($db);
    }
}

// disable
if ($action == $acts[1])
{
    $sql = "UPDATE ".MAIN_DB_PREFIX."c_transporteurs SET active = 0 WHERE rowid='".$rowid."'";

    $result = $db->query($sql);
    if (!$result)
    {
        dol_print_error($db);
    }
}

if (GETPOST('actionadd') || GETPOST('actionmodify'))
{
    $transporter = new Transporter($db);


    if (GETPOST('actionadd'))
    {
        $transporter->label = GETPOST('label');
        $transporter->ref = GETPOST('ref');
        $transporter->out_of_range = GETPOST('out_of_range');
        $transporter->subject_to_fuel_tax = GETPOST('subject_to_fuel_tax');
        $transporter->fk_shipping_method = GETPOST('fk_shipping_method');

        $result = $transporter->create($user);
    }
    else
    {
        $transporter->fetch($rowid);
        $transporter->label = GETPOST('label');
        $transporter->ref = GETPOST('ref');
        $transporter->out_of_range = GETPOST('out_of_range');
        $transporter->subject_to_fuel_tax = GETPOST('subject_to_fuel_tax');
        $transporter->fk_shipping_method = GETPOST('fk_shipping_method');

        $result = $transporter->update($rowid, $user);        
    }
    

    if ($result < 0)
    {
        setEventMessages($langs->trans($transporter->error), null, 'errors');
    }
}

if ($action == 'confirm_delete' && $confirm == 'yes')       // delete
{
    $transporter = new Transporter($db);
    $transporter->fetch($rowid);
    $result = $transporter->delete($user);        

    if ($result < 0)
    {
        setEventMessages($langs->trans($transporter->error), null, 'errors');
    }
}

/*
 * View
 */

llxHeader('', $langs->trans('FraisTransportTransporters'));

// Subheader
$linkback = '<a href="' . DOL_URL_ROOT . '/admin/modules.php">' . $langs->trans("BackToModuleList") . '</a>';

print load_fiche_titre($langs->trans('FraisTransportTransporters'), $linkback);

// Configuration header
$head = fraistransport_prepare_admin_head();
dol_fiche_head(
	$head,
	'transporters',
	$langs->trans("ModuleFraisTransportName"),
	0,
	"fraistransport@fraistransport"
);

// Setup page goes here
echo $langs->trans("FraisTransportTransportersPage");


$form = new Form($db);
$transporter = new Transporter($db);

$out_of_ranges = array(
    Transporter::DO_NOT_TAKE => $langs->trans("TransporterDoNotTake"),
    Transporter::TAKE_MIN => $langs->trans("TransporterTakeMin"),
    Transporter::TAKE_MAX => $langs->trans("TransporterTakeMax"),
    Transporter::FORCE_TO_ZERO => $langs->trans("TransporterForceToZero")
);

$subjecttofueltax_translate = array(0 => $langs->trans("TransporterSubjectToFuelTaxNo"), 1=>$langs->trans("TransporterSubjectToFuelTaxYes"));

// Confirmation de la suppression de la ligne
if ($action == 'delete')
{
    print $form->formconfirm($_SERVER["PHP_SELF"].'?rowid='.$rowid.$paramwithsearch, $langs->trans('DeleteLine'), $langs->trans('ConfirmDeleteLine'), 'confirm_delete','',0,1);
}

print '<form action="'.$_SERVER['PHP_SELF'].'?rowid='.$rowid.'" method="POST">';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';


print '<div class="div-table-responsive-no-min">';
print '<table class="noborder" width="100%">';

// Line for title
print '<tr class="liste_titre">';
print '<td>'.$langs->trans("TransporterLabel").'</td>';
print '<td>'.$langs->trans("TransporterRef").'</td>';
print '<td>'.$langs->trans("TransporterOutOfRange").'</td>';
print '<td>'.$langs->trans("TransporterSubjectToFuelTax").'</td>';
print '<td style="min-width: 26px;"></td>';
print '<td style="min-width: 26px;"></td>';
 print '</tr>';

// Line to enter new values
print '<tr class="oddeven nodrag nodrop nohover">';

    
print '<td>';
$form->selectShippingMethod('', 'fk_shipping_method', '', 1);
print '</td>';
print '<td><input type="text" class="flat" name="ref" /></td>';
print '<td>'.$form->selectarray('out_of_range', $out_of_ranges, 0).'</td>';
print '<td>'.$form->selectarray('subject_to_fuel_tax', $subjecttofueltax_translate, 0).'</td>';

print '<td colspan="2" align="center">';
if ($action != 'edit')
{
    print '<input type="submit" class="button" name="actionadd" value="'.$langs->trans("Add").'">';
}
print '</td>';
print "</tr>";


print '</table>';
print '</div>';

print '</form>';


print '<br />';

print '<form action="'.$_SERVER['PHP_SELF'].'?rowid='.$rowid.'" method="POST">';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';


print '<div class="div-table-responsive">';
print '<table class="noborder" width="100%">';

// Title line with search boxes
print '<tr class="liste_titre_filter">';
print '<td class="liste_titre">';
print '</td>';
print '<td class="liste_titre">';
print '<input type="text" class="flat" name="search_ref" value="'.dol_escape_htmltag($search_ref).'">';
print '</td>';
print '<td class="liste_titre">';
print '</td>';           
print '<td class="liste_titre">';
print '</td>';   

print '<td class="liste_titre" colspan="3" align="right">';
print $form->showFilterAndCheckAddButtons(0);
print '</td>';
print '</tr>';

// Title of lines
print '<tr class="liste_titre">';


print getTitleFieldOfList($langs->trans("TransporterLabel"), 0, $_SERVER["PHP_SELF"], 'c.label', '', $param, "align='left'", $sortfield, $sortorder);
print getTitleFieldOfList($langs->trans("TransporterProduct"), 0, $_SERVER["PHP_SELF"], 'c.fk_product', '', $param, "align='left'", $sortfield, $sortorder);
print getTitleFieldOfList($langs->trans("TransporterOutOfRange"), 0, $_SERVER["PHP_SELF"], 'c.out_of_range', '', $param, "align='left'", $sortfield, $sortorder);
print getTitleFieldOfList($langs->trans("TransporterSubjectToFuelTax"), 0, $_SERVER["PHP_SELF"], 'c.subject_to_fuel_tax', '', $param, "align='left'", $sortfield, $sortorder);
print getTitleFieldOfList($langs->trans("TransporterActive"), 0, $_SERVER["PHP_SELF"], 'c.active', '', $param, "align='left'", $sortfield, $sortorder);
print getTitleFieldOfList('');
print getTitleFieldOfList('');


print '</tr>';


$sql = "SELECT c.rowid, c.label, sm.libelle as shipment_mode_label, c.ref, c.datec, c.fk_product, c.fk_shipping_method, c.out_of_range, c.subject_to_fuel_tax, c.active";
$sql.= " FROM ".MAIN_DB_PREFIX."c_transporteurs c";
$sql.= " LEFT JOIN ".MAIN_DB_PREFIX."c_shipment_mode sm ON sm.rowid = c.fk_shipping_method";
$sql.= " WHERE c.entity = ".$conf->entity;
$sql.= !empty($search_label) ? " AND c.label LIKE '%".$db->escape($search_label)."%'" : "";
$sql.= !empty($search_ref) ? " AND c.ref LIKE '%".$db->escape($search_ref)."%'" : "";

$sql.= " ORDER BY ".$sortfield." ".$sortorder;

$resql = $db->query($sql);
if ( $resql )
{
    $num = $db->num_rows($resql);

    if ($num)
    {
        $i = 0;
        // Lines with values
        while ($i < $num)
        {
            $obj = $db->fetch_object($resql);
            $obj->label = !empty($obj->shipment_mode_label) ? $obj->shipment_mode_label : $obj->label;

            $productstatic = new Product($db);
            $productstatic->fetch($obj->fk_product);

            print '<tr class="oddeven" id="rowid-'.$obj->rowid.'">';

            if ($action == 'edit' && ($rowid == $obj->rowid))
            {
                print '<td align="left">';
                $form->selectShippingMethod($obj->fk_shipping_method, 'fk_shipping_method', '', 1);
                print '</td>';
                print '<td align="left"><input type="text" class="flat" name="ref" value="'.$obj->ref.'" /></td>';
                print '<td align="left">'.$form->selectarray('out_of_range', $out_of_ranges, $obj->out_of_range).'</td>';
                print '<td align="left">'.$form->selectarray('subject_to_fuel_tax', $subjecttofueltax_translate, $obj->subject_to_fuel_tax).'</td>';

                print '<td align="left">&nbsp;</td>';
                print '<td colspan="2" align="center">';
                print '<input type="hidden" name="rowid" value="'.$rowid.'">';
                print '<input type="submit" class="button" name="actionmodify" value="'.$langs->trans("Modify").'">';
                print '<input type="submit" class="button" name="actioncancel" value="'.$langs->trans("Cancel").'">';
                print '</td>';
            }
            else
            {
                $url = $_SERVER["PHP_SELF"].'?sortfield='.$sortfield.'&sortorder='.$sortorder.'&rowid='.$obj->rowid;
                if ($param) $url .= '&'.$param;
                $url.='&';

                print '<td align="left">'.$obj->label.'</td>';
                print '<td align="left">'.$productstatic->getNomUrl().'</td>';
                print '<td align="left">'.$out_of_ranges[$obj->out_of_range].'</td>';
                print '<td align="left">'.$subjecttofueltax_translate[$obj->subject_to_fuel_tax].'</td>';
                print '<td align="left"><a href="'.$url.'action='.$acts[$obj->active].'">'.$actl[$obj->active].'</a></td>';

                // Modify link         
                print '<td align="center"><a class="reposition" href="'.$url.'action=edit">'.img_edit().'</a></td>';
         

                // Delete link
                print '<td align="center">';
                print '<a href="'.$url.'action=delete">'.img_delete().'</a>';
                print '</td>';

                print "</tr>\n";
            }

            $i++;
        }
    }
}
else
{
    dol_print_error($db);
}


print '</table>';
print '</div>';


print '</form>';



// Page end
dol_fiche_end();
llxFooter();
