<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017 Mikael Carlavan <contact@mika-carl.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       htdocs/fraistransport/admin/setup.php
 *  \ingroup    fraistransport
 *  \brief      Admin page
 */


$res=@include("../../main.inc.php");                   // For root directory
if (! $res) $res=@include("../../../main.inc.php");    // For "custom" directory

// Libraries
require_once DOL_DOCUMENT_ROOT . "/core/lib/admin.lib.php";
require_once DOL_DOCUMENT_ROOT . "/product/class/product.class.php";
require_once DOL_DOCUMENT_ROOT .'/core/lib/product.lib.php';
require_once DOL_DOCUMENT_ROOT .'/product/class/html.formproduct.class.php';

dol_include_once("/fraistransport/lib/fraistransport.lib.php");
dol_include_once("/fraistransport/class/weightrange.class.php");
dol_include_once("/fraistransport/class/html.form.transporters.class.php");
dol_include_once("/fraistransport/class/html.form.zones.class.php");


// Translations
$langs->load("admin");
$langs->load("other");

$langs->load("fraistransport@fraistransport");

// Access control
if (! $user->admin) accessforbidden();

// Parameters
$action = GETPOST('action', 'alpha');
$confirm = GETPOST('confirm', 'alpha');

$rowid = GETPOST('rowid', 'int');

$search_zone = GETPOST('search_zone', 'alpha');
$search_transporter = GETPOST('search_transporter', 'alpha');

$sortfield = GETPOST("sortfield",'alpha') ? GETPOST("sortfield",'alpha') : 'c.rowid';
$sortorder = GETPOST("sortorder",'alpha') ? GETPOST("sortorder",'alpha') : 'DESC';

$param = '';
if ($search_transporter != '')     $param.= '&search_transporter='.urlencode($search_transporter);
if ($search_zone != '')     $param.= '&search_zone='.urlencode($search_zone);


$paramwithsearch = $param;
$paramwithsearch.= '&sortorder='.$sortorder;
$paramwithsearch.= '&sortfield='.$sortfield;

$acts[0] = "activate";
$acts[1] = "disable";
$actl[0] = img_picto($langs->trans("Disabled"),'switch_off');
$actl[1] = img_picto($langs->trans("Activated"),'switch_on');

$act2s[0] = "setpoidsreel";
$act2s[1] = "unsetpoidsreel";

// activate
if ($action == $act2s[0])
{
    $weightrange = new WeightRange($db);
    $sql = "UPDATE ".MAIN_DB_PREFIX.$weightrange->table_element." SET poids_reel = 1 WHERE rowid='".$rowid."'";

    $result = $db->query($sql);
    if (!$result)
    {
        dol_print_error($db);
    }
}

// activate
if ($action == $act2s[1])
{
    $weightrange = new WeightRange($db);
    $sql = "UPDATE ".MAIN_DB_PREFIX.$weightrange->table_element." SET poids_reel = 0 WHERE rowid='".$rowid."'";

    $result = $db->query($sql);
    if (!$result)
    {
        dol_print_error($db);
    }
}

// activate
if ($action == $acts[0])
{
    $weightrange = new WeightRange($db);
    $sql = "UPDATE ".MAIN_DB_PREFIX.$weightrange->table_element." SET active = 1 WHERE rowid='".$rowid."'";

    $result = $db->query($sql);
    if (!$result)
    {
        dol_print_error($db);
    }
}

// disable
if ($action == $acts[1])
{
    $weightrange = new WeightRange($db);
    $sql = "UPDATE ".MAIN_DB_PREFIX.$weightrange->table_element." SET active = 0 WHERE rowid='".$rowid."'";

    $result = $db->query($sql);
    if (!$result)
    {
        dol_print_error($db);
    }
}

/*
 * Actions
 */
if (GETPOST('actionadd') || GETPOST('actionmodify'))
{
    $weightrange = new WeightRange($db);


    if (GETPOST('actionadd'))
    {
        $weightrange->fk_transporteur = GETPOST('fk_transporteur');
        $weightrange->fk_zone = GETPOST('fk_zone');
        $weightrange->poids_min = price2num(GETPOST('poids_min'));
        $weightrange->poids_max = price2num(GETPOST('poids_max'));
        $weightrange->fk_unit_min = GETPOST('fk_unit_min');
        $weightrange->fk_unit_max = GETPOST('fk_unit_max');
        $weightrange->montant = price2num(GETPOST('montant'));
        $weightrange->poids_reel = GETPOST('poids_reel');

        $result = $weightrange->create($user);
    }
    else
    {
        $weightrange->fetch($rowid);
        $weightrange->fk_transporteur = GETPOST('fk_transporteur');
        $weightrange->fk_zone = GETPOST('fk_zone');
        $weightrange->poids_min = price2num(GETPOST('poids_min'));
        $weightrange->poids_max = price2num(GETPOST('poids_max'));
        $weightrange->fk_unit_min = GETPOST('fk_unit_min');
        $weightrange->fk_unit_max = GETPOST('fk_unit_max');
        $weightrange->montant = price2num(GETPOST('montant'));
        $weightrange->active = GETPOST('active');
        $weightrange->poids_reel = GETPOST('poids_reel');

        $result = $weightrange->update($rowid, $user);        
    }
    

    if ($result < 0)
    {
        setEventMessages($langs->trans($weightrange->error), null, 'errors');
    }
}

if ($action == 'confirm_delete' && $confirm == 'yes')       // delete
{
    $weightrange = new WeightRange($db);
    $weightrange->fetch($rowid);
    $result = $weightrange->delete($user);        

    if ($result < 0)
    {
        setEventMessages($langs->trans($weightrange->error), null, 'errors');
    }
}

if ($action == 'confirm_clone' && $confirm == 'yes')       // delete
{
    $weightrange = new WeightRange($db);
    $weightrange->fetch($rowid);
    $weightrange->id = 0;
    $weightrange->fk_transporteur = GETPOST('clone_fk_transporteur');

    $result = $weightrange->create($user);        

    if ($result < 0)
    {
        setEventMessages($langs->trans($weightrange->error), null, 'errors');
    }
}

/*
 * View
 */

llxHeader('', $langs->trans('FraisTransportWeightRanges'));

// Subheader
$linkback = '<a href="' . DOL_URL_ROOT . '/admin/modules.php">' . $langs->trans("BackToModuleList") . '</a>';

print load_fiche_titre($langs->trans('FraisTransportWeightRanges'), $linkback);

// Configuration header
$head = fraistransport_prepare_admin_head();
dol_fiche_head(
	$head,
	'weightranges',
	$langs->trans("ModuleFraisTransportName"),
	0,
	"fraistransport@fraistransport"
);

// Setup page goes here
echo $langs->trans("FraisTransportWeightRangesPage");


$form = new Form($db);
$transportersform = new TransportersForm($db);
$zonesform = new ZonesForm($db);

$formproduct = new FormProduct($db);

$weightrange = new WeightRange($db);

// Confirmation de la suppression de la ligne
if ($action == 'delete')
{
    print $form->formconfirm($_SERVER["PHP_SELF"].'?rowid='.$rowid.$paramwithsearch, $langs->trans('DeleteLine'), $langs->trans('ConfirmDeleteLine'), 'confirm_delete','',0,1);
}

if ($action == 'clone')
{
    $formquestions = array();

    // Create an array for form
    $formquestions[] = array(
        'type' => 'other',
        'name' => 'clone_fk_transporteur',
        'label' => $langs->trans("SelectTransporter"),
        'value' => $transportersform->select_transporteurs('', 'clone_fk_transporteur')
    );

    print $form->formconfirm($_SERVER["PHP_SELF"].'?rowid='.$rowid.$paramwithsearch, $langs->trans('CloneLine'), $langs->trans('ConfirmCloneLine'), 'confirm_clone',$formquestions,0,1);
}

print '<form action="'.$_SERVER['PHP_SELF'].'?id='.$id.'" method="POST">';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';


print '<div class="div-table-responsive-no-min">';
print '<table class="noborder" width="100%">';

// Line for title
print '<tr class="liste_titre">';
print '<td>'.$langs->trans("WeightRangeTransporter").'</td>';
print '<td>'.$langs->trans("WeightRangeZone").'</td>';
print '<td>'.$langs->trans("WeightRangeWeightMin").'</td>';
print '<td>'.$langs->trans("WeightRangeWeightMax").'</td>';
print '<td>'.$langs->trans("WeightRangeWeightAmount").'</td>';
if ($conf->global->USE_REAL_WEIGHT) {
    print '<td>'.$langs->trans("WeightRangeWeightReal").'</td>';
}
print '<td style="min-width: 26px;"></td>';
print '<td style="min-width: 26px;"></td>';
print '<td style="min-width: 26px;"></td>';
print '</tr>';

// Line to enter new values
print '<tr class="oddeven nodrag nodrop nohover">';

print '<td>'.$transportersform->select_transporteurs('', 'fk_transporteur').'</td>';
print '<td>'.$zonesform->select_zones('', 'fk_zone').'</td>';   
print '<td>';
print '<input type="text" class="flat" name="poids_min" value="" />';
if (floatval(DOL_VERSION) >= 10)
{
    print $formproduct->select_measuring_units("fk_unit_min", "weight", '0', 0, 2);
}
else
{
    print $formproduct->select_measuring_units("fk_unit_min", "weight", '0', 0);
}
print '</td>';
print '<td>';
print '<input type="text" class="flat" name="poids_max" value="" />';
if (floatval(DOL_VERSION) >= 10)
{
    print $formproduct->select_measuring_units("fk_unit_max", "weight", '0', 0, 2);
}
else
{
    print $formproduct->select_measuring_units("fk_unit_max", "weight", '0', 0);   
}
print '</td>';
print '<td><input type="text" class="flat" name="montant" value="" /></td>';
if ($conf->global->USE_REAL_WEIGHT) {
    print '<td align="left">'.$form->selectyesno('poids_reel', 1, 1).'</td>';
}
print '<td colspan="3" align="center">';
if ($action != 'edit')
{
    print '<input type="submit" class="button" name="actionadd" value="'.$langs->trans("Add").'">';
}
print '</td>';
print "</tr>";


print '</table>';
print '</div>';

print '</form>';


print '<br />';

print '<form action="'.$_SERVER['PHP_SELF'].'?id='.$id.'" method="POST">';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';


print '<div class="div-table-responsive">';
print '<table class="noborder" width="100%">';

// Title line with search boxes
print '<tr class="liste_titre_filter">';
print '<td class="liste_titre">';
print '<input type="text" class="flat" name="search_transporter" value="'.dol_escape_htmltag($search_transporter).'">';
print '</td>';
print '<td class="liste_titre">';
print '<input type="text" class="flat" name="search_zone" value="'.dol_escape_htmltag($search_zone).'">';
print '</td>';
print '<td class="liste_titre">';
print '</td>';
print '<td class="liste_titre">';
print '</td>'; 
print '<td class="liste_titre">';
print '</td>';
if ($conf->global->USE_REAL_WEIGHT) {
    print '<td class="liste_titre">';
    print '</td>';
}
print '<td class="liste_titre">';
print '</td>';   

print '<td class="liste_titre" colspan="3" align="right">';
print $form->showFilterAndCheckAddButtons(0);
print '</td>';
print '</tr>';

// Title of lines
print '<tr class="liste_titre">';


print getTitleFieldOfList($langs->trans("WeightRangeTransporter"), 0, $_SERVER["PHP_SELF"], 'transporter_label', '', $param, "align='left'", $sortfield, $sortorder);
print getTitleFieldOfList($langs->trans("WeightRangeZone"), 0, $_SERVER["PHP_SELF"], 'zone_label', '', $param, "align='left'", $sortfield, $sortorder);
print getTitleFieldOfList($langs->trans("WeightRangeWeightMin"), 0, $_SERVER["PHP_SELF"], 'c.poids_min', '', $param, "align='left'", $sortfield, $sortorder);
print getTitleFieldOfList($langs->trans("WeightRangeWeightMax"), 0, $_SERVER["PHP_SELF"], 'c.poids_max', '', $param, "align='left'", $sortfield, $sortorder);
print getTitleFieldOfList($langs->trans("WeightRangeWeightAmount"), 0, $_SERVER["PHP_SELF"], 'c.montant', '', $param, "align='left'", $sortfield, $sortorder);
if ($conf->global->USE_REAL_WEIGHT) {
    print getTitleFieldOfList($langs->trans("WeightRangeWeightReal"), 0, $_SERVER["PHP_SELF"], 'c.poids_reel', '', $param, "align='left'", $sortfield, $sortorder);
}
print getTitleFieldOfList($langs->trans("WeightRangeWeightActive"), 0, $_SERVER["PHP_SELF"], 'c.active', '', $param, "align='left'", $sortfield, $sortorder);
print getTitleFieldOfList('');
print getTitleFieldOfList('');
print getTitleFieldOfList('');

print '</tr>';

$sql = "SELECT c.rowid, c.fk_transporteur, c.fk_zone, c.poids_min, c.poids_max, c.fk_unit_max, c.fk_unit_min, c.poids_reel, c.montant, c.datec, c.active, ct.label as transporter_label";

if ($conf->global->TRANSPORTER_ZONES_MODE == 'departements') {
    $sql.= ", co.nom as zone_label";
} else if ($conf->global->TRANSPORTER_ZONES_MODE == 'regions') {
    $sql.= ", co.nom as zone_label";
} else {
    $sql.= ", co.label as zone_label";
}

if ($conf->global->TRANSPORTER_ZONES_MODE == 'departements') {
    $sql.= " FROM ".MAIN_DB_PREFIX."c_tranches_poids_departements c";
} else if ($conf->global->TRANSPORTER_ZONES_MODE == 'regions') {
    $sql.= " FROM ".MAIN_DB_PREFIX."c_tranches_poids_regions c";
} else {
    $sql.= " FROM ".MAIN_DB_PREFIX."c_tranches_poids_pays c";
}

$sql.= " LEFT JOIN ".MAIN_DB_PREFIX."c_transporteurs ct ON c.fk_transporteur = ct.rowid";

if ($conf->global->TRANSPORTER_ZONES_MODE == 'departements') {
    $sql.= " LEFT JOIN ".MAIN_DB_PREFIX."c_zones_departements cz ON c.fk_zone = cz.rowid";
} else if ($conf->global->TRANSPORTER_ZONES_MODE == 'regions') {
    $sql.= " LEFT JOIN ".MAIN_DB_PREFIX."c_zones_regions cz ON c.fk_zone = cz.rowid";
} else {
    $sql.= " LEFT JOIN ".MAIN_DB_PREFIX."c_zones_pays cz ON c.fk_zone = cz.rowid";
}

if ($conf->global->TRANSPORTER_ZONES_MODE == 'departements') {
    $sql.= " LEFT JOIN ".MAIN_DB_PREFIX."c_departements co ON co.rowid = cz.fk_object";
} else if ($conf->global->TRANSPORTER_ZONES_MODE == 'regions') {
    $sql.= " LEFT JOIN ".MAIN_DB_PREFIX."c_regions co ON co.rowid = cz.fk_object";
} else {
    $sql.= " LEFT JOIN ".MAIN_DB_PREFIX."c_country co ON co.rowid = cz.fk_object";
}

$sql.= " WHERE c.entity = ".$conf->entity." AND cz.entity = ".$conf->entity;
$sql.= !empty($search_transporter) ? " AND ct.label LIKE '%".$db->escape($search_transporter)."%'" : "";

if ($conf->global->TRANSPORTER_ZONES_MODE == 'departements') {
    $sql.= !empty($search_zone) ? " AND co.nom LIKE '%".$db->escape($search_zone)."%'" : "";
} else if ($conf->global->TRANSPORTER_ZONES_MODE == 'regions') {
    $sql.= !empty($search_zone) ? " AND co.nom LIKE '%".$db->escape($search_zone)."%'" : "";
} else {
    $sql.= !empty($search_zone) ? " AND co.label LIKE '%".$db->escape($search_zone)."%'" : "";
}


$sql.= " ORDER BY ".$sortfield." ".$sortorder;

$resql = $db->query($sql);
if ( $resql )
{
    $num = $db->num_rows($resql);

    if ($num)
    {
        $i = 0;
        // Lines with values
        while ($i < $num)
        {
            $obj = $db->fetch_object($resql);

            print '<tr class="oddeven" id="rowid-'.$obj->rowid.'">';

            if ($action == 'edit' && ($rowid == $obj->rowid))
            {
                print '<td align="left">'.$transportersform->select_transporteurs($obj->fk_transporteur, 'fk_transporteur').'</td>';
                print '<td align="left">'.$zonesform->select_zones($obj->fk_zone, 'fk_zone').'</td>';   
                print '<td align="left">';
                print '<input type="text" class="flat" name="poids_min" value="'.price($obj->poids_min).'" />';
                if (floatval(DOL_VERSION) >= 10)
                {
                    print $formproduct->select_measuring_units("fk_unit_min", "weight", $obj->fk_unit_min, 0, 2);
                }
                else
                {
                    print $formproduct->select_measuring_units("fk_unit_min", "weight", $obj->fk_unit_min, 0);
                }
                print '</td>';
                print '<td align="left">';
                print '<input type="text" class="flat" name="poids_max" value="'.price($obj->poids_max).'" />';
                if (floatval(DOL_VERSION) >= 10)
                {
                    print  $formproduct->select_measuring_units("fk_unit_max", "weight", $obj->fk_unit_max, 0, 2);
                }
                else
                {
                    print  $formproduct->select_measuring_units("fk_unit_max", "weight", $obj->fk_unit_max, 0);
                }
                print '</td>';
                print '<td align="left"><input type="text" class="flat" name="montant" value="'.price($obj->montant).'" />' . '</td>';
                if ($conf->global->USE_REAL_WEIGHT) {
                    print '<td align="left">'.$form->selectyesno('poids_reel', $obj->poids_reel, 1).'</td>';
                }
                print '<td align="left">'.$form->selectyesno('active', $obj->active, 1).'</td>';
                print '<td colspan="3" align="center">';
                print '<input type="hidden" name="rowid" value="'.$rowid.'">';
                print '<input type="submit" class="button" name="actionmodify" value="'.$langs->trans("Modify").'">';
                print '<input type="submit" class="button" name="actioncancel" value="'.$langs->trans("Cancel").'">';
                print '</td>';
            }
            else
            {
                $url = $_SERVER["PHP_SELF"].'?sortfield='.$sortfield.'&sortorder='.$sortorder.'&rowid='.$obj->rowid;
                if ($param) $url .= '&'.$param;
                $url.='&';

                print '<td align="left">'.$obj->transporter_label.'</td>';
                print '<td align="left">'.$obj->zone_label.'</td>';
                print '<td align="left">'.price($obj->poids_min).' '.measuring_units_string($obj->fk_unit_min, 'weight').'</td>';
                print '<td align="left">'.price($obj->poids_max).' '.measuring_units_string($obj->fk_unit_max, 'weight').'</td>';
                print '<td align="left">'.price($obj->montant, 0, $langs, 1, -1, -1, $conf->currency).'</td>';
                if ($conf->global->USE_REAL_WEIGHT) {
                    print '<td align="left"><a href="'.$url.'action='.$act2s[$obj->poids_reel].'">'.$actl[$obj->poids_reel].'</a></td>';
                }
                print '<td align="left"><a href="'.$url.'action='.$acts[$obj->active].'">'.$actl[$obj->active].'</a></td>';

                // Clone link
                print '<td align="center">';
                print '<a href="'.$url.'action=clone">'.img_picto($langs->trans("CloneLine"), 'view.png').'</a>';
                print '</td>';

                // Modify link         
                print '<td align="center"><a class="reposition" href="'.$url.'action=edit">'.img_edit().'</a></td>';
                // Delete link
                print '<td align="center">';
                print '<a href="'.$url.'action=delete">'.img_delete().'</a>';
                print '</td>';

                print "</tr>\n";
            }

            $i++;
        }
    }
}
else
{
    dol_print_error($db);
}


print '</table>';
print '</div>';


print '</form>';



// Page end
dol_fiche_end();
llxFooter();
