<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017 Mikael Carlavan <contact@mika-carl.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       htdocs/fraistransport/admin/setup.php
 *  \ingroup    fraistransport
 *  \brief      Admin page
 */


$res=@include("../../main.inc.php");                   // For root directory
if (! $res) $res=@include("../../../main.inc.php");    // For "custom" directory

// Libraries
require_once DOL_DOCUMENT_ROOT . "/core/lib/admin.lib.php";
require_once DOL_DOCUMENT_ROOT . "/product/class/product.class.php";
require_once DOL_DOCUMENT_ROOT . "/core/class/html.formcompany.class.php";

dol_include_once("/fraistransport/class/html.form.zones.class.php");
dol_include_once("/fraistransport/lib/fraistransport.lib.php");
dol_include_once("/fraistransport/class/zone.class.php");


// Translations
$langs->load("admin");
$langs->load("other");
$langs->load("dict");

$langs->load("fraistransport@fraistransport");

// Access control
if (! $user->admin) accessforbidden();

// Parameters
$action = GETPOST('action', 'alpha');
$confirm = GETPOST('confirm', 'alpha');

$rowid = GETPOST('rowid', 'int');


$sortfield = GETPOST("sortfield",'alpha') ? GETPOST("sortfield",'alpha') : 'label';
$sortorder = GETPOST("sortorder",'alpha') ? GETPOST("sortorder",'alpha') : 'ASC';

$param = '';
if ($search_label != '')     $param.= '&search_label='.urlencode($search_label);


$paramwithsearch = $param;
$paramwithsearch.= '&sortorder='.$sortorder;
$paramwithsearch.= '&sortfield='.$sortfield;

$acts[0] = "activate";
$acts[1] = "disable";
$actl[0] = img_picto($langs->trans("Disabled"),'switch_off');
$actl[1] = img_picto($langs->trans("Activated"),'switch_on');

// activate
if ($action == $acts[0])
{
    $zone = new Zone($db);
    $sql = "UPDATE ".MAIN_DB_PREFIX.$zone->table_element." SET active = 1 WHERE rowid='".$rowid."'";

    $result = $db->query($sql);
    if (!$result)
    {
        dol_print_error($db);
    }
}

// disable
if ($action == $acts[1])
{
    $zone = new Zone($db);
    $sql = "UPDATE ".MAIN_DB_PREFIX.$zone->table_element." SET active = 0 WHERE rowid='".$rowid."'";

    $result = $db->query($sql);
    if (!$result)
    {
        dol_print_error($db);
    }
}

/*
 * Actions
 */
if (GETPOST('actionadd') || GETPOST('actionmodify'))
{
    $zone = new Zone($db);


    if (GETPOST('actionadd'))
    {
        $zone->fk_object = GETPOST('fk_object');

        $result = $zone->create($user);
    }
    else
    {
        $zone->fetch($rowid);
        $zone->fk_object = GETPOST('fk_object');

        $result = $zone->update($rowid, $user);        
    }
    

    if ($result < 0)
    {
        setEventMessages($langs->trans($zone->error), null, 'errors');
    }
}

if ($action == 'confirm_delete' && $confirm == 'yes')       // delete
{
    $zone = new Zone($db);
    $zone->fetch($rowid);
    $result = $zone->delete($user);        

    if ($result < 0)
    {
        setEventMessages($langs->trans($zone->error), null, 'errors');
    }
}

/*
 * View
 */

llxHeader('', $langs->trans('FraisTransportZones'));

// Subheader
$linkback = '<a href="' . DOL_URL_ROOT . '/admin/modules.php">' . $langs->trans("BackToModuleList") . '</a>';

print load_fiche_titre($langs->trans('FraisTransportZones'), $linkback);

// Configuration header
$head = fraistransport_prepare_admin_head();
dol_fiche_head(
	$head,
	'zones',
	$langs->trans("ModuleFraisTransportName"),
	0,
	"fraistransport@fraistransport"
);

// Setup page goes here
echo $langs->trans("FraisTransportZonesPage");


$form = new Form($db);
$formcompany = new FormCompany($db);
$zonesform = new ZonesForm($db);


$zone = new Zone($db);

// Confirmation de la suppression de la ligne
if ($action == 'delete')
{
    print $form->formconfirm($_SERVER["PHP_SELF"].'?rowid='.$rowid.$paramwithsearch, $langs->trans('DeleteLine'), $langs->trans('ConfirmDeleteLine'), 'confirm_delete','',0,1);
}


print '<form action="'.$_SERVER['PHP_SELF'].'?id='.$id.'" method="POST">';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';


print '<div class="div-table-responsive-no-min">';
print '<table class="noborder" width="100%">';

// Line for title
print '<tr class="liste_titre">';
print '<td>'.$langs->trans("ZoneLabel").'</td>';
print '<td style="min-width: 26px;"></td>';
print '<td style="min-width: 26px;"></td>';
 print '</tr>';

// Line to enter new values
print '<tr class="oddeven nodrag nodrop nohover">';

 if ($conf->global->TRANSPORTER_ZONES_MODE == 'departements')
{
    print '<td align="left">';
    if ($action != 'edit')
    {
        print $formcompany->select_state('', 0, 'fk_object');
    }
    print '</td>';
}
else if ($conf->global->TRANSPORTER_ZONES_MODE == 'regions')
{
    print '<td align="left">';
    if ($action != 'edit')
    {
        print $zonesform->select_region('', 'fk_object');
    }
    print '</td>';
}     
else
{
    print '<td align="left">';
    if ($action != 'edit')
    {
        print $form->select_country('', 'fk_object');
    }
    print '</td>';
}    

print '<td colspan="2" align="center">';
if ($action != 'edit')
{
    print '<input type="submit" class="button" name="actionadd" value="'.$langs->trans("Add").'">';
}
print '</td>';
print "</tr>";


print '</table>';
print '</div>';

print '</form>';


print '<br />';

print '<form action="'.$_SERVER['PHP_SELF'].'?id='.$id.'" method="POST">';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';


print '<div class="div-table-responsive">';
print '<table class="noborder" width="100%">';


// Title line with search boxes
print '<tr class="liste_titre_filter">';
print '<td class="liste_titre">';
print '<input type="text" class="flat" name="search_label" value="'.dol_escape_htmltag($search_label).'">';
print '</td>';  
print '<td class="liste_titre">';
print '</td>';           
print '<td class="liste_titre">';
print '</td>';   

print '<td class="liste_titre" colspan="2" align="right">';
print $form->showFilterAndCheckAddButtons(0);
print '</td>';
print '</tr>';

// Title of lines
print '<tr class="liste_titre">';


print getTitleFieldOfList($langs->trans("ZoneLabel"), 0, $_SERVER["PHP_SELF"], 'label', '', $param, "align='left'", $sortfield, $sortorder);
print getTitleFieldOfList($langs->trans("ZoneActive"), 0, $_SERVER["PHP_SELF"], 'cz.active', '', $param, "align='left'", $sortfield, $sortorder);
print getTitleFieldOfList('');
print getTitleFieldOfList('');


print '</tr>';


$sql = "SELECT cz.rowid, cz.fk_object, cz.datec, cz.active";

if ($conf->global->TRANSPORTER_ZONES_MODE == 'departements') {
    $sql.= ", cd.nom as label";
} else if ($conf->global->TRANSPORTER_ZONES_MODE == 'regions') {
    $sql.= ", cr.nom as label";
} else {
    $sql.= ", cp.label as label";
}

$sql.= " FROM ".MAIN_DB_PREFIX.$zone->table_element. " cz";

if ($conf->global->TRANSPORTER_ZONES_MODE == 'departements') {
    $sql.= " LEFT JOIN ".MAIN_DB_PREFIX."c_departements cd ON cd.rowid = cz.fk_object";
} else if ($conf->global->TRANSPORTER_ZONES_MODE == 'regions') {
    $sql.= " LEFT JOIN ".MAIN_DB_PREFIX."c_regions cr ON cr.rowid = cz.fk_object";
} else {
    $sql.= " LEFT JOIN ".MAIN_DB_PREFIX."c_country cp ON cp.rowid = cz.fk_object";
}


$sql.= " WHERE cz.entity = ".$conf->entity;
$sql.= !empty($search_label) ? " AND label LIKE '%".$db->escape($search_label)."%'" : "";

$sql.= " ORDER BY ".$sortfield." ".$sortorder;

$resql = $db->query($sql);
if ( $resql )
{
    $num = $db->num_rows($resql);

    if ($num)
    {
        $i = 0;
        // Lines with values
        while ($i < $num)
        {
            $obj = $db->fetch_object($resql);

            print '<tr class="oddeven" id="rowid-'.$obj->rowid.'">';

            if ($action == 'edit' && ($rowid == $obj->rowid))
            {
                //print '<td align="left">'.$form->select_country($obj->country_id ? $obj->country_id : $mysoc->country_code, 'country_id').'</td>';

                if ($conf->global->TRANSPORTER_ZONES_MODE == 'departements')
                {
                    print '<td colspan="2" align="left">';
                    print $formcompany->select_state($obj->fk_object, 0, 'fk_object');
                    print '</td>';
                }
                else if ($conf->global->TRANSPORTER_ZONES_MODE == 'regions')
                {
                    print '<td colspan="2" align="left">';
                    print $zonesform->select_region($obj->fk_object, 'fk_object');
                    print '</td>';
                }     
                else
                {
                    print '<td colspan="2" align="left">';
                    print $form->select_country($obj->fk_object, 'fk_object');
                    print '</td>';
                }          

                print '<td colspan="2" align="center">';
                print '<input type="hidden" name="rowid" value="'.$rowid.'">';
                print '<input type="submit" class="button" name="actionmodify" value="'.$langs->trans("Modify").'">';
                print '<input type="submit" class="button" name="actioncancel" value="'.$langs->trans("Cancel").'">';
                print '</td>';
            }
            else
            {
                $url = $_SERVER["PHP_SELF"].'?sortfield='.$sortfield.'&sortorder='.$sortorder.'&rowid='.$obj->rowid;
                if ($param) $url .= '&'.$param;
                $url.='&';
                
                print '<td align="left">'.$obj->label.'</td>';
                print '<td align="left"><a href="'.$url.'action='.$acts[$obj->active].'">'.$actl[$obj->active].'</a></td>';
                // Modify link         
                print '<td align="center"><a class="reposition" href="'.$url.'action=edit">'.img_edit().'</a></td>';
         

                // Delete link
                print '<td align="center">';
                print '<a href="'.$url.'action=delete">'.img_delete().'</a>';
                print '</td>';

                print "</tr>\n";
            }

            $i++;
        }
    }
}
else
{
    dol_print_error($db);
}


print '</table>';
print '</div>';


print '</form>';



// Page end
dol_fiche_end();
llxFooter();
