<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017 Mikael Carlavan <contact@mika-carl.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
/**
 *  \file       htdocs/fraistransport/class/actions_fraistransport.class.php
 *  \ingroup    fraistransport
 *  \brief      File of class to manage actions on propal
 */

require_once DOL_DOCUMENT_ROOT.'/core/class/commonobject.class.php';
require_once DOL_DOCUMENT_ROOT.'/product/class/product.class.php';
require_once DOL_DOCUMENT_ROOT.'/contact/class/contact.class.php';

require_once DOL_DOCUMENT_ROOT . '/categories/class/categorie.class.php';
require_once DOL_DOCUMENT_ROOT . "/core/class/html.formcompany.class.php";

dol_include_once("/fraistransport/class/zone.class.php");
dol_include_once("/fraistransport/class/weightrange.class.php");
dol_include_once("/fraistransport/class/transporter.class.php");
dol_include_once("/fraistransport/class/weighttool.class.php");

dol_include_once("/fraistransport/class/html.form.zones.class.php");

class ActionsFraisTransport
{
	function doActions($parameters, &$object, &$action, $hookmanager)
	{
		global $langs, $db, $mysoc, $conf, $user;
		
		$langs->load('fraistransport@fraistransport');
			
		$fk_range = GETPOST('fk_range', 'int');
		$fk_zone = GETPOST('zone', 'int');
		$weight = GETPOST('weight', 'int');
		$use_hayon= GETPOST('use_hayon', 'int');

		$qty = 1;
		$remise_percent = '';
		$fournprice = '';
		$buyingprice = '';
		$date_start = '';
		$date_end = '';
		$info_bits = 0;
		
		$price_min = 0;
		$price_base_type = 'HT';
		$label = '';
		$tva_npr = 0;

		if ($action == 'setfkzone')
		{
			$fk_zone = GETPOST('fk_zone', 'int');

			$array_options = $object->array_options;
			$array_options['options_fk_zone'] = $fk_zone;
			$object->array_options = $array_options;
			$object->insertExtraFields();	
		}

		if ($action == 'selectzone')
		{
			$array_options = $object->thirdparty->array_options;
			$fk_zone = $array_options['options_fk_zone'] ? $array_options['options_fk_zone'] : 0;

			if ($fk_zone > 0)
			{
				$action = 'selecttransporter';
			}
		}

		if ($action == 'confirm_selecttransporter')
		{
            $tool = new WeightTool($db);
            $weight = $tool->getTotalWeight($object);

			$range = new WeightRange($db);
			$result = $range->fetch($fk_range);

			if ($result > 0)
			{
				$fk_transporteur = $range->fk_transporteur;

				$transporteur = new Transporter($db);
				$result = $transporteur->fetch($fk_transporteur);

				if ($result > 0)
				{
					$idprod = $transporteur->fk_product;
                    $montant = $range->montant;
                    $buyingprice = $range->montant;

                    if ($conf->global->USE_REAL_WEIGHT && $range->poids_reel) {
                        $montant = $montant * $weight;
                        $buyingprice = $buyingprice * $weight;
                    }

                    if($conf->global->USE_HAYON && $use_hayon==1){
                    	$mt_hayon= $conf->global->HAYON_PRICE;
                    }else{
                    	$mt_hayon=0;
                    }

					$pu_ht = $this->ValorisePriceTransport($montant, $transporteur->subject_to_fuel_tax)+$mt_hayon;

					$prod = new Product($db);
					$result = $prod->fetch($idprod);

					if ($result > 0)
					{
						//$tva_tx = price2num(preg_replace('/\s*\(.*\)/', '', $prod->tva_tx));
						$pu_ttc = price2num($pu_ht * (1 + ($tva_tx / 100)), 'MU');
						
                    	#PRAGMATECH TVA 0% EU
						$tva_tx = get_default_tva($mysoc, $object->thirdparty, $prod->id);
                    
                    
						$desc = $prod->description;
						$type = $prod->type;
						$fk_unit = $prod->fk_unit;
						$detailzone = new Zone($db);
						$zonedet = $detailzone->getzone($fk_zone);
						$infopoids='Poids (en kg) : ' .$weight;
							if($use_hayon==1){
							$infohayon='<br>Transport avec Hayon';
							}
						// Local Taxes
						$localtax1_tx = get_localtax($prod->tva_tx, 1, $object->thirdparty, $tva_npr);
						$localtax2_tx = get_localtax($prod->tva_tx, 2, $object->thirdparty, $tva_npr);	


						if ($object->element == 'propal') {
							$result = $object->addline($desc.$infohayon.'<br />'.$zonedet.'<br />'.$infopoids, $pu_ht, $qty, $tva_tx, $localtax1_tx, $localtax2_tx, $idprod, $remise_percent, $price_base_type, $pu_ttc, $info_bits, $type, -1, 0, '', $fournprice, $buyingprice, $label, $date_start, $date_end, '', $fk_unit, '', 0, '');
						} else {

							$result = $object->addline($desc.'<br />'.$zonedet.'<br />'.$infopoids, $pu_ht, $qty, $tva_tx, $localtax1_tx, $localtax2_tx, $idprod, $remise_percent, $info_bits, 0, $price_base_type, $pu_ttc, $date_start, $date_end, $type, -1, 0, 0, $fournprice, $buyingprice, $label, 0, $fk_unit);
						}

					

						if ($result > 0)
						{
						    if ($transporteur->fk_shipping_method > 0) {
                                $object->setShippingMethod($transporteur->fk_shipping_method);
                            }

							setEventMessages($langs->trans("FraisTransportAdded"), null, 'mesgs');

							header("Location: " . $_SERVER["PHP_SELF"] . "?id=".$object->id);
							exit;

						}
						else
						{
			                if (count($object->errors))
			                {
			                    setEventMessages($object->error, $object->errors, 'errors');
			                }
			                else
			                {
			                    setEventMessages($object->error, null, 'errors');
			                }
						}				
					}
					else
					{
		                if (count($prod->errors))
		                {
		                    setEventMessages($prod->error, $prod->errors, 'errors');
		                }
		                else
		                {
		                    setEventMessages($prod->error, null, 'errors');
		                }
					}
				}
				else
				{
	                if (count($transporteur->errors))
	                {
	                    setEventMessages($transporteur->error, $transporteur->errors, 'errors');
	                }
	                else
	                {
	                    setEventMessages($transporteur->error, null, 'errors');
	                }
				}
				
			}
			else
			{
                if (count($range->errors))
                {
                    setEventMessages($range->error, $range->errors, 'errors');
                }
                else
                {
                    setEventMessages($range->error, null, 'errors');
                }
			}
			

			$object->fetch_lines();
		}

		$this->resprints = '';
		return 0;
		
	}

	function formConfirm($parameters, &$object, &$action, $hookmanager)
	{
		global $langs, $db, $mysoc, $conf;
		
		$form = new Form($db);

		$langs->load('fraistransport@fraistransport');

		$formconfirm = '';


		if ($action == 'selectzone')
		{

			$state_id = 0;
			$fk_object = 0;

			// Destinataire
			$ids = $object->getIdContact('external','SHIPPING');
			$contact = new Contact($db);

			if (sizeof($ids) && ! empty($id[0]))
			{
				// Contact address
				$contact->fetch($ids[0]);
				
				// Traitement spécifique FR, le code postal du contact détermine le code du département
				$state_id = ($contact->zip && $contact->country_code == 'FR') ? dol_getIdFromCode($db, substr($contact->zip, 0, 2),'c_departements', 'code_departement', 'rowid') : 0;	        	// Id of department
				$country_id = ($contact->country_id ? $contact->country_id : 0);

				if ($state_id <= 0)
				{
					$state_id = ($contact->state_id ? $contact->state_id : 0);
				}
			}	

			$thirdparty = $object->thirdparty;

			if ($state_id <= 0)
			{
				// Thirdparty address
				$state_id = ($thirdparty->zip  && $thirdparty->country_code == 'FR') ? dol_getIdFromCode($db, substr($thirdparty->zip, 0, 2),'c_departements', 'code_departement', 'rowid') : 0;

				if ($state_id <= 0)
				{
					$state_id = ($thirdparty->state_id ? $thirdparty->state_id : 0);
				}
			}	

			if ($country_id <= 0)
			{
				$country_id = ($thirdparty->country_id ? $thirdparty->country_id : 0);
			}

			if ($conf->global->TRANSPORTER_ZONES_MODE == 'departements') {
				$fk_object = $state_id;
			} else if ($conf->global->TRANSPORTER_ZONES_MODE == 'regions') {
				$zone = new Zone($db);
				$fk_object = $zone->getRegionId($state_id);
			} else {
				$fk_object = $country_id;
			}
			
			$zone = new Zone($db);
			$zones = $zone->getZones();
			$firstZone = count($zones) > 0 ? array_shift($zones) : null;

			if ($conf->global->TRANSPORTER_ZONES_MODE == 'departements') {
            	$formcompany = new FormCompany($db);
            	$selected = $fk_object > 0 ? $fk_object : ($firstZone ? $firstZone->fk_object : 0);
				$html = $formcompany->select_state($state_id, 0, 'fk_address');
			} else if ($conf->global->TRANSPORTER_ZONES_MODE == 'regions') {
				$zonesform = new ZonesForm($db);
				$selected = $fk_object > 0 ? $fk_object : ($firstZone ? $firstZone->fk_object : 0);
				$html = $zonesform->select_region($selected, 'fk_address');

			} else {
				$zonesform = new ZonesForm($db);
				$selected = $fk_object > 0 ? $fk_object : ($firstZone ? $firstZone->fk_object : 0);
				$html = $zonesform->select_country($selected, 'fk_address');
			}

			// Create an array for form
			$formquestion = array(
				array('type' => 'other','name' => 'fk_address','label' => $langs->trans("DeliveryZone"),'value' => $html)
			);

			$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"] . '?id=' . $object->id, $langs->trans('SelectDeliveryZone'), $langs->trans('ConfirmSelectDeliveryZone'), 'selecttransporter', $formquestion, 'yes', 1);				
		}

		if ($action == 'selecttransporter')
		{
			$fk_address = GETPOST('fk_address');

			// Get zone for this address
			$array_options = $object->thirdparty->array_options;
			$fk_zone = $array_options['options_fk_zone'] ? $array_options['options_fk_zone'] : 0;

			$zone = new Zone($db);

			if ($fk_address > 0 && empty($fk_zone))
			{
				if ($zone->fetch('', $fk_address) > 0)
				{
					$fk_zone = $zone->id;
				}
			} 


			$transporter = new Transporter($db);
						
			$transporteurs = $transporter->getTransporters(1);

			$tool = new WeightTool($db);
			$weight = $tool->getTotalWeight($object);
	    	
	    	// Get transporters for this zone
	    	$range = new WeightRange($db);

	    	list($minTransporteurs, $maxTransporteurs) = $range->getMinMaxForTransporters();
	    	$ranges = $range->getRangesForZone($fk_zone);

	    	$selectedTransporteurs = array();

	    	// Loop on transporter which match weight
	    	if (is_array($ranges) && sizeof($ranges))
	    	{	
	    		foreach ($ranges as $fk_transporteur => $rangesByT)
	    		{

	    			// Get min/max
	    			$minT = $minTransporteurs[$fk_transporteur];
	    			$maxT = $maxTransporteurs[$fk_transporteur];

                    if ($weight >= $maxT->poids_max)
	    			{
	    				if (isset($transporteurs[$fk_transporteur]))
	    				{
	    					if ($transporteurs[$fk_transporteur]->out_of_range == Transporter::TAKE_MAX)
	    					{
	    						$selectedTransporteurs[] = $maxT;
	    						
							}
							elseif($transporteurs[$fk_transporteur]->out_of_range == Transporter::FORCE_TO_ZERO)
	    					{
								// Sans clonage, assigner le montant à zéro modifie également la variable $ranges
								$m = clone($maxT);
								$m->montant = 0;
	    						$selectedTransporteurs[] = $m;
	    					} 
	    				}
	    			}
	    			else if ($weight < $minT->poids_min)
	    			{
	    				if (isset($transporteurs[$fk_transporteur]))
	    				{
		    				if ($transporteurs[$fk_transporteur]->out_of_range == Transporter::TAKE_MIN)
	    					{
	    						$selectedTransporteurs[] = $minT;
	    						
	    					} elseif($transporteurs[$fk_transporteur]->out_of_range == Transporter::FORCE_TO_ZERO) {
								// Sans clonage, assigner le montant à zéro modifie également la variable $ranges
								$m = clone($minT);
								$m->montant = 0;
	    						$selectedTransporteurs[] = $m;
	    					} 

	    				}
	    			}
	    			else
	    			{
		    			if (sizeof($rangesByT))
		    			{
		    				foreach ($rangesByT as $r)
		    				{
				    			$poids_min = $r->poids_min;
				    			$poids_max = $r->poids_max;
								
				    			if ($weight >= $poids_min && $weight < $poids_max)
				    			{
				    				$selectedTransporteurs[] = $r;
				    			}			
		    				}
		    			}	    				
	    			}
	    		}
	    	}
			
	    	if (sizeof($selectedTransporteurs))
	    	{
	    		$formquestion = array();

	    		$transporteursSelect = array();

	    		$height = 270;
	    		$width = 800;
	    		$meilleurtarif = 1000000000;
	    		$i = 1;

	    		foreach ($selectedTransporteurs as $selectedTransporteur)
	    		{
	    			$height += 50;

	    			$montant = $selectedTransporteur->montant;

	    			if ($conf->global->USE_REAL_WEIGHT && $selectedTransporteur->poids_reel) {
	    			    $montant = $montant * $weight;
                    }
	    			//on ajoute les taxes et le coeff au prix brut
	    			//$pricevalorise=$this->ValorisePriceTransport($selectedTransporteur->montant,$transporteurs[$i]->subject_to_fuel_tax);
	    			$pricevalorise = $this->ValorisePriceTransport($montant, $transporteurs[$selectedTransporteur->fk_transporteur]->subject_to_fuel_tax);

	    			if($selectedTransporteurs->montant == 0 && $transporteurs[$fk_transporteur]->out_of_range == Transporter::FORCE_TO_ZERO){
						$achtung='<br>'.img_picto($langs->trans('PriceForceToZero'),'warning.png').'<span style="color:red;"> '.$langs->trans("PriceForceToZero").'</span>';	    			
	    			}else{
	    				$achtung='';	
	    			}

					$formquestions[] = array(
						'type' => 'other',
						'name' => 'transporteur_'.$selectedTransporteur->id,
						'label' => $transporteurs[$selectedTransporteur->fk_transporteur]->label.$achtung, 
						'value' => price($pricevalorise, 0, $langs, 1, -1, -1, $conf->currency)
					);

					if ($pricevalorise < $meilleurtarif) {
						$meilleurtarif = $pricevalorise;

						$besttarifselected = array(
							$selectedTransporteur->id => $transporteurs[$selectedTransporteur->fk_transporteur]->label
						);
					}

					$transporteursSelect[$selectedTransporteur->id] = $transporteurs[$selectedTransporteur->fk_transporteur]->label;
					$i++;
	    		}

	    		//place le meilleur tarif en premier dans le tableau
	    		$transporteursSelect = $besttarifselected + $transporteursSelect;
	    			    		
				// Create an array for form
				$formquestions[] = array(
					'type' => 'other',
					'name' => 'fk_range',
					'label' => $langs->trans("SelectTransporter"),
					'value' => $form->selectarray('fk_range', $transporteursSelect, GETPOST('fk_range', 'int'))
				);

				$zones = $zone->getZones();
				$formquestions['text'] = $langs->trans('SelectTransporterDesc').'<br>'.$langs->trans('WeighttoKeep', round($weight, 1)).'<br>'.$langs->trans('ToDestination').$zones[$fk_zone]->label.'<br><br>';
				

				//select for hayon
				if($conf->global->USE_HAYON){
					$hayonchoice=array(
						0 	=>	$langs->trans("NoHayon"),
						1	=>	$langs->trans("YesIWant").' (+'.$conf->global->HAYON_PRICE.')',
					);
					$formquestions['other'] = array(
						'type' => 'other',
						'name' => 'use_hayon',
						'label' => $langs->trans("DoYouWantAnHayon"),
						'value' => $form->selectarray('use_hayon', $hayonchoice, GETPOST('fk_range', 'int'))
					);
				}
	    		$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"] . '?weight='.$weight.'&zone='.$fk_zone.'&id=' . $object->id, $langs->trans('SelectTransporter'), $langs->trans('ConfirmSelectTransporter'), 'confirm_selecttransporter', $formquestions, 'yes', 1, $height,$width);
	    	}
	    	else
	    	{
				$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"] . '?id=' . $object->id, $langs->trans('SelectTransporter'), $langs->trans('ConfirmNoTransportersAvailable'), '', '', 'yes', 1);			    		
	    	}

		
		}

		$this->resprints = $formconfirm;
		return 0;		
	}

	function formObjectOptions($parameters, &$object, &$action, $hookmanager)
	{
		global $langs, $db, $mysoc, $conf, $user;
		
		$langs->load('fraistransport@fraistransports');
		$result = 0;

		if ($object->id > 0 && $object->element == 'societe')
		{
			$array_options = $object->array_options;
			$fk_zone = $array_options['options_fk_zone'] ? $array_options['options_fk_zone'] : 0;
			//$object->array_options = $array_options;
			//$object->insertExtraFields();

			$zonesform = new ZonesForm($db);

			// zone
			print '<tr><td class="nowrap">';
				print '<table width="100%" class="nobordernopadding"><tr><td class="nowrap">';
				print $langs->trans('DeliveredZone');
				print '</td>';
				if ($action != 'editfkzone')
					print '<td align="right"><a href="'.$_SERVER["PHP_SELF"].'?action=editfkzone&amp;id='.$object->id.'">'.img_edit($langs->trans('SetZone'),1).'</a></td>';
				print '</tr>';
				print '</table>';
			print '</td><td>';
			if ($action == 'editfkzone') {
				print '<form method="POST" action="'.$_SERVER["PHP_SELF"].'?id='.$object->id.'">';
				print '<input type="hidden" name="action" value="setfkzone">';
				print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
				print $zonesform->select_zones($fk_zone, 'fk_zone', '', true);
				//print $form->select_type_commande($gestion->fk_type_commande, 'fk_type_commande');
				print '<input type="submit" class="button valignmiddle" value="'.$langs->trans("Modify").'">';
				print '</form>';
			} else {
				$zone = new Zone($db);
				$zones = $zone->getZones();
				print isset($zones[$fk_zone]) ? $zones[$fk_zone]->label : '&nbsp;';
			}
			print '</td>';
			print '</tr>';
	
		}
		

		$this->resprints = '';
		return 0;			
	}

    function addMoreActionsButtons($parameters, &$object, &$action, $hookmanager)
    {
        global $langs, $db, $mysoc, $conf, $user;

        $langs->load('fraistransport@fraistransport');

        if (($object->element == 'propal' && $object->statut == Propal::STATUS_DRAFT) || ($object->element == 'commande' && $object->statut == Commande::STATUS_DRAFT))
        {
            print '<div class="inline-block divButAction"><a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '&amp;action=selectzone">' . $langs->trans('AddFraisTransport') . '</a></div>';
        }

        $this->resprints = '';
        return 0;
    }

    /**
     * Valorise le prix du transport si une taxe a définie et/ou si un coef a été défini
     * @param int $montant le montant du frais de transport sur lequel c'est appliqué
     */
    function ValorisePriceTransport($montant, $subject_to_fuel_tax)
    {
        global $conf;

        $transport_tx_tax = $conf->global->TRANSPORTER_TX_TAX;
        $transport_coef = $conf->global->TRANSPORTER_TX_COEF;

        //si il y a une taxe sur le transport et si le transporteur est soumis à la taxe sur le caburant
        if ($transport_tx_tax > 0 && $subject_to_fuel_tax == 1) {
            $montant = $montant * (($transport_tx_tax/100) + 1);
        }

        //si il y a un coeff sur le transport
        if ($transport_coef > 1) {
            $montant = $montant * $transport_coef;
        }

        return $montant;
    }
}


