<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017 Mikael Carlavan <contact@mika-carl.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       htdocs/transporters/class/html.form.transporters.class.php
 *  \ingroup    transporters
 *  \brief      File of class to manage form for transporters
 */
dol_include_once("/fraistransport/class/transporter.class.php");


class TransportersForm
{
    var $db;
    var $error;

    /**
     * Constructor
     * @param      $DB      Database handler
     */
    function TransportersForm($DB)
    {
        $this->db = $DB;
    }

    /**
     *    Return combo list of products sets
     *    @param     selected         Id preselected products set
     *    @param     htmlname         Name of html select object
     *    @param     htmloption       Options html on select object
     *    @return    string           HTML string with select
     */
    function select_transporteurs($selected='',$htmlname='fk_transporter', $htmloption='')
    {
        global $conf, $langs;

        
        $transporter = new Transporter($this->db);

        $transporters = $transporter->getTransporters();
        
        //Build select
        $select = '<select class="flat" id = "'.$htmlname.'" name = "'.$htmlname.'" '.$htmloption.'>';
        if (is_array($transporters) && sizeof($transporters))
        {
            foreach ($transporters as $id => $transporter)
            {
                $select .= '<option value="'.$id.'" '.($id == $selected ? 'selected="selected"' : '').'>'.$transporter->label.'</option>';
            }            
        }

        
        $select .= '</select>';

        return $select;
    }

       	
}

?>
