<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017 Mikael Carlavan <contact@mika-carl.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       htdocs/fraistransport/class/fraistransport.class.php
 *  \ingroup    fraistransport
 *  \brief      File of class to manage predefined products sets
 */
require_once DOL_DOCUMENT_ROOT.'/core/class/commonobject.class.php';
require_once DOL_DOCUMENT_ROOT.'/product/class/product.class.php';


/**
 * Class to manage products or services
 */
class WeightTool extends CommonObject
{
	public $element='';
	public $table_element='';
	public $fk_element='';
	public $picto = 'generic';
	public $ismultientitymanaged = 1;	// 0=No test on entity, 1=Test with field entity, 2=Test with link by societe

	
	/**
	 *  Constructor
	 *
	 *  @param      DoliDB		$db      Database handler
	 */
	function __construct($db)
	{
		global $langs, $conf;

		$this->db = $db;
	}


    function getTotalWeight(&$object, $useObjectMethod = true)
    {
        global $conf;

        $weight = 0;

		// Compute total weight of object
        $lines = $object->lines;
        $weight = 0;

        if ($useObjectMethod && method_exists($object, 'getTotalWeightVolume')) {
            $tmparray = $object->getTotalWeightVolume();
            $weight = $tmparray['weight'];
        } else {
            for ($i = 0; $i < sizeof($lines); $i++)
            {
                $line = $lines[$i];

                if ($line->fk_product > 0 && $line->product_type == 0)
                {
                    $product = new Product($this->db);
                    $result = $product->fetch($line->fk_product);

                    if ($result > 0)
                    {
                        $product_weight = WeightTool::toKg($product->weight, $product->weight_units);

                        $weight += $line->qty * $product_weight;
                    }
                }
            }
        }

        //Si module poids total est activé et plus grand que le calculé, alors on prend sa valeur à lui
        //(pour la prise en compte du poids des produits libres dont le poids est ajouté via un extrafield)
        $customweight = 0;
        if ($conf->poidstotal->enabled){
            $customweightletter = $object->array_options['options_poids_total'];
            $customweight = preg_replace('/[^0-9^.^,]/', '', $customweightletter);
        }

        if ($customweight > $weight){
            $weight = $customweight;
        }

        return $weight;
    }

	public static function toKg($weight, $fk_unit = 0)
	{
		// Convert to kg
		$weight = floatval($weight);

		if ($fk_unit == 98)
		{
			$weight = $weight * 0.0283495;
		}
		else if ($fk_unit == 99)
		{
			$weight = $weight * 0.453592;
		}
		else 
		{
			$weight = $fk_unit != 0 ? $weight * (10 ** $fk_unit) : $weight;
		}

		return $weight;		
	}
}
