<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017 Mikael Carlavan <contact@mika-carl.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * 	\defgroup   fraistransport     Module fraistransport
 *  \brief      fraistransport module descriptor.
 *
 *  \file       htdocs/fraistransport/core/modules/modfraistransport.class.php
 *  \ingroup    fraistransport
 *  \brief      Description and activation file for module fraistransport
 */
include_once DOL_DOCUMENT_ROOT .'/core/modules/DolibarrModules.class.php';


// The class name should start with a lower case mod for Dolibarr to pick it up
// so we ignore the Squiz.Classes.ValidClassName.NotCamelCaps rule.
// @codingStandardsIgnoreStart
/**
 *  Description and activation class for module fraistransport
 */
class modFraisTransport extends DolibarrModules
{
	// @codingStandardsIgnoreEnd
	/**
	 * Constructor. Define names, constants, directories, boxes, permissions
	 *
	 * @param DoliDB $db Database handler
	 */
	public function __construct($db)
	{
        global $langs,$conf;

        $this->db = $db;

		// Id for module (must be unique).
		// Use here a free id (See in Home -> System information -> Dolibarr for list of used modules id).
		$this->numero = 446103;		// TODO Go on page https://wiki.dolibarr.org/index.php/List_of_modules_id to reserve id number for your module
		// Key text used to identify module (for permissions, menus, etc...)
		$this->rights_class = 'fraistransport';

		// Family can be 'crm','financial','hr','projects','products','ecm','technic','interface','other'
		// It is used to group modules by family in module setup page
		$this->family = "iouston";
		// Module position in the family
		$this->module_position = 500;
		// Gives the possibility to the module, to provide his own family info and position of this family (Overwrite $this->family and $this->module_position. Avoid this)
		//$this->familyinfo = array('myownfamily' => array('position' => '001', 'label' => $langs->trans("MyOwnFamily")));

		// Module label (no space allowed), used if translation string 'ModulefraistransportName' not found (MyModue is name of module).
		$this->name = preg_replace('/^mod/i','',get_class($this));
		// Module description, used if translation string 'ModulefraistransportDesc' not found (MyModue is name of module).
		$this->description = "Frais de transport";
		// Used only if file README.md and README-LL.md not found.
		$this->descriptionlong = "Module permettant d'insérer des frais de transport sur les devis et commandes";

		$this->editor_name = 'iouston informatique';
		$this->editor_url = 'http://www.iouston.com';

		// Possible values for version are: 'development', 'experimental', 'dolibarr', 'dolibarr_deprecated' or a version string like 'x.y.z'
		$this->version = '1.0.11';
		// Key used in llx_const table to save module status enabled/disabled (where SITFAC is value of property name of module in uppercase)
		$this->const_name = 'MAIN_MODULE_'.strtoupper($this->name);
		// Name of image file used for this module.
		// If file is in theme/yourtheme/img directory under name object_pictovalue.png, use this->picto='pictovalue'
		// If file is in module/img directory under name object_pictovalue.png, use this->picto='pictovalue@module'
		$this->picto='fraistransport@fraistransport';

		// Defined all module parts (triggers, login, substitutions, menus, css, etc...)
		// for default path (eg: /fraistransport/core/xxxxx) (0=disable, 1=enable)
		// for specific path of parts (eg: /fraistransport/core/modules/barcode)
		// for specific css file (eg: /fraistransport/css/fraistransport.css.php)
		$this->module_parts = array(
		                        	'triggers' => 0,                                 	// Set this to 1 if module has its own trigger directory (core/triggers)
									'login' => 0,                                    	// Set this to 1 if module has its own login method directory (core/login)
									'substitutions' => 0,                            	// Set this to 1 if module has its own substitution function file (core/substitutions)
									'menus' => 0,                                    	// Set this to 1 if module has its own menus handler directory (core/menus)
									'theme' => 0,                                    	// Set this to 1 if module has its own theme directory (theme)
		                        	'tpl' => 0,                                      	// Set this to 1 if module overwrite template dir (core/tpl)
									'barcode' => 0,                                  	// Set this to 1 if module has its own barcode directory (core/modules/barcode)
									'models' => 0,                                   	// Set this to 1 if module has its own models directory (core/modules/xxx)
									'css' => array(),	// Set this to relative path of css file if module has its own css file
	 								'js' => array(),          // Set this to relative path of js file if module must load a js on all pages
									'hooks' => array('propalcard', 'ordercard', 'thirdpartycard') 	// Set here all hooks context managed by module. You can also set hook context 'all'
		                        );

		// Data directories to create when module is enabled.
		// Example: this->dirs = array("/fraistransport/temp","/fraistransport/subdir");
		$this->dirs = array();

		// Config pages. Put here list of php page, stored into fraistransport/admin directory, to use to setup module.
		$this->config_page_url = array("setup.php@fraistransport");

		// Dependencies
		$this->hidden = false;			// A condition to hide module
		$this->depends = array();		// List of module class names as string that must be enabled if this module is enabled
		$this->requiredby = array();	// List of module ids to disable if this one is disabled
		$this->conflictwith = array();	// List of module class names as string this module is in conflict with
		$this->phpmin = array(5,3);					// Minimum version of PHP required by module
		$this->need_dolibarr_version = array(4,0);	// Minimum version of Dolibarr required by module
		$this->langfiles = array("fraistransport@fraistransport");
		$this->warnings_activation = array();                     // Warning to show when we activate module. array('always'='text') or array('FR'='textfr','ES'='textes'...)
		$this->warnings_activation_ext = array();                 // Warning to show when we activate an external module. array('always'='text') or array('FR'='textfr','ES'='textes'...)

		$this->const = array();

        $this->tabs = array();  	// To add a new tab identified by code tabname1

		if (! isset($conf->fraistransport) || ! isset($conf->fraistransport->enabled))
        {
        	$conf->fraistransport=new stdClass();
        	$conf->fraistransport->enabled=0;
        }

        // Dictionaries
		$this->dictionaries=array();


        // Boxes/Widgets
		// Add here list of php file(s) stored in fraistransport/core/boxes that contains class to show a widget.
        $this->boxes = array();


		// Cronjobs (List of cron jobs entries to add when module is enabled)
		$this->cronjobs = array();


		// Permissions
		$this->rights = array();		// Permission array used by this module
		
		// Main menu entries
		$this->menu = array();			// List of menus to add


		// Exports
		//--------
		$r=0;

		// Export list of transporteur
		$r++;
		$this->export_code[$r]=$this->rights_class.'_'.$r;
		$this->export_label[$r]='ExportFraisTransportTransporteur';
		$this->export_icon[$r]=$this->picto;
		$this->export_permission[$r]=array(array("societe","export"));
		$this->export_fields_array[$r]=array(
			't.rowid'=>"Id",'t.label'=>"Name",'t.ref'=>"Ref",'t.datec'=>"CreationDate",'t.fk_product'=>"AssociatedProduct",'t.out_of_range'=>"OutOfRange",'t.subject_to_fuel_tax'=>"SubjectToFuelTax",'t.tms'=>"Tms",'t.entity'=>"Entity",'t.active'=>"Active");
		$this->export_TypeFields_array[$r]=array('t.rowid'=>"Numeric", 't.label'=>"Text",'t.ref'=>"Text",'t.datec'=>"Date",'t.fk_product'=>"Numeric",'t.out_of_range'=>"Numeric",'t.subject_to_fuel_tax'=>"Numeric",'t.tms'=>"Date",'t.entity'=>"Numeric",'t.active'=>"Numeric");
		$this->export_sql_start[$r]='SELECT DISTINCT ';
		$this->export_sql_end[$r]  =' FROM '.MAIN_DB_PREFIX.'c_transporteurs as t';

		// Export departments 
		$r++;
		$this->export_code[$r]=$this->rights_class.'_'.$r;
		$this->export_label[$r]='ExportFraisTransportZoneDepartement';
		$this->export_icon[$r]=$this->picto;
		$this->export_permission[$r]=array(array("societe","export"));
		$this->export_fields_array[$r]=array(
			'z.rowid'=>"ZoneId",'co.nom'=>"ZoneName", 'co.code_departement'=>"DepartmentCode", 'co.rowid' => "DepartmentId");
		
		$this->export_TypeFields_array[$r]=array('z.rowid'=>"Numeric", 'co.nom'=>"Text",'co.code_departement'=>"Text", 'co.rowid'=>"Numeric");

		$this->export_sql_start[$r]='SELECT DISTINCT ';
		$this->export_sql_end[$r]  =' FROM '.MAIN_DB_PREFIX.'c_zones_departements as z';
        $this->export_sql_end[$r] .=' LEFT JOIN '.MAIN_DB_PREFIX.'c_departements as co ON co.rowid = z.fk_object';
		$this->export_sql_end[$r] .=' ORDER BY co.nom';

		// Export regions
		$r++;
		$this->export_code[$r]=$this->rights_class.'_'.$r;
		$this->export_label[$r]='ExportFraisTransportZoneRegion';
		$this->export_icon[$r]=$this->picto;
		$this->export_permission[$r]=array(array("societe","export"));
		$this->export_fields_array[$r]=array(
			'z.rowid'=>"ZoneId",'co.nom'=>"ZoneName", 'co.code_region'=>"RegionCode", 'co.rowid' => "RegionId");
		
		$this->export_TypeFields_array[$r]=array('z.rowid'=>"Numeric", 'co.nom'=>"Text",'co.code_region'=>"Text", 'co.rowid'=>"Numeric");

		$this->export_sql_start[$r]='SELECT DISTINCT ';
		$this->export_sql_end[$r]  =' FROM '.MAIN_DB_PREFIX.'c_zones_regions as z';
        $this->export_sql_end[$r] .=' LEFT JOIN '.MAIN_DB_PREFIX.'c_regions as co ON co.rowid = z.fk_object';
		$this->export_sql_end[$r] .=' ORDER BY co.nom';

		// Export countries
		$r++;
		$this->export_code[$r]=$this->rights_class.'_'.$r;
		$this->export_label[$r]='ExportFraisTransportZonePays';
		$this->export_icon[$r]=$this->picto;
		$this->export_permission[$r]=array(array("societe","export"));
		$this->export_fields_array[$r]=array(
			'z.rowid'=>"ZoneId",'co.label'=>"ZoneName", 'co.code'=>"CountryCode", 'co.rowid' => "CountryId");
		
		$this->export_TypeFields_array[$r]=array('z.rowid'=>"Numeric", 'co.label'=>"Text",'co.code'=>"Text", 'co.rowid'=>"Numeric");

		$this->export_sql_start[$r]='SELECT DISTINCT ';
		$this->export_sql_end[$r]  =' FROM '.MAIN_DB_PREFIX.'c_zones_pays as z';
        $this->export_sql_end[$r] .=' LEFT JOIN '.MAIN_DB_PREFIX.'c_country as co ON co.rowid = z.fk_object';
		$this->export_sql_end[$r] .=' ORDER BY co.label';


		// Export list of slices weight and amount
		$r++;
		$this->export_code[$r]=$this->rights_class.'_'.$r;
		$this->export_label[$r]='ExportFraisTransportZoneDepartementPoidsTarif';
		$this->export_icon[$r]=$this->picto;
		$this->export_permission[$r]=array(array("societe","export"));
		$this->export_fields_array[$r]=array(
			't.rowid'=>"Id",'t.fk_transporteur'=>"TransporterId",'t.fk_zone'=>"ZoneId",'co.nom'=>"ZoneName",'t.poids_min'=>"MinWeight",'t.poids_max'=>"MaxWeight",'t.fk_unit_min'=>"MinUnit",'t.fk_unit_max'=>"MaxUnit",'t.montant'=>"Amount",'t.datec'=>"CreationDate",'t.tms'=>"Tms",'t.active'=>"Active",'t.entity'=>"Entity");
		$this->export_TypeFields_array[$r]=array('t.rowid'=>"Numeric",'t.fk_transporteur'=>"Numeric",'t.fk_zone'=>"Numeric",'co.nom'=>"Text",'t.poids_min'=>"Numeric",'t.poids_max'=>"Numeric",'t.fk_unit_min'=>"Numeric",'t.fk_unit_max'=>"Numeric",'t.montant'=>"Numeric",'t.datec'=>"Date",'t.tms'=>"Date",'t.active'=>"Numeric",'t.entity'=>"Numeric");
		$this->export_sql_start[$r]='SELECT DISTINCT ';
		$this->export_sql_end[$r]  =' FROM '.MAIN_DB_PREFIX.'c_tranches_poids_departements as t';
		$this->export_sql_end[$r] .=' LEFT JOIN '.MAIN_DB_PREFIX.'c_zones_departements as z ON t.fk_zone = z.rowid';
		$this->export_sql_end[$r] .=' LEFT JOIN '.MAIN_DB_PREFIX.'c_departements as co ON co.rowid = z.fk_object';

		$r++;
		$this->export_code[$r]=$this->rights_class.'_'.$r;
		$this->export_label[$r]='ExportFraisTransportZoneRegionPoidsTarif';
		$this->export_icon[$r]=$this->picto;
		$this->export_permission[$r]=array(array("societe","export"));
		$this->export_fields_array[$r]=array(
			't.rowid'=>"Id",'t.fk_transporteur'=>"TransporterId",'t.fk_zone'=>"ZoneId",'co.nom'=>"ZoneName",'t.poids_min'=>"MinWeight",'t.poids_max'=>"MaxWeight",'t.fk_unit_min'=>"MinUnit",'t.fk_unit_max'=>"MaxUnit",'t.montant'=>"Amount",'t.datec'=>"CreationDate",'t.tms'=>"Tms",'t.active'=>"Active",'t.entity'=>"Entity");
		$this->export_TypeFields_array[$r]=array('t.rowid'=>"Numeric",'t.fk_transporteur'=>"Numeric",'t.fk_zone'=>"Numeric",'co.nom'=>"Text",'t.poids_min'=>"Numeric",'t.poids_max'=>"Numeric",'t.fk_unit_min'=>"Numeric",'t.fk_unit_max'=>"Numeric",'t.montant'=>"Numeric",'t.datec'=>"Date",'t.tms'=>"Date",'t.active'=>"Numeric",'t.entity'=>"Numeric");
		$this->export_sql_start[$r]='SELECT DISTINCT ';
		$this->export_sql_end[$r]  =' FROM '.MAIN_DB_PREFIX.'c_tranches_poids_regions as t';
		$this->export_sql_end[$r] .=' LEFT JOIN '.MAIN_DB_PREFIX.'c_zones_regions as z ON t.fk_zone = z.rowid';
		$this->export_sql_end[$r] .=' LEFT JOIN '.MAIN_DB_PREFIX.'c_regions as co ON co.rowid = z.fk_object';

		$r++;
		$this->export_code[$r]=$this->rights_class.'_'.$r;
		$this->export_label[$r]='ExportFraisTransportZonePaysPoidsTarif';
		$this->export_icon[$r]=$this->picto;
		$this->export_permission[$r]=array(array("societe","export"));
		$this->export_fields_array[$r]=array(
			't.rowid'=>"Id",'t.fk_transporteur'=>"TransporterId",'t.fk_zone'=>"ZoneId",'co.label'=>"ZoneName",'t.poids_min'=>"MinWeight",'t.poids_max'=>"MaxWeight",'t.fk_unit_min'=>"MinUnit",'t.fk_unit_max'=>"MaxUnit",'t.montant'=>"Amount",'t.datec'=>"CreationDate",'t.tms'=>"Tms",'t.active'=>"Active",'t.entity'=>"Entity");
		$this->export_TypeFields_array[$r]=array('t.rowid'=>"Numeric",'t.fk_transporteur'=>"Numeric",'t.fk_zone'=>"Numeric",'co.label'=>"Text",'t.poids_min'=>"Numeric",'t.poids_max'=>"Numeric",'t.fk_unit_min'=>"Numeric",'t.fk_unit_max'=>"Numeric",'t.montant'=>"Numeric",'t.datec'=>"Date",'t.tms'=>"Date",'t.active'=>"Numeric",'t.entity'=>"Numeric");
		$this->export_sql_start[$r]='SELECT DISTINCT ';
		$this->export_sql_end[$r]  =' FROM '.MAIN_DB_PREFIX.'c_tranches_poids_pays as t';
		$this->export_sql_end[$r] .=' LEFT JOIN '.MAIN_DB_PREFIX.'c_zones_pays as z ON t.fk_zone = z.rowid';
		$this->export_sql_end[$r] .=' LEFT JOIN '.MAIN_DB_PREFIX.'c_country as co ON co.rowid = z.fk_object';


		// Imports
		//--------
		$r=0;

		$r++;
		$this->import_code[$r]=$this->rights_class.'_'.$r;
		$this->import_label[$r]="ImportFraisTransportTranchesPoidsDepartement";	// Translation key
		$this->import_icon[$r]=$this->picto;
		$this->import_entities_array[$r]=array();		// We define here only fields that use another icon that the one defined into import_icon
		$this->import_tables_array[$r]=array('ft'=>MAIN_DB_PREFIX.'c_tranches_poids_departements');
		$this->import_fields_array[$r]=array('ft.fk_transporteur'=>"TransporterId",'ft.fk_zone'=>"ZoneId",'ft.poids_min'=>"MinWeight",'ft.poids_max'=>"MaxWeight",'ft.fk_unit_min'=>"MinUnit",'ft.fk_unit_max'=>"MaxUnit",'ft.montant'=>"Amount",'ft.datec'=>"CreationDate",'ft.tms'=>"Tms",'ft.active'=>"Active");				
		$this->import_examplevalues_array[$r]=array('ft.fk_transporteur'=>"",'ft.fk_zone'=>"",'ft.poids_min'=>"min. weight with point decimal separator",'ft.poids_max'=>"max. weight with point decimal separator",'ft.fk_unit_min'=>"'-6=mg, -3=g, 0=kg, 3=t, 98=onces, 99=livres",'ft.fk_unit_max'=>"'-6=mg, -3=g, 0=kg, 3=t, 98=onces, 99=livres",'ft.montant'=>"amount with point decimal separator",'ft.datec'=>"yyyy-mm-dd",'ft.tms'=>"yyyy-mm-dd",'ft.active'=>"1=active / 0=inactive");

		$r++;
		$this->import_code[$r]=$this->rights_class.'_'.$r;
		$this->import_label[$r]="ImportFraisTransportTranchesPoidsRegion";	// Translation key
		$this->import_icon[$r]=$this->picto;
		$this->import_entities_array[$r]=array();		// We define here only fields that use another icon that the one defined into import_icon
		$this->import_tables_array[$r]=array('ft'=>MAIN_DB_PREFIX.'c_tranches_poids_regions');
		$this->import_fields_array[$r]=array('ft.fk_transporteur'=>"TransporterId",'ft.fk_zone'=>"ZoneId",'ft.poids_min'=>"MinWeight",'ft.poids_max'=>"MaxWeight",'ft.fk_unit_min'=>"MinUnit",'ft.fk_unit_max'=>"MaxUnit",'ft.montant'=>"Amount",'ft.datec'=>"CreationDate",'ft.tms'=>"Tms",'ft.active'=>"Active");				
		$this->import_examplevalues_array[$r]=array('ft.fk_transporteur'=>"",'ft.fk_zone'=>"",'ft.poids_min'=>"min. weight with point decimal separator",'ft.poids_max'=>"max. weight with point decimal separator",'ft.fk_unit_min'=>"'-6=mg, -3=g, 0=kg, 3=t, 98=onces, 99=livres",'ft.fk_unit_max'=>"'-6=mg, -3=g, 0=kg, 3=t, 98=onces, 99=livres",'ft.montant'=>"amount with point decimal separator",'ft.datec'=>"yyyy-mm-dd",'ft.tms'=>"yyyy-mm-dd",'ft.active'=>"1=active / 0=inactive");

		$r++;
		$this->import_code[$r]=$this->rights_class.'_'.$r;
		$this->import_label[$r]="ImportFraisTransportTranchesPoidsPays";	// Translation key
		$this->import_icon[$r]=$this->picto;
		$this->import_entities_array[$r]=array();		// We define here only fields that use another icon that the one defined into import_icon
		$this->import_tables_array[$r]=array('ft'=>MAIN_DB_PREFIX.'c_tranches_poids_pays');
		$this->import_fields_array[$r]=array('ft.fk_transporteur'=>"TransporterId",'ft.fk_zone'=>"ZoneId",'ft.poids_min'=>"MinWeight",'ft.poids_max'=>"MaxWeight",'ft.fk_unit_min'=>"MinUnit",'ft.fk_unit_max'=>"MaxUnit",'ft.montant'=>"Amount",'ft.datec'=>"CreationDate",'ft.tms'=>"Tms",'ft.active'=>"Active");				
		$this->import_examplevalues_array[$r]=array('ft.fk_transporteur'=>"",'ft.fk_zone'=>"",'ft.poids_min'=>"min. weight with point decimal separator",'ft.poids_max'=>"max. weight with point decimal separator",'ft.fk_unit_min'=>"'-6=mg, -3=g, 0=kg, 3=t, 98=onces, 99=livres",'ft.fk_unit_max'=>"'-6=mg, -3=g, 0=kg, 3=t, 98=onces, 99=livres",'ft.montant'=>"amount with point decimal separator",'ft.datec'=>"yyyy-mm-dd",'ft.tms'=>"yyyy-mm-dd",'ft.active'=>"1=active / 0=inactive");

	}

	/**
	 *		Function called when module is enabled.
	 *		The init function add constants, boxes, permissions and menus (defined in constructor) into Dolibarr database.
	 *		It also creates data directories
	 *
     *      @param      string	$options    Options when enabling module ('', 'noboxes')
	 *      @return     int             	1 if OK, 0 if KO
	 */
	public function init($options='')
	{
		$sql = array();

		$this->_load_tables('/fraistransport/sql/');

		// Create extrafields
		include_once DOL_DOCUMENT_ROOT.'/core/class/extrafields.class.php';
		$extrafields = new ExtraFields($this->db);
		
		$result = $extrafields->addExtraField('fk_zone', "Zone de livraison préférée", 'int', 1, 10, 'thirdparty', 0, 0, 0, '', 0, '', 0);

		return $this->_init($sql, $options);
	}

	/**
	 * Function called when module is disabled.
	 * Remove from database constants, boxes and permissions from Dolibarr database.
	 * Data directories are not deleted
	 *
	 * @param      string	$options    Options when enabling module ('', 'noboxes')
	 * @return     int             	1 if OK, 0 if KO
	 */
	public function remove($options = '')
	{
		$sql = array();

		return $this->_remove($sql, $options);
	}

}
