<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017 Mikael Carlavan <contact@mika-carl.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       htdocs/poidstotal/admin/setup.php
 *  \ingroup    poidstotal
 *  \brief      Admin page
 */


$res=@include("../../main.inc.php");                   // For root directory
if (! $res) $res=@include("../../../main.inc.php");    // For "custom" directory

// Libraries
require_once DOL_DOCUMENT_ROOT . "/core/lib/admin.lib.php";
require_once DOL_DOCUMENT_ROOT .'/product/class/html.formproduct.class.php';
require_once DOL_DOCUMENT_ROOT .'/core/lib/product.lib.php';


dol_include_once("/poidstotal/lib/poidstotal.lib.php");

// Translations
$langs->load("poidstotal@poidstotal");
$langs->load("errors");

$langs->load("admin");
$langs->load("other");

// Access control
if (! $user->admin) accessforbidden();

// Parameters
$action = GETPOST('action', 'alpha');


/*
 * Actions
 */

include DOL_DOCUMENT_ROOT.'/core/actions_setmoduleoptions.inc.php';

// Action mise a jour ou ajout d'une constante
if ($action == 'update')
{
	$constname=GETPOST('constname','alpha');
	$constvalue=(GETPOST('constvalue_'.$constname) ? GETPOST('constvalue_'.$constname) : GETPOST('constvalue'));


	$consttype=GETPOST('consttype','alpha');
	$constnote=GETPOST('constnote');
	$res = dolibarr_set_const($db,$constname,$constvalue,'chaine',0,$constnote,$conf->entity);

	if (! $res > 0) $error++;

	if (! $error)
	{
		setEventMessages($langs->trans("SetupSaved"), null, 'mesgs');
	}
	else
	{
		setEventMessages($langs->trans("Error"), null, 'errors');
	}
}

/*
 * View
 */

llxHeader('', $langs->trans('PoidsTotalSetup'));

// Subheader
$linkback = '<a href="' . DOL_URL_ROOT . '/admin/modules.php">' . $langs->trans("BackToModuleList") . '</a>';

print load_fiche_titre($langs->trans('PoidsTotalSetup'), $linkback);

// Configuration header
$head = poidstotal_prepare_admin_head();
dol_fiche_head(
	$head,
	'settings',
	$langs->trans("ModulePoidsTotalName"),
	0,
	"poidstotal@poidstotal"
);

// Setup page goes here
echo $langs->trans("PoidsTotalSetupPage");


$constantes = array(
	'POIDSTOTAL_UNIT_ID',
);

print load_fiche_titre($langs->trans("PoidsTotalOptions"),'','');


$formproduct = new FormProduct($db);
$form = new Form($db);

print '<table class="noborder" width="100%">';
print '<tr class="liste_titre">';
print '<td>'.$langs->trans("Description").'</td>';
print '<td>';
$text = $langs->trans("Value");
print $form->textwithpicto($text, $helptext, 1, 'help', '', 0, 2, 'idhelptext');
print '</td>';
print '<td align="center" width="80">'.$langs->trans("Action").'</td>';
print "</tr>\n";

print '<form action="'.$_SERVER["PHP_SELF"].'" method="POST">';

print '<tr class="oddeven">';

// Show constant
print '<td>';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
print '<input type="hidden" name="action" value="update">';
print '<input type="hidden" name="constname" value="POIDSTOTAL_UNIT_ID">';
print '<input type="hidden" name="constnote" value="">';

print $langs->trans('DescPOIDSTOTAL_UNIT_ID');
print '</td>';
print '<td>';
print $formproduct->select_measuring_units("constvalue", "weight", $conf->global->POIDSTOTAL_UNIT_ID, 0, 2);
print '<input type="hidden" name="consttype" value="chaine">';
print '</td>';
print '<td align="center">';
print '<input type="submit" class="button" value="'.$langs->trans("Update").'" name="Button">';
print '</td>';
print '</tr>';
print '</form>';

print '<tr class="oddeven">';
// Show constant
print '<td>';
print $langs->trans('DescPOIDSTOTAL_DISABLE_WEIGHT_AUTO_UPDATE');
print '</td>';
print '<td colspan="2">';
if ($conf->use_javascript_ajax)
{
    print ajax_constantonoff('POIDSTOTAL_DISABLE_WEIGHT_AUTO_UPDATE');
}
else
{
    if (empty($conf->global->POIDSTOTAL_DISABLE_WEIGHT_AUTO_UPDATE))
    {
        print '<a href="'.$_SERVER['PHP_SELF'].'?action=set_POIDSTOTAL_DISABLE_WEIGHT_AUTO_UPDATE">'.img_picto($langs->trans("Disabled"),'switch_off').'</a>';
    }
    else
    {
        print '<a href="'.$_SERVER['PHP_SELF'].'?action=del_POIDSTOTAL_DISABLE_WEIGHT_AUTO_UPDATE">'.img_picto($langs->trans("Enabled"),'switch_on').'</a>';
    }
}
print '</td>';
print '</tr>';

print '</table>';


// Page end
dol_fiche_end();
llxFooter();
