<?php
/* Copyright (C) 2017 Mikael Carlavan <contact@mika-carl.fr>
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * \file    core/triggers/interface_99_modPoidsTotal_PoidsTotalTriggers.class.php
 * \ingroup poidstotal
 * \brief   Example trigger.
 *
 */

require_once DOL_DOCUMENT_ROOT.'/core/triggers/dolibarrtriggers.class.php';
require_once DOL_DOCUMENT_ROOT.'/comm/propal/class/propal.class.php';
require_once DOL_DOCUMENT_ROOT .'/core/lib/product.lib.php';
require_once DOL_DOCUMENT_ROOT.'/product/class/product.class.php';
require_once DOL_DOCUMENT_ROOT.'/commande/class/commande.class.php';

/**
 *  Class of triggers for PoidsTotal module
 */
class InterfacePoidsTotalTriggers extends DolibarrTriggers
{
	/**
	 * @var DoliDB Database handler
	 */
	protected $db;

	/**
	 * Constructor
	 *
	 * @param DoliDB $db Database handler
	 */
	public function __construct($db)
	{
		$this->db = $db;

		$this->name = preg_replace('/^Interface/i', '', get_class($this));
		$this->family = "products";
		$this->description = "PoidsTotal triggers.";
		// 'development', 'experimental', 'dolibarr' or version
		$this->version = '1.0.0';
		$this->picto = 'poidstotal@poidstotal';
	}

	/**
	 * Trigger name
	 *
	 * @return string Name of trigger file
	 */
	public function getName()
	{
		return $this->name;
	}

	/**
	 * Trigger description
	 *
	 * @return string Description of trigger file
	 */
	public function getDesc()
	{
		return $this->description;
	}


	/**
	 * Function called when a Dolibarrr business event is done.
	 * All functions "runTrigger" are triggered if file
	 * is inside directory core/triggers
	 *
	 * @param string 		$action 	Event action code
	 * @param CommonObject 	$object 	Object
	 * @param User 			$user 		Object user
	 * @param Translate 	$langs 		Object langs
	 * @param Conf 			$conf 		Object conf
	 * @return int              		<0 if KO, 0 if no triggered ran, >0 if OK
	 */
	public function runTrigger($action, $object, User $user, Translate $langs, Conf $conf)
	{

        if (empty($conf->poidstotal->enabled)) return 0;     // Module not active, we do nothing

	    // Put here code you want to execute when a Dolibarr business events occurs.
		// Data and type of action are stored into $object and $action

        $langs->load("other");

        switch ($action) {

		        // Proposals
		    case 'LINEPROPAL_INSERT':
		    case 'LINEPROPAL_UPDATE':
		    case 'LINEPROPAL_DELETE':

            case 'LINEORDER_INSERT':
            case 'LINEORDER_UPDATE':
            case 'LINEORDER_DELETE':

		        dol_syslog("Trigger '".$this->name."' for action '$action' launched by ".__FILE__.". id=".$object->id);

		        $langs->load("other");
		        /*
		        $poidsproduct = GETPOST('poidsproduct');
		        
		        //poids du produit libre
                if ($action == 'LINEPROPAL_INSERT') {
		        	dol_syslog("Trigger '".$this->name."' for action '$action' launched by ".__FILE__.". id=".$object->id, LOG_DEBUG);
					
					//On ajoute le décompte en négoce ou en fourniture pour le recap fpt
					$prod_entry_mode = GETPOST('prod_entry_mode');
					if ($prod_entry_mode == 'free')//pour les produits libres uniquement
					{
						$object->array_options['options_poidsproduct'] = $poidsproduct;
						$object->updateExtraField('poidsproduct');
					}
		    	}

				// Lorsqu'on edite un produit libre
		    	if ($action =='LINEPROPAL_UPDATE') {
                    if(empty($object->fk_product)){
                        $object->array_options['options_poidsproduct']=$poidsproduct;
                        $object->updateExtraField('poidsproduct');
                    }
		    	}
                */

                if (empty($conf->global->POIDSTOTAL_DISABLE_WEIGHT_AUTO_UPDATE)) {
                    $fk_object = $object->element == 'propaldet' ? $object->fk_propal : $object->fk_commande;

                    $objectsrc = $object->element == 'propaldet' ? new Propal($this->db) : new Commande($this->db);
                    $objectsrc->fetch($fk_object);

                    $lines = $objectsrc->lines;
                    $weight = 0;

                    for ($i = 0; $i < sizeof($lines); $i++)
                    {
                        $line = $lines[$i];

                        if ($line->fk_product > 0 && $line->product_type == 0)
                        {
                            $product = new Product($this->db);
                            $result = $product->fetch($line->fk_product);

                            if ($result > 0)
                            {
                                // Convert to kg
                                if ($product->weight_units == 98)
                                {
                                    $product_weight = $product->weight * 0.0283495;
                                }
                                else if ($product->weight_units == 99)
                                {
                                    $product_weight = $product->weight * 0.453592;
                                }
                                else
                                {
                                    $product_weight = $product->weight_units != 0 ? $product->weight * (10 ** $product->weight_units) : $product->weight;
                                }

                                $weight += $line->qty * $product_weight;
                            }
                        } elseif ($line->fk_product <= 0 && $line->array_options['options_poidsproduct'] > 0) {//cas des produits libres dont le poids a été saisi
                            $product_weight = $line->array_options['options_poidsproduct'];
                            $product_weight = preg_replace('/[^0-9^.^,]/', '', $product_weight);
                            $product_weight = price2num($product_weight);

                            $weight += $line->qty * $product_weight;
                        }
                    }

                    // Convert to desired unit
                    if ($conf->global->POIDSTOTAL_UNIT_ID == 98)
                    {
                        $weight = $weight/0.0283495;
                    }
                    else if ($conf->global->POIDSTOTAL_UNIT_ID == 99)
                    {
                        $weight = $weight/0.453592;
                    }
                    else
                    {
                        $weight = $conf->global->POIDSTOTAL_UNIT_ID != 0 ? $weight  * (10 ** -$conf->global->POIDSTOTAL_UNIT_ID) : $weight;
                    }

                    $weight_string = price($weight).' '.measuring_units_string($conf->global->POIDSTOTAL_UNIT_ID, 'weight');
                    $array_options = $objectsrc->array_options;
                    $array_options['options_poids_total'] = $weight_string;
                    $objectsrc->array_options = $array_options;
                    $objectsrc->insertExtraFields();
                }
				

		        break;

		    }

		return 0;
	}
}
