<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2019 Toto SuperAdmin <marie@inovea-conseil.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * \file    tpdeduplication/admin/setup.php
 * \ingroup tpdeduplication
 * \brief   Tpdeduplication setup page.
 */

// Load Dolibarr environment
$res = 0;
// Try main.inc.php into web root known defined into CONTEXT_DOCUMENT_ROOT (not always defined)
if (!$res && !empty($_SERVER["CONTEXT_DOCUMENT_ROOT"])) $res = @include $_SERVER["CONTEXT_DOCUMENT_ROOT"] . "/main.inc.php";
// Try main.inc.php into web root detected using web root calculated from SCRIPT_FILENAME
$tmp = empty($_SERVER['SCRIPT_FILENAME']) ? '' : $_SERVER['SCRIPT_FILENAME'];
$tmp2 = realpath(__FILE__);
$i = strlen($tmp) - 1;
$j = strlen($tmp2) - 1;
while ($i > 0 && $j > 0 && isset($tmp[$i]) && isset($tmp2[$j]) && $tmp[$i] == $tmp2[$j]) {
    $i--;
    $j--;
}
if (!$res && $i > 0 && file_exists(substr($tmp, 0, ($i + 1)) . "/main.inc.php")) $res = @include substr($tmp, 0, ($i + 1)) . "/main.inc.php";
if (!$res && $i > 0 && file_exists(dirname(substr($tmp, 0, ($i + 1))) . "/main.inc.php")) $res = @include dirname(substr($tmp, 0, ($i + 1))) . "/main.inc.php";
// Try main.inc.php using relative path
if (!$res && file_exists("../../main.inc.php")) $res = @include "../../main.inc.php";
if (!$res && file_exists("../../../main.inc.php")) $res = @include "../../../main.inc.php";
if (!$res) die("Include of main fails");

global $langs, $user;

// Libraries
require_once DOL_DOCUMENT_ROOT . "/core/lib/admin.lib.php";
require_once '../lib/tpdeduplication.lib.php';
//require_once "../class/myclass.class.php";

// Translations
$langs->loadLangs(array("admin", "tpdeduplication@tpdeduplication"));

// Access control
if (!$user->admin) accessforbidden();

// Parameters
$action = GETPOST('action', 'alpha');
$backtopage = GETPOST('backtopage', 'alpha');

if (((preg_match('/MariaDB/', $db->getVersion()) && (float) $db->getVersion() >= 5.5) || (preg_match('/MySQL/', $db->getVersion()) && (float) $db->getVersion() >= 8.0)) /*&& false*/) {
    $arrayofparameters = array(
        'TPDEDUPLICATION_PROVIDER_CODE' => array('css' => 'minwidth200', 'type' => "checkbox", 'enabled' => 1, 'val' => 'code_fournisseur'),
        'TPDEDUPLICATION_ADDRESS' => array('css' => 'minwidth200', 'type' => "checkbox", 'enabled' => 1, 'val' => 'address'),
        'TPDEDUPLICATION_CP' => array('css' => 'minwidth200', 'type' => "checkbox", 'enabled' => 1, 'val' => 'zip'),
        'TPDEDUPLICATION_CITY' => array('css' => 'minwidth200', 'type' => "checkbox", 'enabled' => 1, 'val' => 'town'),
        'TPDEDUPLICATION_COUNTRY' => array('css' => 'minwidth200', 'type' => "checkbox", 'enabled' => 1, 'val' => 'fk_pays'),
        'TPDEDUPLICATION_DEPARTEMENT' => array('css' => 'minwidth200', 'type' => "checkbox", 'enabled' => 1, 'val' => 'fk_departement'),
        'TPDEDUPLICATION_EMAIL' => array('css' => 'minwidth200', 'type' => "checkbox", 'enabled' => 1, 'val' => 'email'),
        'TPDEDUPLICATION_PHONE' => array('css' => 'minwidth200', 'type' => "checkbox", 'enabled' => 1, 'val' => 'phone'),
        'TPDEDUPLICATION_SIREN' => array('css' => 'minwidth200', 'type' => "checkbox", 'enabled' => 1, 'val' => 'siren'),
        'TPDEDUPLICATION_SIRET' => array('css' => 'minwidth200', 'type' => "checkbox", 'enabled' => 1, 'val' => 'siret'),
        'TPDEDUPLICATION_NAF_APE' => array('css' => 'minwidth200', 'type' => "checkbox", 'enabled' => 1, 'val' => 'ape'),
        'TPDEDUPLICATION_RCS-RM' => array('css' => 'minwidth200', 'type' => "checkbox", 'enabled' => 1, 'val' => 'idprof4'),
        'TPDEDUPLICATION_NUMTVA' => array('css' => 'minwidth200', 'type' => "checkbox", 'enabled' => 1, 'val' => 'tva_intra'),
        'TPDEDUPLICATION_TP_STATUS' => array('css' => 'minwidth200', 'type' => "checkbox", 'enabled' => 1, 'val' => 'statut'),
        'TPDEDUPLICATION_TP_NAME' => array('css' => 'minwidth200', 'type' => "checkbox", 'enabled' => 1, 'val' => 'nom'),
    );
} else {
    $arrayofparameters = array(
        'TPDEDUPLICATION_PROVIDER_CODE' => array('css' => 'minwidth200', 'type' => radio, 'enabled' => 1, 'val' => 'code_fournisseur'),
        'TPDEDUPLICATION_ADDRESS' => array('css' => 'minwidth200', 'type' => radio, 'enabled' => 1, 'val' => 'address'),
        'TPDEDUPLICATION_CP' => array('css' => 'minwidth200', 'type' => radio, 'enabled' => 1, 'val' => 'zip'),
        'TPDEDUPLICATION_CITY' => array('css' => 'minwidth200', 'type' => radio, 'enabled' => 1, 'val' => 'town'),
        'TPDEDUPLICATION_COUNTRY' => array('css' => 'minwidth200', 'type' => radio, 'enabled' => 1, 'val' => 'fk_pays'),
        'TPDEDUPLICATION_DEPARTEMENT' => array('css' => 'minwidth200', 'type' => radio, 'enabled' => 1, 'val' => 'fk_departement'),
        'TPDEDUPLICATION_EMAIL' => array('css' => 'minwidth200', 'type' => radio, 'enabled' => 1, 'val' => 'email'),
        'TPDEDUPLICATION_PHONE' => array('css' => 'minwidth200', 'type' => radio, 'enabled' => 1, 'val' => 'phone'),
        'TPDEDUPLICATION_SIREN' => array('css' => 'minwidth200', 'type' => radio, 'enabled' => 1, 'val' => 'siren'),
        'TPDEDUPLICATION_SIRET' => array('css' => 'minwidth200', 'type' => radio, 'enabled' => 1, 'val' => 'siret'),
        'TPDEDUPLICATION_NAF_APE' => array('css' => 'minwidth200', 'type' => radio, 'enabled' => 1, 'val' => 'ape'),
        'TPDEDUPLICATION_RCS-RM' => array('css' => 'minwidth200', 'type' => radio, 'enabled' => 1, 'val' => 'idprof4'),
        'TPDEDUPLICATION_NUMTVA' => array('css' => 'minwidth200', 'type' => radio, 'enabled' => 1, 'val' => 'tva_intra'),
        'TPDEDUPLICATION_TP_STATUS' => array('css' => 'minwidth200', 'type' => radio, 'enabled' => 1, 'val' => 'statut'),
        'TPDEDUPLICATION_TP_NAME' => array('css' => 'minwidth200', 'type' => radio, 'enabled' => 1, 'val' => 'nom'),
    );
}


/*
 * Actions
 */
include DOL_DOCUMENT_ROOT . '/core/actions_setmoduleoptions.inc.php';

if ((int) DOL_VERSION < 7 && $action == 'update' && ((preg_match('/MariaDB/', $db->getVersion()) && (float) $db->getVersion() >= 5.5) || (preg_match('/MySQL/', $db->getVersion()) && (float) $db->getVersion() >= 8.0)) /*&& false*/) {
    foreach ($arrayofparameters as $k => $v) {
        $res = dolibarr_set_const($db, $k, GETPOST($k), 'int', 0, '', $conf->entity);
        if (!$res > 0) $error++;
    }

    if (!$error) {
        $db->commit();
        if (empty($nomessageinsetmoduleoptions)) setEventMessages($langs->trans("SetupSaved"), null, 'mesgs');
    } else {
        $db->rollback();
        if (empty($nomessageinsetmoduleoptions)) setEventMessages($langs->trans("SetupNotSaved"), null, 'errors');
    }
} else {
    $res = dolibarr_set_const($db, 'DB_TPDEDUPLICATION_OPTION_DEDUPLICATE', GETPOST('DB_TPDEDUPLICATION_OPTION_DEDUPLICATE'), 'int', 0, '', $conf->entity);
    $keyFound = false;
    foreach ($arrayofparameters as $key => $val) {
        if ($val['val'] == GETPOST('DB_TPDEDUPLICATION_OPTION_DEDUPLICATE')) {
            $keyFound = $key;
        }
    }
    if ($keyFound) {
        $res = dolibarr_set_const($db, $keyFound, 1, 'int', 0, '', $conf->entity);
    }
    if (!$res > 0) $error++;
    if (!$error) {
        $db->commit();
        if (empty($nomessageinsetmoduleoptions)) setEventMessages($langs->trans("SetupSaved"), null, 'mesgs');
    } else {
        $db->rollback();
        if (empty($nomessageinsetmoduleoptions)) setEventMessages($langs->trans("SetupNotSaved"), null, 'errors');
    }
}

/*
 * View
 */

$page_name = "TpdeduplicationSetup";
llxHeader('', $langs->trans($page_name));

// Subheader
$linkback = '<a href="' . ($backtopage ? $backtopage : DOL_URL_ROOT . '/admin/modules.php?restore_lastsearch_values=1') . '">' . $langs->trans("BackToModuleList") . '</a>';

print load_fiche_titre($langs->trans($page_name), $linkback, 'object_tpdeduplication@tpdeduplication');

// Configuration header
$head = tpdeduplicationAdminPrepareHead();
dol_fiche_head($head, 'settings', '', -1, "tpdeduplication@tpdeduplication");

// Setup page goes here
echo '<span class="opacitymedium">' . $langs->trans("TpdeduplicationSetupPage") . '</span><br><br>';


if ($action == 'edit') {
    print '<form method="POST" action="' . $_SERVER["PHP_SELF"] . '">';
    print '<input type="hidden" name="token" value="' . $_SESSION['newtoken'] . '">';
    print '<input type="hidden" name="action" value="update">';

    print '<table class="noborder" width="100%">';
    print '<tr class="liste_titre"><td class="titlefield">' . $langs->trans("Parameter") . '</td><td>' . $langs->trans("Value") . '</td></tr>';

    foreach ($arrayofparameters as $key => $val) {
        print '<tr class="oddeven"><td>';
        $tooltiphelp = (($langs->trans($key . 'Tooltip') != $key . 'Tooltip') ? $langs->trans($key . 'Tooltip') : '');
        print $form->textwithpicto($langs->trans($key), $tooltiphelp);
        print '</td><td>';
        switch ($val['type']) {
            case 'textarea' :
                print '<textarea name="' . $key . '"  class="flat ' . (empty($val['css']) ? 'minwidth200' : $val['css']) . '" rows="8">';
                print $conf->global->$key;
                print '</textarea>';
                break;

            case 'checkbox' :
                print '<input type="hidden" name="' . $key . '"  class="flat" value="0" >';
                print '<input type="checkbox" name="' . $key . '"  class="flat" value="' . $val['val'] . '"' . ($conf->global->$key == $val['val'] ? 'checked="checked"' : '') . '>';
                break;

            case 'select' :
                print '<select name="' . $key . '"  class="flat ' . (empty($val['css']) ? 'minwidth200' : $val['css']) . '">';
                foreach ($val['value'] as $value => $label) {
                    print '<option value="' . $value . '" ' . ($conf->global->$key == $value ? 'selected="selected"' : '') . '>';
                    print $langs->trans($label);
                    print '</option>';
                }
                print '</select>';
                break;

            case 'radio' :
                print '<input type="hidden" name="' . $key . '"  class="flat" value="0" >';
                print '<input type="radio" name="DB_TPDEDUPLICATION_OPTION_DEDUPLICATE"  class="flat" value="' . $val['val'] . '"' . ($conf->global->$key == 1 ? 'checked="checked"' : '') . '>';
                break;

            default:
                print '<input name="' . $key . '"  class="flat ' . (empty($val['css']) ? 'minwidth200' : $val['css']) . '" value="' . $conf->global->$key . '">';
        }
        print '</td></tr>';
    }
    print '</table>';

    print '<br><div class="center">';
    print '<input class="button" type="submit" value="' . $langs->trans("Save") . '">';
    print '</div>';

    print '</form>';
    print '<br>';
} else {
    if (!empty($arrayofparameters)) {
        print '<table class="noborder centpercent">';
        print '<tr class="liste_titre"><td class="titlefield">' . $langs->trans("Parameter") . '</td><td>' . $langs->trans("Value") . '</td></tr>';

        foreach ($arrayofparameters as $key => $val) {
            print '<tr class="oddeven"><td>';
            $tooltiphelp = (($langs->trans($key . 'Tooltip') != $key . 'Tooltip') ? $langs->trans($key . 'Tooltip') : '');
            print $form->textwithpicto($langs->trans($key), $tooltiphelp);
            print '</td><td>' . $conf->global->$key . '</td></tr>';
        }

        print '</table>';

        print '<div class="tabsAction">';
        print '<a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?action=edit">' . $langs->trans("Modify") . '</a>';
        print '</div>';
    } else {
        print '<br>' . $langs->trans("NothingToSetup");
    }
}


// Page end
dol_fiche_end();

llxFooter();
$db->close();
