<?php

/**
 * Class TextCleaner
 *
 * TextCleaner is used to clean text for an SQL request
 */
class TextCleaner
{
    /**
     * @var array contains multiple array which contains group of diacritics character to change into one character
     */
    var $tabsRegexpReplace = array();

    /**
     * TextCleaner constructor.
     */
    public function __construct()
    {
        $this->tabsRegexpReplace = array(
            array('[éèêë]', 'e'),
            array('[àáâäåãæ]', 'a'),
            array('[íìîï]', 'i'),
            array('[óòôöõøœ]', 'o'),
            array('[úùûü]', 'u'),
            array('[ç¢]', 'c'),
            array('[ýÿ]', 'y'),
            array('[ñ]', 'n'),
            array('[ß]','ss'),
        );
    }

    /**
     * Used to prepare an SQL Request with transform functions sure as LOWER, REPLACE, REGEXP_REPLACE, ...
     *
     * @param null $valueToClean
     * @param int $tabsReplace
     * @param int $needsRegexpReplace
     * @param null $tabsRegexp
     * @param null $othersFunction
     * @param int $testSQLNoNULL
     * @return int|string|null
     */
    public function prepareAndCleanSQL($valueToClean = null, $tabsReplace = 0, $needsRegexpReplace = 0, $tabsRegexp = null, $othersFunction = null, $testSQLNoNULL = 0)
    {
        $val = $valueToClean;
        if ($valueToClean == null) {
            echo 'no value';
            return 0;
        }
        if ($othersFunction != null){
            foreach ($othersFunction as $key => $value) {
                $valueToClean = $value . '(' . $valueToClean . ')';
            }
        }
        if ($tabsReplace != null){
            foreach ($tabsReplace as $key => $value) {
                $valueToClean = "REPLACE(" . $valueToClean . ",'" . $value[0] . "','" . $value[1] . "')";
            }
        }
        if ($needsRegexpReplace) {
            foreach ($this->tabsRegexpReplace as $key => $value) {
                $valueToClean = "REGEXP_REPLACE(" . $valueToClean . ",'" . $value[0] . "','" . $value[1] . "')";
            }
        }
        if ($tabsRegexp != null){
            foreach ($tabsRegexp as $key => $value) {
                $valueToClean = "REGEXP_REPLACE(" . $valueToClean . ",'" . $value[0] . "','" . $value[1] . "')";
            }
        }
        if ($testSQLNoNULL) {
            $valueToClean = $valueToClean . " AND " . $val . " IS NOT NULL" . " AND " . $val . " != ''";
        }
        return $valueToClean;
    }
}
