<?php
/* Copyright (C) 2019 Toto SuperAdmin <marie@inovea-conseil.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * \file    tpdeduplication/class/actions_tpdeduplication.class.php
 * \ingroup tpdeduplication
 * \brief   Example hook overload.
 *
 * Put detailed description here.
 */

/**
 * Class ActionsTpdeduplication
 */
class ActionsTpdeduplication
{
    /**
     * @var DoliDB Database handler.
     */
    public $db;

    /**
     * @var string Error code (or message)
     */
    public $error = '';

    /**
     * @var array Errors
     */
    public $errors = array();


    /**
     * @var array Hook results. Propagated to $hookmanager->resArray for later reuse
     */
    public $results = array();

    /**
     * @var string String displayed by executeHook() immediately after return
     */
    public $resprints;


    /**
     * Constructor
     *
     *  @param		DoliDB		$db      Database handler
     */
    public function __construct($db)
    {
        $this->db = $db;
    }


    /**
     * Execute action
     *
     * @param	array			$parameters		Array of parameters
     * @param	CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
     * @param	string			$action      	'add', 'update', 'view'
     * @return	int         					<0 if KO,
     *                           				=0 if OK but we want to process standard actions too,
     *                            				>0 if OK and we want to replace standard actions.
     */
    public function getNomUrl($parameters, &$object, &$action)
    {
        global $db,$langs,$conf,$user;
        $this->resprints = '';
        return 0;
    }

    /**
     * Overloading the doActions function : replacing the parent's function with the one below
     *
     * @param   array           $parameters     Hook metadatas (context, etc...)
     * @param   CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
     * @param   string          $action         Current action (if set). Generally create or edit or null
     * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
     * @return  int                             < 0 on error, 0 on success, 1 to replace standard code
     */
    public function doActions($parameters, &$object, &$action, $hookmanager)
    {
        global $conf, $user, $langs;

        $error = 0; // Error counter

        /* print_r($parameters); print_r($object); echo "action: " . $action; */
        if (in_array($parameters['currentcontext'], array('somecontext1','somecontext2')))	    // do something only for the context 'somecontext1' or 'somecontext2'
        {
            // Do what you want here...
            // You can for example call global vars like $fieldstosearchall to overwrite them, or update database depending on $action and $_POST values.
        }

        if (! $error) {
            $this->results = array('myreturn' => 999);
            $this->resprints = 'A text to show';
            return 0; // or return 1 to replace standard code
        } else {
            $this->errors[] = 'Error message';
            return -1;
        }
    }


    /**
     * Overloading the doActions function : replacing the parent's function with the one below
     *
     * @param   array           $parameters     Hook metadatas (context, etc...)
     * @param   CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
     * @param   string          $action         Current action (if set). Generally create or edit or null
     * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
     * @return  int                             < 0 on error, 0 on success, 1 to replace standard code
     */
    public function doMassActions($parameters, &$object, &$action, $hookmanager)
    {
        global $conf, $user, $langs;

        $error = 0; // Error counter

        /* print_r($parameters); print_r($object); echo "action: " . $action; */
        if (in_array($parameters['currentcontext'], array('somecontext1','somecontext2')))		// do something only for the context 'somecontext1' or 'somecontext2'
        {
            foreach($parameters['toselect'] as $objectid)
            {
                // Do action on each object id
            }
        }

        if (! $error) {
            $this->results = array('myreturn' => 999);
            $this->resprints = 'A text to show';
            return 0; // or return 1 to replace standard code
        } else {
            $this->errors[] = 'Error message';
            return -1;
        }
    }


    /**
     * Overloading the addMoreMassActions function : replacing the parent's function with the one below
     *
     * @param   array           $parameters     Hook metadatas (context, etc...)
     * @param   CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
     * @param   string          $action         Current action (if set). Generally create or edit or null
     * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
     * @return  int                             < 0 on error, 0 on success, 1 to replace standard code
     */
    public function addMoreMassActions($parameters, &$object, &$action, $hookmanager)
    {
        global $conf, $user, $langs;

        $error = 0; // Error counter

        /* print_r($parameters); print_r($object); echo "action: " . $action; */
        if (in_array($parameters['currentcontext'], array('somecontext1','somecontext2')))		// do something only for the context 'somecontext1' or 'somecontext2'
        {
            $this->resprints = '<option value="0"'.($disabled?' disabled="disabled"':'').'>'.$langs->trans("TpdeduplicationMassAction").'</option>';
        }

        if (! $error) {
            return 0; // or return 1 to replace standard code
        } else {
            $this->errors[] = 'Error message';
            return -1;
        }
    }



    /**
     * Execute action
     *
     * @param	array	$parameters     Array of parameters
     * @param   Object	$object		   	Object output on PDF
     * @param   string	$action     	'add', 'update', 'view'
     * @return  int 		        	<0 if KO,
     *                          		=0 if OK but we want to process standard actions too,
     *  	                            >0 if OK and we want to replace standard actions.
     */
    public function beforePDFCreation($parameters, &$object, &$action)
    {
        global $conf, $user, $langs;
        global $hookmanager;

        $outputlangs=$langs;

        $ret=0; $deltemp=array();
        dol_syslog(get_class($this).'::executeHooks action='.$action);

        /* print_r($parameters); print_r($object); echo "action: " . $action; */
        if (in_array($parameters['currentcontext'], array('somecontext1','somecontext2')))		// do something only for the context 'somecontext1' or 'somecontext2'
        {
        }

        return $ret;
    }

    /**
     * Execute action
     *
     * @param	array	$parameters     Array of parameters
     * @param   Object	$pdfhandler     PDF builder handler
     * @param   string	$action         'add', 'update', 'view'
     * @return  int 		            <0 if KO,
     *                                  =0 if OK but we want to process standard actions too,
     *                                  >0 if OK and we want to replace standard actions.
     */
    public function afterPDFCreation($parameters, &$pdfhandler, &$action)
    {
        global $conf, $user, $langs;
        global $hookmanager;

        $outputlangs=$langs;

        $ret=0; $deltemp=array();
        dol_syslog(get_class($this).'::executeHooks action='.$action);

        /* print_r($parameters); print_r($object); echo "action: " . $action; */
        if (in_array($parameters['currentcontext'], array('somecontext1','somecontext2')))		// do something only for the context 'somecontext1' or 'somecontext2'
        {
        }

        return $ret;
    }

    /**
     * Used when 2 thirdparty are merged into one
     *
     * @param $parameters
     * @param $object
     * @param $action
     */
    public function replaceThirdparty($parameters, &$object, &$action)
    {
        global $db;

        //Remove thirdparty in table of deduplication
        $sql = "DELETE FROM " . MAIN_DB_PREFIX . "tpdeduplication_notduplicatethirdparty WHERE fk_soc1 = ". $parameters['soc_origin'] . " OR fk_soc2 = " . $parameters['soc_origin'];
        $results = $db->query($sql);
        if (!$results) {
            dol_print_error($db);
            exit;
        }
        $db->free();

        //Update table of contacttracking if exist
        $sql = "SELECT fk_soc FROM " . MAIN_DB_PREFIX . "contacttracking";
        $results = $db->query($sql);
        if ($results) {
            $sql = "UPDATE ". MAIN_DB_PREFIX . "contacttracking SET `fk_soc`=" . $parameters['soc_dest'] . " WHERE fk_soc=" . $parameters['soc_origin'];
            $res = $db->query($sql);
            if (!$res) {
                dol_print_error($db);
                exit;
            }
        }
        $db->free();

        //Update table of ActivityReport if exist
        $sql = "SELECT fk_soc FROM " . MAIN_DB_PREFIX . "activityreport";
        $results = $db->query($sql);
        if ($results) {
            $sql = "UPDATE ". MAIN_DB_PREFIX . "activityreport SET `fk_soc`=" . $parameters['soc_dest'] . " WHERE fk_soc=" . $parameters['soc_origin'];
            $res = $db->query($sql);
            if (!$res) {
                dol_print_error($db);
                exit;
            }
        }
        $db->free();

        //Update table of deplacement if exist
        $sql = "SELECT fk_soc FROM " . MAIN_DB_PREFIX . "deplacement";
        $results = $db->query($sql);
        if ($results) {
            $sql = "UPDATE ". MAIN_DB_PREFIX . "deplacement SET `fk_soc`=" . $parameters['soc_dest'] . " WHERE fk_soc=" . $parameters['soc_origin'];
            $res = $db->query($sql);
            if (!$res) {
                dol_print_error($db);
                exit;
            }
        }
        $db->free();

        //Update table of ticket if exist
        $sql = "SELECT fk_soc FROM " . MAIN_DB_PREFIX . "ticket";
        $results = $db->query($sql);
        if ($results) {
            $sql = "UPDATE ". MAIN_DB_PREFIX . "ticket SET `fk_soc`=" . $parameters['soc_dest'] . " WHERE fk_soc=" . $parameters['soc_origin'];
            $res = $db->query($sql);
            if (!$res) {
                dol_print_error($db);
                exit;
            }
        }
        $db->free();

        //Update table of don if exist
        $sql = "SELECT fk_soc FROM " . MAIN_DB_PREFIX . "don";
        $results = $db->query($sql);
        if ($results) {
            $sql = "UPDATE ". MAIN_DB_PREFIX . "don SET `fk_soc`=" . $parameters['soc_dest'] . " WHERE fk_soc=" . $parameters['soc_origin'];
            $res = $db->query($sql);
            if (!$res) {
                dol_print_error($db);
                exit;
            }
        }
        $db->free();

        //Update table of categorie fournisseur if exist
        $sql = "SELECT fk_soc FROM " . MAIN_DB_PREFIX . "categorie_fournisseur WHERE fk_soc = " . $parameters['soc_dest'];
        $results = $db->query($sql);
        if ($results) {
            if ($db->num_rows($results) == 0) {
                $sql = "UPDATE ". MAIN_DB_PREFIX . "categorie_fournisseur SET `fk_soc`=" . $parameters['soc_dest'] . " WHERE fk_soc=" . $parameters['soc_origin'];
                $res = $db->query($sql);
                if (!$res) {
                    dol_print_error($db);
                    exit;
                }
            }
        }
        $db->free();
    }
}
