<?php
/* Copyright (C) 2004-2018  Laurent Destailleur     <eldy@users.sourceforge.net>
 * Copyright (C) 2018-2019  Nicolas ZABOURI         <info@inovea-conseil.com>
 * Copyright (C) 2019       Frédéric France         <frederic.france@netlogic.fr>
 * Copyright (C) 2019 Toto SuperAdmin <marie@inovea-conseil.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *    \defgroup   tpdeduplication     Module Tpdeduplication
 *  \brief      Tpdeduplication module descriptor.
 *
 *  \file       htdocs/tpdeduplication/core/modules/modTpdeduplication.class.php
 *  \ingroup    tpdeduplication
 *  \brief      Description and activation file for module Tpdeduplication
 */
include_once DOL_DOCUMENT_ROOT . '/core/modules/DolibarrModules.class.php';
include_once DOL_DOCUMENT_ROOT . '/core/lib/admin.lib.php';

/**
 *  Description and activation class for module Tpdeduplication
 */
class modTpdeduplication extends DolibarrModules
{
    /**
     * Constructor. Define names, constants, directories, boxes, permissions
     *
     * @param DoliDB $db Database handler
     */
    public function __construct($db)
    {
        global $langs, $conf, $user;
        $this->db = $db;

        // Id for module (must be unique).
        // Use here a free id (See in Home -> System information -> Dolibarr for list of used modules id).
        $this->numero = 432478; // TODO Go on page https://wiki.dolibarr.org/index.php/List_of_modules_id to reserve an id number for your module
        // Key text used to identify module (for permissions, menus, etc...)
        $this->rights_class = 'tpdeduplication';
        // Family can be 'base' (core modules),'crm','financial','hr','projects','products','ecm','technic' (transverse modules),'interface' (link with external tools),'other','...'
        // It is used to group modules by family in module setup page
        $this->family = "Inovea Conseil";
        // Module position in the family on 2 digits ('01', '10', '20', ...)
        $this->module_position = '90';
        // Gives the possibility for the module, to provide his own family info and position of this family (Overwrite $this->family and $this->module_position. Avoid this)
        //$this->familyinfo = array('myownfamily' => array('position' => '01', 'label' => $langs->trans("MyOwnFamily")));
        // Module label (no space allowed), used if translation string 'ModuleTpdeduplicationName' not found (Tpdeduplication is name of module).
        $this->name = preg_replace('/^mod/i', '', get_class($this));
        // Module description, used if translation string 'ModuleTpdeduplicationDesc' not found (Tpdeduplication is name of module).
        $this->description = "TpdeduplicationDescription";
        // Used only if file README.md and README-LL.md not found.
        $this->descriptionlong = "Tpdeduplication description (Long)";
        $this->editor_name = 'Inovea Conseil';
        $country = explode(":", $conf->global->MAIN_INFO_SOCIETE_COUNTRY);
        if ($country[0] == $conf->entity && $country[2] == "France")
            $this->editor_url = "https://www.inovea-conseil.com (<a target='_blank' href='https://www.dolibiz.com/wp-content/uploads/attestation/attestation-" . $this->name . "-" . $this->version . ".pdf'>Attestation NF525</a>)";
        else
            $this->editor_url = 'https://www.inovea-conseil.com';
        // Possible values for version are: 'development', 'experimental', 'dolibarr', 'dolibarr_deprecated' or a version string like 'x.y.z'
        $this->version = '1.0.1';
        // Url to the file with your last numberversion of this module
        $this->url_last_version = "https://www.dolibiz.com/wp-content/uploads/lastversion/last_version-tpdeduplication.txt";


        // Key used in llx_const table to save module status enabled/disabled (where TPDEDUPLICATION is value of property name of module in uppercase)
        $this->const_name = 'MAIN_MODULE_' . strtoupper($this->name);
        // Name of image file used for this module.
        // If file is in theme/yourtheme/img directory under name object_pictovalue.png, use this->picto='pictovalue'
        // If file is in module/img directory under name object_pictovalue.png, use this->picto='pictovalue@module'
        $this->picto = 'inoveaconseil@tpdeduplication';
        // Define some features supported by module (triggers, login, substitutions, menus, css, etc...)
        $this->module_parts = array(
            // Set this to 1 if module has its own trigger directory (core/triggers)
            'triggers' => 0,
            // Set this to 1 if module has its own login method file (core/login)
            'login' => 0,
            // Set this to 1 if module has its own substitution function file (core/substitutions)
            'substitutions' => 0,
            // Set this to 1 if module has its own menus handler directory (core/menus)
            'menus' => 0,
            // Set this to 1 if module overwrite template dir (core/tpl)
            'tpl' => 0,
            // Set this to 1 if module has its own barcode directory (core/modules/barcode)
            'barcode' => 0,
            // Set this to 1 if module has its own models directory (core/modules/xxx)
            'models' => 0,
            // Set this to 1 if module has its own theme directory (theme)
            'theme' => 0,
            // Set this to relative path of css file if module has its own css file
            'css' => array(//    '/tpdeduplication/css/tpdeduplication.css.php',
            ),
            // Set this to relative path of js file if module must load a js on all pages
            'js' => array(//   '/tpdeduplication/js/tpdeduplication.js.php',
            ),
            // Set here all hooks context managed by module. To find available hook context, make a "grep -r '>initHooks(' *" on source code. You can also set hook context to 'all'
            'hooks' => array(
                'data' => array(
                    'thirdpartycard',
                    'globalcard',
                    'tpduplicatethirdparty_page',
                ),
                //   'entity' => '0',
            ),
            // Set this to 1 if features of module are opened to external users
            'moduleforexternal' => 0,
        );
        // Data directories to create when module is enabled.
        // Example: this->dirs = array("/tpdeduplication/temp","/tpdeduplication/subdir");
        $this->dirs = array("/tpdeduplication/temp");
        // Config pages. Put here list of php page, stored into tpdeduplication/admin directory, to use to setup module.
        $this->config_page_url = array("setup.php@tpdeduplication");
        // Dependencies
        // A condition to hide module
        $this->hidden = false;
        // List of module class names as string that must be enabled if this module is enabled. Example: array('always1'=>'modModuleToEnable1','always2'=>'modModuleToEnable2', 'FR1'=>'modModuleToEnableFR'...)
        $this->depends = array();
        $this->requiredby = array();    // List of module class names as string to disable if this one is disabled. Example: array('modModuleToDisable1', ...)
        $this->conflictwith = array();    // List of module class names as string this module is in conflict with. Example: array('modModuleToDisable1', ...)
        $this->langfiles = array("tpdeduplication@tpdeduplication");
        $this->phpmin = array(5, 5);                        // Minimum version of PHP required by module
        $this->need_dolibarr_version = array(7, 0);        // Minimum version of Dolibarr required by module
        $this->warnings_activation = array();            // Warning to show when we activate module. array('always'='text') or array('FR'='textfr','ES'='textes'...)
        $this->warnings_activation_ext = array();        // Warning to show when we activate an external module. array('always'='text') or array('FR'='textfr','ES'='textes'...)
        //$this->automatic_activation = array('FR'=>'TpdeduplicationWasAutomaticallyActivatedBecauseOfYourCountryChoice');
        //$this->always_enabled = true;								// If true, can't be disabled

        // Constants
        // List of particular constants to add when module is enabled (key, 'chaine', value, desc, visible, 'current' or 'allentities', deleteonunactive)
        // Example: $this->const=array(1 => array('TPDEDUPLICATION_MYNEWCONST1', 'chaine', 'myvalue', 'This is a constant to add', 1),
        //                             2 => array('TPDEDUPLICATION_MYNEWCONST2', 'chaine', 'myvalue', 'This is another constant to add', 0, 'current', 1)
        // );
        $this->const = array(
            2 => array('TPDEDUPLICATION_PROVIDER_CODE', 'text', '0', 'This is a constant to check provider code for deduplication', 1, 'current'),
            3 => array('TPDEDUPLICATION_ADDRESS', 'text', '0', 'This is a constant to check addres for deduplication', 1, 'current'),
            4 => array('TPDEDUPLICATION_CP', 'text', '0', 'This is a constant to check postal code for deduplication', 1, 'current'),
            5 => array('TPDEDUPLICATION_CITY', 'text', '0', 'This is a constant to check city for deduplication', 1, 'current'),
            6 => array('TPDEDUPLICATION_COUNTRY', 'text', '0', 'This is a constant to check country for deduplication', 1, 'current'),
            7 => array('TPDEDUPLICATION_DEPARTEMENT', 'text', '0', 'This is a constant to check department for deduplication', 1, 'current'),
            8 => array('TPDEDUPLICATION_EMAIL', 'text', '0', 'This is a constant to check email for deduplication', 1, 'current'),
            9 => array('TPDEDUPLICATION_PHONE', 'text', '0', 'This is a constant to check phone for deduplication', 1, 'current'),
            10 => array('TPDEDUPLICATION_SIREN', 'text', '0', 'This is a constant to check siren for deduplication', 1, 'current'),
            11 => array('TPDEDUPLICATION_SIRET', 'text', '0', 'This is a constant to check siret for deduplication', 1, 'current'),
            12 => array('TPDEDUPLICATION_NAF_APE', 'text', '0', 'This is a constant to check naf-ape for deduplication', 1, 'current'),
            13 => array('TPDEDUPLICATION_RCS-RM', 'text', '0', 'This is a constant to check rcs-rm for deduplication', 1, 'current'),
            14 => array('TPDEDUPLICATION_NUMTVA', 'text', '0', 'This is a constant to check num tva for deduplication', 1, 'current'),
            16 => array('TPDEDUPLICATION_TP_STATUS', 'text', '0', 'This is a constant to check thirdparty status for deduplication', 1, 'current'),
            17 => array('TPDEDUPLICATION_TP_NAME', 'text', '0', 'This is a constant to check thirdparty name for deduplication', 1, 'current'),
        );

        // Some keys to add into the overwriting translation tables
        /*$this->overwrite_translation = array(
            'en_US:ParentCompany'=>'Parent company or reseller',
            'fr_FR:ParentCompany'=>'Maison mère ou revendeur'
        )*/

        if (!isset($conf->tpdeduplication) || !isset($conf->tpdeduplication->enabled)) {
            $conf->tpdeduplication = new stdClass();
            $conf->tpdeduplication->enabled = 0;
        }

        // Array to add new pages in new tabs
        $this->tabs = array();
        // Example:
        // $this->tabs[] = array('data'=>'objecttype:+tabname1:Title1:mylangfile@tpdeduplication:$user->rights->tpdeduplication->read:/tpdeduplication/mynewtab1.php?id=__ID__');  					// To add a new tab identified by code tabname1
        // $this->tabs[] = array('data'=>'objecttype:+tabname2:SUBSTITUTION_Title2:mylangfile@tpdeduplication:$user->rights->othermodule->read:/tpdeduplication/mynewtab2.php?id=__ID__',  	// To add another new tab identified by code tabname2. Label will be result of calling all substitution functions on 'Title2' key.
        // $this->tabs[] = array('data'=>'objecttype:-tabname:NU:conditiontoremove');                                                     										// To remove an existing tab identified by code tabname
        //
        // Where objecttype can be
        // 'categories_x'	  to add a tab in category view (replace 'x' by type of category (0=product, 1=supplier, 2=customer, 3=member)
        // 'contact'          to add a tab in contact view
        // 'contract'         to add a tab in contract view
        // 'group'            to add a tab in group view
        // 'intervention'     to add a tab in intervention view
        // 'invoice'          to add a tab in customer invoice view
        // 'invoice_supplier' to add a tab in supplier invoice view
        // 'member'           to add a tab in fundation member view
        // 'opensurveypoll'	  to add a tab in opensurvey poll view
        // 'order'            to add a tab in customer order view
        // 'order_supplier'   to add a tab in supplier order view
        // 'payment'		  to add a tab in payment view
        // 'payment_supplier' to add a tab in supplier payment view
        // 'product'          to add a tab in product view
        // 'propal'           to add a tab in propal view
        // 'project'          to add a tab in project view
        // 'stock'            to add a tab in stock view
        // 'thirdparty'       to add a tab in third party view
        // 'user'             to add a tab in user view

        // Dictionaries
        $this->dictionaries = array();
        /* Example:
        $this->dictionaries=array(
            'langs'=>'mylangfile@tpdeduplication',
            // List of tables we want to see into dictonnary editor
            'tabname'=>array(MAIN_DB_PREFIX."table1",MAIN_DB_PREFIX."table2",MAIN_DB_PREFIX."table3"),
            // Label of tables
            'tablib'=>array("Table1","Table2","Table3"),
            // Request to select fields
            'tabsql'=>array('SELECT f.rowid as rowid, f.code, f.label, f.active FROM '.MAIN_DB_PREFIX.'table1 as f','SELECT f.rowid as rowid, f.code, f.label, f.active FROM '.MAIN_DB_PREFIX.'table2 as f','SELECT f.rowid as rowid, f.code, f.label, f.active FROM '.MAIN_DB_PREFIX.'table3 as f'),
            // Sort order
            'tabsqlsort'=>array("label ASC","label ASC","label ASC"),
            // List of fields (result of select to show dictionary)
            'tabfield'=>array("code,label","code,label","code,label"),
            // List of fields (list of fields to edit a record)
            'tabfieldvalue'=>array("code,label","code,label","code,label"),
            // List of fields (list of fields for insert)
            'tabfieldinsert'=>array("code,label","code,label","code,label"),
            // Name of columns with primary key (try to always name it 'rowid')
            'tabrowid'=>array("rowid","rowid","rowid"),
            // Condition to show each dictionary
            'tabcond'=>array($conf->tpdeduplication->enabled,$conf->tpdeduplication->enabled,$conf->tpdeduplication->enabled)
        );
        */

        // Boxes/Widgets
        // Add here list of php file(s) stored in tpdeduplication/core/boxes that contains a class to show a widget.
        $this->boxes = array(
            //  0 => array(
            //      'file' => 'tpdeduplicationwidget1.php@tpdeduplication',
            //      'note' => 'Widget provided by Tpdeduplication',
            //      'enabledbydefaulton' => 'Home',
            //  ),
            //  ...
        );

        // Cronjobs (List of cron jobs entries to add when module is enabled)
        // unit_frequency must be 60 for minute, 3600 for hour, 86400 for day, 604800 for week
        $this->cronjobs = array(
            //  0 => array(
            //      'label' => 'MyJob label',
            //      'jobtype' => 'method',
            //      'class' => '/tpdeduplication/class/notduplicatethirdparty.class.php',
            //      'objectname' => 'NotDuplicateThirdparty',
            //      'method' => 'doScheduledJob',
            //      'parameters' => '',
            //      'comment' => 'Comment',
            //      'frequency' => 2,
            //      'unitfrequency' => 3600,
            //      'status' => 0,
            //      'test' => '$conf->tpdeduplication->enabled',
            //      'priority' => 50,
            //  ),
        );
        // Example: $this->cronjobs=array(
        //    0=>array('label'=>'My label', 'jobtype'=>'method', 'class'=>'/dir/class/file.class.php', 'objectname'=>'MyClass', 'method'=>'myMethod', 'parameters'=>'param1, param2', 'comment'=>'Comment', 'frequency'=>2, 'unitfrequency'=>3600, 'status'=>0, 'test'=>'$conf->tpdeduplication->enabled', 'priority'=>50),
        //    1=>array('label'=>'My label', 'jobtype'=>'command', 'command'=>'', 'parameters'=>'param1, param2', 'comment'=>'Comment', 'frequency'=>1, 'unitfrequency'=>3600*24, 'status'=>0, 'test'=>'$conf->tpdeduplication->enabled', 'priority'=>50)
        // );

        // Permissions provided by this module
        $this->rights = array();
        $r = 0;
        // Add here entries to declare new permissions
        /* BEGIN MODULEBUILDER PERMISSIONS */
        $this->rights[$r][0] = $this->numero . $r;    // Permission id (must not be already used)
        $this->rights[$r][1] = $langs->trans('TPDEDUPLICATION_RIGHT_1');    // Permission label
        $this->rights[$r][3] = 0;      // Permission by default for new user (0/1)
        $this->rights[$r][4] = 'read';                // In php code, permission will be checked by test if ($user->rights->tpdeduplication->level1->level2)
        $this->rights[$r][5] = '';
        $r++;
        $this->rights[$r][0] = $this->numero . $r;    // Permission id (must not be already used)
        $this->rights[$r][1] = $langs->trans('TPDEDUPLICATION_RIGHT_2');    // Permission label
        $this->rights[$r][3] = 0;      // Permission by default for new user (0/1)
        $this->rights[$r][4] = 'delete';                // In php code, permission will be checked by test if ($user->rights->tpdeduplication->level1->level2)
        $this->rights[$r][5] = '';                    // In php code, permission will be checked by test if ($user->rights->tpdeduplication->level1->level2)
        $r++;
        // In php code, permission will be checked by test if ($user->rights->tpdeduplication->level1->level2)

        /*
        $r++;
        $this->rights[$r][0] = $this->numero + $r;	// Permission id (must not be already used)
        $this->rights[$r][1] = 'Create/Update objects of Tpdeduplication';	// Permission label
        $this->rights[$r][4] = 'write';				// In php code, permission will be checked by test if ($user->rights->tpdeduplication->level1->level2)
        $this->rights[$r][5] = '';				    // In php code, permission will be checked by test if ($user->rights->tpdeduplication->level1->level2)
        $r++;
        $this->rights[$r][0] = $this->numero + $r;	// Permission id (must not be already used)
        $this->rights[$r][1] = 'Delete objects of Tpdeduplication';	// Permission label
        $this->rights[$r][4] = 'delete';				// In php code, permission will be checked by test if ($user->rights->tpdeduplication->level1->level2)
        $this->rights[$r][5] = '';				    // In php code, permission will be checked by test if ($user->rights->tpdeduplication->level1->level2)
        $r++;
        */
        /* END MODULEBUILDER PERMISSIONS */

        // Main menu entries to add
        $this->menu = array();
        $r = 0;
        // Add here entries to declare new menus
        /* BEGIN MODULEBUILDER TOPMENU */
        $this->menu[$r++] = array(
            'fk_menu' => 'fk_mainmenu=tools',                          // '' if this is a top menu. For left menu, use 'fk_mainmenu=xxx' or 'fk_mainmenu=xxx,fk_leftmenu=yyy' where xxx is mainmenucode and yyy is a leftmenucode
            'type' => 'left',                          // This is a Top menu entry
            'titre' => $langs->trans('TPDEDUPLICATION_TITLE_LEFT_MENU'),
            'mainmenu' => 'tools',
            'leftmenu' => 'tpdeduplication',
            'url' => '/tpdeduplication/tpdeduplicationindex.php',
            'langs' => 'tpdeduplication@tpdeduplication',            // Lang file to use (without .lang) by module. File must be in langs/code_CODE/ directory.
            'position' => 1000 + $r,
            'enabled' => '$conf->tpdeduplication->enabled',  // Define condition to show or hide menu entry. Use '$conf->tpdeduplication->enabled' if entry must be visible if module is enabled.
            'perms' => '$user->rights->tpdeduplication->read',                            // Use 'perms'=>'$user->rights->tpdeduplication->level1->level2' if you want your menu with a permission rules
            'target' => '',
            'user' => 0,                                // 0=Menu for internal users, 1=external users, 2=both
        );

        $this->menu[$r++] = array(
            'fk_menu' => 'fk_mainmenu=tools,fk_leftmenu=tpdeduplication',        // '' if this is a top menu. For left menu, use 'fk_mainmenu=xxx' or 'fk_mainmenu=xxx,fk_leftmenu=yyy' where xxx is mainmenucode and yyy is a leftmenucode
            'type' => 'left',                            // This is a Left menu entry
            'titre' => $langs->trans('TPDEDUPLICATION_TITLE_LIST'),
            'mainmenu' => 'tools',
            'leftmenu' => 'tpdeduplication',
            'url' => '/tpdeduplication/notduplicatethirdparty_list.php',
            'langs' => 'tpdeduplication@tpdeduplication',            // Lang file to use (without .lang) by module. File must be in langs/code_CODE/ directory.
            'position' => 1100 + $r,
            'enabled' => '$conf->tpdeduplication->enabled',  // Define condition to show or hide menu entry. Use '$conf->tpdeduplication->enabled' if entry must be visible if module is enabled. Use '$leftmenu==\'system\'' to show if leftmenu system is selected.
            'perms' => '$user->rights->tpdeduplication->read',                                // Use 'perms'=>'$user->rights->tpdeduplication->level1->level2' if you want your menu with a permission rules
            'target' => '',
            'user' => 0
        );                                // 0=Menu for internal users, 1=external users, 2=both

        /*$this->menu[$r++]=array(
            'fk_menu'=>'fk_mainmenu=tpdeduplication,fk_leftmenu=tpdeduplication_notduplicatethirdparty',	    // '' if this is a top menu. For left menu, use 'fk_mainmenu=xxx' or 'fk_mainmenu=xxx,fk_leftmenu=yyy' where xxx is mainmenucode and yyy is a leftmenucode
            'type'=>'left',			                // This is a Left menu entry
            'titre'=>'New NotDuplicateThirdparty',
            'mainmenu'=>'tpdeduplication',
            'leftmenu'=>'tpdeduplication_notduplicatethirdparty',
            'url'=>'/tpdeduplication/notduplicatethirdparty_card.php?action=create',
            'langs'=>'tpdeduplication@tpdeduplication',	        // Lang file to use (without .lang) by module. File must be in langs/code_CODE/ directory.
            'position'=>1100+$r,
            'enabled'=>'$conf->tpdeduplication->enabled',  // Define condition to show or hide menu entry. Use '$conf->tpdeduplication->enabled' if entry must be visible if module is enabled. Use '$leftmenu==\'system\'' to show if leftmenu system is selected.
            'perms'=>'1',			                // Use 'perms'=>'$user->rights->tpdeduplication->level1->level2' if you want your menu with a permission rules
            'target'=>'',
            'user'=>0
        );*/                                // 0=Menu for internal users, 1=external users, 2=both

        /* END MODULEBUILDER LEFTMENU NOTDUPLICATETHIRDPARTY */

        // Exports profiles provided by this module
        /* BEGIN MODULEBUILDER EXPORT NOTDUPLICATETHIRDPARTY */
        /*
        $langs->load("tpdeduplication@tpdeduplication");
        $this->export_code[$r]=$this->rights_class.'_'.$r;
        $this->export_label[$r]='NotDuplicateThirdpartyLines';	// Translation key (used only if key ExportDataset_xxx_z not found)
        $this->export_icon[$r]='notduplicatethirdparty@tpdeduplication';
        $keyforclass = 'NotDuplicateThirdparty'; $keyforclassfile='/mymobule/class/notduplicatethirdparty.class.php'; $keyforelement='notduplicatethirdparty';
        include DOL_DOCUMENT_ROOT.'/core/commonfieldsinexport.inc.php';
        $keyforselect='notduplicatethirdparty'; $keyforaliasextra='extra'; $keyforelement='notduplicatethirdparty';
        include DOL_DOCUMENT_ROOT.'/core/extrafieldsinexport.inc.php';
        //$this->export_dependencies_array[$r]=array('mysubobject'=>'ts.rowid', 't.myfield'=>array('t.myfield2','t.myfield3')); // To force to activate one or several fields if we select some fields that need same (like to select a unique key if we ask a field of a child to avoid the DISTINCT to discard them, or for computed field than need several other fields)
        $this->export_sql_start[$r]='SELECT DISTINCT ';
        $this->export_sql_end[$r]  =' FROM '.MAIN_DB_PREFIX.'notduplicatethirdparty as t';
        $this->export_sql_end[$r] .=' WHERE 1 = 1';
        $this->export_sql_end[$r] .=' AND t.entity IN ('.getEntity('notduplicatethirdparty').')';
        $r++; */
        /* END MODULEBUILDER EXPORT NOTDUPLICATETHIRDPARTY */

        // Imports profiles provided by this module
        /* BEGIN MODULEBUILDER IMPORT NOTDUPLICATETHIRDPARTY */
        /*
         $langs->load("tpdeduplication@tpdeduplication");
         $this->export_code[$r]=$this->rights_class.'_'.$r;
         $this->export_label[$r]='NotDuplicateThirdpartyLines';	// Translation key (used only if key ExportDataset_xxx_z not found)
         $this->export_icon[$r]='notduplicatethirdparty@tpdeduplication';
         $keyforclass = 'NotDuplicateThirdparty'; $keyforclassfile='/mymobule/class/notduplicatethirdparty.class.php'; $keyforelement='notduplicatethirdparty';
         include DOL_DOCUMENT_ROOT.'/core/commonfieldsinexport.inc.php';
         $keyforselect='notduplicatethirdparty'; $keyforaliasextra='extra'; $keyforelement='notduplicatethirdparty';
         include DOL_DOCUMENT_ROOT.'/core/extrafieldsinexport.inc.php';
         //$this->export_dependencies_array[$r]=array('mysubobject'=>'ts.rowid', 't.myfield'=>array('t.myfield2','t.myfield3')); // To force to activate one or several fields if we select some fields that need same (like to select a unique key if we ask a field of a child to avoid the DISTINCT to discard them, or for computed field than need several other fields)
         $this->export_sql_start[$r]='SELECT DISTINCT ';
         $this->export_sql_end[$r]  =' FROM '.MAIN_DB_PREFIX.'notduplicatethirdparty as t';
         $this->export_sql_end[$r] .=' WHERE 1 = 1';
         $this->export_sql_end[$r] .=' AND t.entity IN ('.getEntity('notduplicatethirdparty').')';
         $r++; */
        /* END MODULEBUILDER IMPORT NOTDUPLICATETHIRDPARTY */
    }

    /**
     *  Function called when module is enabled.
     *  The init function add constants, boxes, permissions and menus (defined in constructor) into Dolibarr database.
     *  It also creates data directories
     *
     * @param string $options Options when enabling module ('', 'noboxes')
     * @return     int                1 if OK, 0 if KO
     */
    public function init($options = '')
    {
        global $conf, $db;
        dolibarr_set_const($db, "CHECKLASTVERSION_EXTERNALMODULE", '1', 'int', 0, '', $conf->entity);
        $result = $this->_load_tables('/tpdeduplication/sql/');
        if ($result < 0) {
            return -1;
        } // Do not activate module if error 'not allowed' returned when loading module SQL queries (the _load_table run sql with run_sql with the error allowed parameter set to 'default')

        $sql = array();
        return $this->_init($sql, $options);
    }

    /**
     *  Function called when module is disabled.
     *  Remove from database constants, boxes and permissions from Dolibarr database.
     *  Data directories are not deleted
     *
     * @param string $options Options when enabling module ('', 'noboxes')
     * @return     int                 1 if OK, 0 if KO
     */
    public function remove($options = '')
    {
        $sql = array();
        return $this->_remove($sql, $options);
    }
}
