<?php
/* Copyright (C) 2001-2005 Rodolphe Quiedeville <rodolphe@quiedeville.org>
 * Copyright (C) 2004-2015 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2005-2012 Regis Houssin        <regis.houssin@inodbox.com>
 * Copyright (C) 2015      Jean-François Ferry	<jfefe@aternatik.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *    \file       tpdeduplication/tpdeduplicationindex.php
 *    \ingroup    tpdeduplication
 *    \brief      Home page of tpdeduplication top menu
 */

// Load Dolibarr environment
$res = 0;
// Try main.inc.php into web root known defined into CONTEXT_DOCUMENT_ROOT (not always defined)
if (!$res && !empty($_SERVER["CONTEXT_DOCUMENT_ROOT"])) $res = @include $_SERVER["CONTEXT_DOCUMENT_ROOT"] . "/main.inc.php";
// Try main.inc.php into web root detected using web root calculated from SCRIPT_FILENAME
$tmp = empty($_SERVER['SCRIPT_FILENAME']) ? '' : $_SERVER['SCRIPT_FILENAME'];
$tmp2 = realpath(__FILE__);
$i = strlen($tmp) - 1;
$j = strlen($tmp2) - 1;
while ($i > 0 && $j > 0 && isset($tmp[$i]) && isset($tmp2[$j]) && $tmp[$i] == $tmp2[$j]) {
    $i--;
    $j--;
}
if (!$res && $i > 0 && file_exists(substr($tmp, 0, ($i + 1)) . "/main.inc.php")) $res = @include substr($tmp, 0, ($i + 1)) . "/main.inc.php";
if (!$res && $i > 0 && file_exists(dirname(substr($tmp, 0, ($i + 1))) . "/main.inc.php")) $res = @include dirname(substr($tmp, 0, ($i + 1))) . "/main.inc.php";
// Try main.inc.php using relative path
if (!$res && file_exists("../main.inc.php")) $res = @include "../main.inc.php";
if (!$res && file_exists("../../main.inc.php")) $res = @include "../../main.inc.php";
if (!$res && file_exists("../../../main.inc.php")) $res = @include "../../../main.inc.php";
if (!$res) die("Include of main fails");

require_once DOL_DOCUMENT_ROOT . '/core/class/html.formfile.class.php';
require_once DOL_DOCUMENT_ROOT . '/societe/class/societe.class.php';
require_once DOL_DOCUMENT_ROOT . '/categories/class/categorie.class.php';

require_once __DIR__ . '/class/notduplicatethirdparty.class.php';
require_once __DIR__ . '/class/TextCleaner.class.php';

// Load translation files required by the page
$langs->loadLangs(array("tpdeduplication@tpdeduplication"));
$hookmanager->initHooks(array('tpduplicatethirdparty_page'));

$action = GETPOST('action', 'alpha');

// Securite acces client
if (!$user->rights->tpdeduplication->read) accessforbidden();
$socid = GETPOST('socid', 'int');
if (isset($user->societe_id) && $user->societe_id > 0) {
    accessforbidden();
}
$result = restrictedArea($user, 'societe');
$max = 20;
$now = dol_now();

/*
 * GET all tables not updates when thirdparty are merged
 *
$sql = "SELECT DISTINCT TABLE_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME = 'fk_soc'";
$results = $db->query($sql);
$tab = array();
if ($results) {
    $obj = $db->fetch_object($results);
    while ($obj != null){
        array_push($tab, $obj->TABLE_NAME);
        $obj = $db->fetch_object($results);
    }
}

$sql = "SELECT DISTINCT TABLE_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME like 'soc%id'";
$results = $db->query($sql);
$tab2 = array();
if ($results) {
    $obj = $db->fetch_object($results);
    while ($obj != null){
        array_push($tab2, $obj->TABLE_NAME);
        $obj = $db->fetch_object($results);
    }
}

$tabChange = array(
    'llx_actioncomm',
   'llx_propal',
   'llx_user',
   'llx_facture_fourn',
   'llx_product_fournisseur_price',
   'llx_commande_fournisseur',
   'llx_reception',
   'llx_categorie_societe',
   'llx_contrat',
   'llx_facture',
   'llx_facture_rec',
   'llx_supplier_proposal',
   'llx_societe_address',
   'llx_societe_commerciaux',
   'llx_societe_log',
   'llx_societe_prices',
   'llx_societe_remise',
   'llx_societe_remise_except',
   'llx_societe_rib',
   'llx_projet',
   'llx_socpeople',
   'llx_adherent',
   'llx_livraison',
   'llx_fichinter',
   'llx_expedition',
   'llx_product_customer_price',
   'llx_product_customer_price_log',
   'llx_prelevement_lignes',
   'llx_commande',
   'llx_bookmark' // No fk_soc
    //30
);

print '<pre>' . print_r($tab, 1) . '</pre>';

print '<pre>' . print_r(array_diff($tab, $tab2, $tabChange), 1) . '</pre>';
exit();
*/

/*
 * Actions
 */
$action = GETPOST('action', 'aZ09') ? GETPOST('action', 'aZ09') : 'view';
if ($action == 'keep') {
    $soc_origin_id = GETPOST('socidtodelete');
    $socToKeep = GETPOST('socidtokeep');

    $object = new Societe($db);
    $object->fetch($socToKeep);

    $error = 0;
    $soc_origin = new Societe($db);

    if ($soc_origin_id <= 0) {
        $langs->load('errors');
        $langs->load('companies');
        setEventMessages($langs->trans('ErrorThirdPartyIdIsMandatory', $langs->transnoentitiesnoconv('MergeOriginThirdparty')), null, 'errors');
    } else {
        if (!$error && $soc_origin->fetch($soc_origin_id) < 1) {
            setEventMessages($langs->trans('ErrorRecordNotFound'), null, 'errors');
            $error++;
        }

        if (!$error) {
            $db->begin();

            // Recopy some data
            $object->client = $object->client | $soc_origin->client;
            $object->fournisseur = $object->fournisseur | $soc_origin->fournisseur;
            $listofproperties = array(
                'address', 'zip', 'town', 'state_id', 'country_id', 'phone', 'phone_pro', 'fax', 'email', 'skype', 'twitter', 'facebook', 'linkedin', 'url', 'barcode',
                'idprof1', 'idprof2', 'idprof3', 'idprof4', 'idprof5', 'idprof6',
                'tva_intra', 'effectif_id', 'forme_juridique', 'remise_percent', 'remise_supplier_percent', 'mode_reglement_supplier_id', 'cond_reglement_supplier_id', 'name_bis',
                'stcomm_id', 'outstanding_limit', 'price_level', 'parent', 'default_lang', 'ref', 'ref_ext', 'import_key', 'fk_incoterms', 'fk_multicurrency',
                'code_client', 'code_fournisseur', 'code_compta', 'code_compta_fournisseur',
                'model_pdf', 'fk_projet'
            );
            foreach ($listofproperties as $property) {
                if (empty($object->$property)) $object->$property = $soc_origin->$property;
            }

            // Concat some data
            $listofproperties = array(
                'note_public', 'note_private'
            );
            foreach ($listofproperties as $property) {
                $object->$property = dol_concatdesc($object->$property, $soc_origin->$property);
            }

            // Merge extrafields
            if (is_array($soc_origin->array_options)) {
                foreach ($soc_origin->array_options as $key => $val) {
                    if (empty($object->array_options[$key])) $object->array_options[$key] = $val;
                }
            }

            // Merge categories
            $static_cat = new Categorie($db);

            $custcats_ori = $static_cat->containing($soc_origin->id, 'customer', 'id');
            $custcats = $static_cat->containing($object->id, 'customer', 'id');
            $custcats = array_merge($custcats, $custcats_ori);
            $object->setCategories($custcats, 'customer');

            $suppcats_ori = $static_cat->containing($soc_origin->id, 'supplier', 'id');
            $suppcats = $static_cat->containing($object->id, 'supplier', 'id');
            $suppcats = array_merge($suppcats, $suppcats_ori);
            $object->setCategories($suppcats, 'supplier');

            // If thirdparty has a new code that is same than origin, we clean origin code to avoid duplicate key from database unique keys.
            if ($soc_origin->code_client == $object->code_client
                || $soc_origin->code_fournisseur == $object->code_fournisseur
                || $soc_origin->barcode == $object->barcode) {
                dol_syslog("We clean customer and supplier code so we will be able to make the update of target");
                $soc_origin->code_client = '';
                $soc_origin->code_fournisseur = '';
                $soc_origin->barcode = '';
                $soc_origin->update($soc_origin->id, $user, 0, 1, 1, 'merge');
            }

            // Update
            $object->update($object->id, $user, 0, 1, 1, 'merge');
            if ($result < 0) {
                $error++;
            }

            // Move links
            if (!$error) {
                $objects = array(
                    'Adherent' => '/adherents/class/adherent.class.php',
                    'Societe' => '/societe/class/societe.class.php',
                    //'Categorie' => '/categories/class/categorie.class.php',
                    'ActionComm' => '/comm/action/class/actioncomm.class.php',
                    'Propal' => '/comm/propal/class/propal.class.php',
                    'Commande' => '/commande/class/commande.class.php',
                    'Facture' => '/compta/facture/class/facture.class.php',
                    'FactureRec' => '/compta/facture/class/facture-rec.class.php',
                    'LignePrelevement' => '/compta/prelevement/class/ligneprelevement.class.php',
                    'Contact' => '/contact/class/contact.class.php',
                    'Contrat' => '/contrat/class/contrat.class.php',
                    'Expedition' => '/expedition/class/expedition.class.php',
                    'Fichinter' => '/fichinter/class/fichinter.class.php',
                    'CommandeFournisseur' => '/fourn/class/fournisseur.commande.class.php',
                    'FactureFournisseur' => '/fourn/class/fournisseur.facture.class.php',
                    'SupplierProposal' => '/supplier_proposal/class/supplier_proposal.class.php',
                    'ProductFournisseur' => '/fourn/class/fournisseur.product.class.php',
                    'Livraison' => '/livraison/class/livraison.class.php',
                    'Product' => '/product/class/product.class.php',
                    'Project' => '/projet/class/project.class.php',
                    'User' => '/user/class/user.class.php',
                );
                if ((int) DOL_VERSION >= 10) {
                    $objects['Reception'] = '/reception/class/reception.class.php';
                }

                //First, all core objects must update their tables
                foreach ($objects as $object_name => $object_file) {
                    require_once DOL_DOCUMENT_ROOT . $object_file;

                    if (!$error && !$object_name::replaceThirdparty($db, $soc_origin->id, $object->id)) {
                        $error++;
                        setEventMessages($db->lasterror(), null, 'errors');
                    }
                }
            }

            // External modules should update their ones too
            if (!$error) {
                $reshook = $hookmanager->executeHooks('replaceThirdparty', array(
                    'soc_origin' => $soc_origin->id,
                    'soc_dest' => $object->id
                ), $soc_dest, $action);

                if ($reshook < 0) {
                    setEventMessages($hookmanager->error, $hookmanager->errors, 'errors');
                    $error++;
                }
            }


            if (!$error) {
                $object->context = array('merge' => 1, 'mergefromid' => $soc_origin->id);

                // Call trigger
                $result = $object->call_trigger('COMPANY_MODIFY', $user);
                if ($result < 0) {
                    setEventMessages($object->error, $object->errors, 'errors');
                    $error++;
                }

                // End call triggers
            }

            if (!$error) {
                //We finally remove the old thirdparty
                if ($soc_origin->delete($soc_origin->id, $user) < 1) {
                    $error++;
                }
            }

            if (!$error) {
                setEventMessages($langs->trans('ThirdpartiesMergeSuccess'), null, 'mesgs');
                $db->commit();
            } else {
                $langs->load("errors");
                setEventMessages($langs->trans('ErrorsThirdpartyMerge'), null, 'errors');
                $db->rollback();
            }
        }
        header('Location:' . $_SERVER['REQUEST_URI']);
    }
}
if ($action == 'ignore') {
    $socToKeep1 = GETPOST('socidtokeep1');
    $socToKeep2 = GETPOST('socidtokeep2');
    $sql = "INSERT INTO `" . MAIN_DB_PREFIX . "tpdeduplication_notduplicatethirdparty` (`fk_soc1`, `fk_soc2`, `fk_user`, `status`) VALUES";
    $sql .= "(" . $socToKeep1 . "," . $socToKeep2 . "," . $user->id . ",NULL);";
    $result = $db->query($sql);
    if (!$result) {
        return 0;
    }
    header('Location:' . $_SERVER['REQUEST_URI']);
}

/*
 * View
 */

$form = new Form($db);
$formfile = new FormFile($db);
$soc = new Societe($db);
$obj = new NotDuplicateThirdparty($db);
$textCleaner = new TextCleaner();

llxHeader("", $langs->trans("TpdeduplicationArea"));

print load_fiche_titre($langs->trans("TpdeduplicationArea"), '', 'tpdeduplication.png@tpdeduplication');

// Prepare my data
if (!empty($conf->tpdeduplication->enabled) && $user->rights->tpdeduplication->read) {
    if (((preg_match('/MariaDB/', $db->getVersion()) && (float) $db->getVersion() >= 5.5) || (preg_match('/MySQL/', $db->getVersion()) && (float) $db->getVersion() >= 8.0)) /*&& false*/) {
        $sql = "SELECT rowid, " . $db->decrypt('name') . " as name";
        $sql .= " FROM " . MAIN_DB_PREFIX . "const";
        $sql .= " WHERE entity IN (" . $user->entity . "," . $conf->entity . ")";
        $sql .= " AND name LIKE 'TPDEDUPLICATION_%'";

        $result = $db->query($sql);
        if ($result) {
            $tabConstCheck = array();
            $obj = $db->fetch_object($result);
            while ($obj != null) {
                $constName = $obj->name;
                if ((string) $conf->global->$constName != '0') {
                    array_push($tabConstCheck, $conf->global->$constName);
                }
                $obj = $db->fetch_object($result);
            }
        }

        if (count($tabConstCheck) == 0) {
            print $langs->trans('TPDEDUPLICATION_NO_FILTER_FOR_SEARCH');
            exit;
        }

        $sql = 'SELECT DISTINCT rowid, RANK() OVER (ORDER BY ';
        $i = 1;
        foreach ($tabConstCheck as $key => $val) {
            switch ($val) {
                //Only keep numbers
                case 'siren':
                case 'siret':
                case 'phone':
                case 'zip':
                    $sql .= $textCleaner->prepareAndCleanSQL($val, null, 0, array(array('([^0-9])+', '')), array('LOWER'));
                    break;
                //Only keep letters
                case 'town':
                    $sql .= $textCleaner->prepareAndCleanSQL($val, null, 1, array(array('([^A-Za-z])+', '')), array('LOWER'));
                    break;
                //Keep letters and numbers
                case 'nom':
                case 'address':
                case 'code_fournisseur':
                case 'ape':
                case 'idprof4':
                case 'tva_intra':
                    $sql .= $textCleaner->prepareAndCleanSQL($val, null, 1, array(array('([^A-Za-z0-9])+', '')), array('LOWER'));
                    break;
                default:
                    $sql .= $val;
                    break;
            }
            if ($i != count($tabConstCheck)) {
                $sql .= ', ';
            }
            $i++;
        }
        $sql .= ") as my_rank FROM " . MAIN_DB_PREFIX . "societe as t1 ";
        $sql .= "WHERE EXISTS (";
        $sql .= "SELECT rowid FROM " . MAIN_DB_PREFIX . "societe as t2 ";
        $sql .= "WHERE t1.rowid <> t2.rowid ";

        foreach ($tabConstCheck as $key => $val) {
            switch ($val) {
                //Only keep numbers
                case 'siren':
                case 'siret':
                case 'phone':
                case 'zip':
                    $sql .= " AND ";
                    $sql .= $textCleaner->prepareAndCleanSQL('t1.' . $val, null, 0, array(array('([^0-9])+', '')), array('LOWER'));
                    $sql .= "=";
                    $sql .= $textCleaner->prepareAndCleanSQL('t2.' . $val, null, 0, array(array('([^0-9])+', '')), array('LOWER'), 1);
                    break;
                //Only keep letters
                case 'town':
                    $sql .= " AND ";
                    $sql .= $textCleaner->prepareAndCleanSQL('t1.' . $val, null, 1, array(array('([^A-Za-z])+', '')), array('LOWER'));
                    $sql .= "=";
                    $sql .= $textCleaner->prepareAndCleanSQL('t2.' . $val, null, 1, array(array('([^A-Za-z])+', '')), array('LOWER'), 1);
                    break;
                //Keep numbers and letters
                case 'nom':
                case 'address':
                case 'code_fournisseur':
                case 'ape':
                case 'idprof4':
                case 'tva_intra':
                    $sql .= " AND ";
                    $sql .= $textCleaner->prepareAndCleanSQL('t1.' . $val, null, 1, array(array('([^A-Za-z0-9])+', '')), array('LOWER'));
                    $sql .= "=";
                    $sql .= $textCleaner->prepareAndCleanSQL('t2.' . $val, null, 1, array(array('([^A-Za-z0-9])+', '')), array('LOWER'), 1);
                    break;
                case 'fk_departement':
                case 'fk_pays':
                    $sql .= " AND t1." . $val . " = t2." . $val . " AND t1." . $val . " != 0 AND t2." . $val . " != 0";
                    break;
                default:
                    $sql .= " AND t1." . $val . " = t2." . $val;
                    break;
            }
        }
        $sql .= ')';
    } else {
        $sql = "SELECT rowid, " . $db->decrypt('name') . " as name";
        $sql .= " FROM " . MAIN_DB_PREFIX . "const";
        $sql .= " WHERE entity IN (" . $user->entity . "," . $conf->entity . ")";
        $sql .= " AND name LIKE 'TPDEDUPLICATION_%'";

        $result = $db->query($sql);
        if ($result) {
            $tabConstCheck = array();
            $obj = $db->fetch_object($result);
            while ($obj != null) {
                $constName = $obj->name;
                if ((string) $conf->global->$constName != '0') {
                    array_push($tabConstCheck, $conf->global->$constName);
                }
                $obj = $db->fetch_object($result);
            }
        }

        if (count($tabConstCheck) == 0) {
            print $langs->trans('TPDEDUPLICATION_NO_FILTER_FOR_SEARCH');
            exit;
        }

        $sql = 'SELECT DISTINCT rowid , ' . $conf->global->DB_TPDEDUPLICATION_OPTION_DEDUPLICATE . ' as my_rank ';
        $sql .= "FROM " . MAIN_DB_PREFIX . "societe as t1 ";
        $sql .= "WHERE EXISTS (";
        $sql .= "SELECT rowid FROM " . MAIN_DB_PREFIX . "societe as t2 ";
        $sql .= "WHERE t1.rowid <> t2.rowid ";
        $sql .= " AND ";
        $sql .= $textCleaner->prepareAndCleanSQL('t1.' . $conf->global->DB_TPDEDUPLICATION_OPTION_DEDUPLICATE);
        $sql .= "=";
        $sql .= $textCleaner->prepareAndCleanSQL('t2.' . $conf->global->DB_TPDEDUPLICATION_OPTION_DEDUPLICATE, null, null, null, null, 1);
        $sql .= ') ORDER BY ' . $conf->global->DB_TPDEDUPLICATION_OPTION_DEDUPLICATE;
    }
}

$result = $db->query($sql);
$tabDuplicateComp = array();
$tmpDuplicate = array();
$tmpRank = null;
if ($result) {
    $obj = $db->fetch_object($result);
    while ($obj != null) {
        if ($tmpRank == null) {
            $tmpRank = $obj->my_rank;
        }
        if ($obj->my_rank == $tmpRank) {
            array_push($tmpDuplicate, $obj->rowid);
        } else {
            array_push($tabDuplicateComp, $tmpDuplicate);
            $tmpDuplicate = array($obj->rowid);
        }

        $tmpRank = $obj->my_rank;
        $obj = $db->fetch_object($result);
    }
    if ($tmpRank != null) {
        array_push($tabDuplicateComp, $tmpDuplicate);
    }

    $db->free($resql);
} else {
    dol_print_error($db);
}
if (count($tabDuplicateComp) == 0) {
    print $langs->trans('TPDEDUPLICATION_NO_RESULT');
    exit;
}

//Create all pairs combination of duplicate thirdparty and remove pairs non duplicated select by the user

$sql = "SELECT fk_soc1,fk_soc2 FROM " . MAIN_DB_PREFIX . "tpdeduplication_notduplicatethirdparty";
$result = $db->query($sql);

$tabsOfNoDuplicate = array();
if ($result) {
    $obj = $db->fetch_object($result);
    while ($obj != null) {
        $tmp = array($obj->fk_soc1, $obj->fk_soc2);
        sort($tmp);
        array_push($tabsOfNoDuplicate, $tmp);
        $obj = $db->fetch_object($result);
    }
}

$tabOfPairs = array();
foreach ($tabDuplicateComp as $key => $val) {
    $i = 0;
    $j = $i + 1;
    while ($val[$i] !== null && $val[$j] !== null) {
        while ($val[$j] != null) {
            array_push($tabOfPairs, array($val[$i], $val[$j]));
            $j++;
        }
        $i++;
        $j = $i + 1;
    }
}

//Remove pairs which aren't duplicate
if (count($tabsOfNoDuplicate) >= 1) {
    foreach ($tabOfPairs as $key => $value) {
        foreach ($tabsOfNoDuplicate as $k => $val) {
            if (($value[0] == $val[0] && $value[1] == $val[1]) || ($value[0] == $val[1] && $value[1] == $val[0])) {
                unset($tabOfPairs[$key]);
            }
        }
    }
}

//Tabs of elements to print to client

$elementsToPrint = array(
    'name',
    'status',
    'code_client',
    'code_fournisseur',
    'address',
    'zip',
    'town',
    'state',
    'country',
    'idprof1',
    'idprof2',
    'idprof4',
    'tva_intra',
    'contact_array_objects()',
    'date_modification',
    'date_dev',
    'date_fact'
);

$soc1 = new Societe($db);
$soc2 = new Societe($db);

//Print view

print '<div class="fichecenter">';
foreach ($tabOfPairs as $key => $pair) {
    $soc1->fetch($pair[0]);
    $soc2->fetch($pair[1]);
    print '<div class="fiche-duplicate" id="' . $soc1->id . '_' . $soc2->id . '" style="margin:1rem; margin-bottom:6rem; border:0.2rem solid #14d1ff; border-radius: 1rem">';
    print '<div class="div-table-responsive" style="border-radius: 1rem;">';
    print '<table class="tagtable liste">';
    print '<thead">';
    print '<th scope="col" style="font-size: 1.2em; padding: 1.3rem; width: 15%; text-align: left">' . $langs->trans('TPDEDUPLICATION_INFORMATIONS') . '</th>';
    print '<th scope="col" style="font-size: 1.2em;  padding: 1.3rem; width: 25%; text-align: left">' . $soc1->getNomUrl(1) . '</th>';
    print '<th scope="col" style="font-size: 1.2em;  padding: 1.3rem; width: 25%; text-align: left">' . $soc2->getNomUrl(1) . '</th>';
    print '</thead>';

    foreach ($elementsToPrint as $k => $v) {
        print '<tr>';
        switch ($v) {
            case 'address':
            case 'zip':
            case 'town':
            case 'state':
            case 'country':
                print '<td>';
                print '<strong>' . $langs->trans(ucfirst($v)) . '</strong>';
                print '</td>';
                print '<td>';
                print $soc1->$v;
                print '</td>';
                print '<td>';
                print $soc2->$v;
                print '</td>';
                break;
            case (preg_match('/idprof/', $v) ? true : false):
                print '<td>';
                print '<strong>' . $langs->trans('ProfId' . substr($v, -1) . $soc1->country_code) . '</strong>';
                print '</td>';
                print '<td>';
                print $soc1->$v;
                print '</td>';
                print '<td>';
                print $soc2->$v;
                print '</td>';
                break;
            case 'code_client':
                print '<td scope="row">';
                print '<strong>' . $langs->trans('CustomerCode') . '</strong>';
                print '</td>';
                print '<td>';
                print $soc1->$v;
                print '</td>';
                print '<td>';
                print $soc2->$v;
                print '</td>';
                break;
            case 'code_fournisseur':
                print '<td scope="row">';
                print '<strong>' . $langs->trans('SupplierCode') . '</strong>';
                print '</td>';
                print '<td>';
                print $soc1->$v;
                print '</td>';
                print '<td>';
                print $soc2->$v;
                print '</td>';
                break;
            case 'name':
                print '<td scope="row">';
                print '<strong>' . $langs->trans('ThirdPartyName') . '</strong>';
                print '</td>';
                print '<td>';
                print $soc1->$v;
                print '</td>';
                print '<td>';
                print $soc2->$v;
                print '</td>';
                break;
            case 'status':
                print '<td scope="row">';
                print '<strong>' . $langs->trans('TPDEDUPLICATION_STATUT_THIRDPARTY') . '</strong>';
                print '</td>';
                print '<td>';
                if ($soc1->$v) {
                    print '<div class="open-thirdparty" style="height: 1rem;width: 1rem;border-radius: 1em;background-color: green; display: inline-block;"></div>';
                    print $soc1->LibStatut($soc1->$v);
                } else {
                    print '<div class="open-thirdparty" style="height: 1rem;width: 1rem;border-radius: 1em;background-color: red; display: inline-block;"></div>';
                    print $soc1->LibStatut($soc1->$v);
                }
                print '</td>';
                print '<td>';
                if ($soc2->$v) {
                    print '<div class="open-thirdparty" style="height: 1rem;width: 1rem;border-radius: 1em;background-color: green; display: inline-block;"></div>';
                    print $soc1->LibStatut($soc1->$v);
                } else {
                    print '<div class="open-thirdparty" style="height: 1rem;width: 1rem;border-radius: 1em;background-color: red; display: inline-block;"></div>';
                    print $soc1->LibStatut($soc1->$v);
                }
                print '</td>';
                break;
            case 'tva_intra':
                print '<td scope="row">';
                print '<strong>' . $langs->trans('VATIntra') . '</strong>';
                print '</td>';
                print '<td>';
                print $soc1->tva_intra;
                print '</td>';
                print '<td>';
                print $soc2->$v;
                print '</td>';
                break;
            case 'contact_array_objects()':
                print '<td scope="row">';
                print '<strong>' . $langs->trans('TPDEDUPLICATION_NUMBER_OF_CONTACT') . '</strong>';
                print '</td>';
                print '<td>';
                print count($soc1->contact_array_objects());
                print '</td>';
                print '<td>';
                print count($soc2->contact_array_objects());
                print '</td>';
                break;
            case 'date_modification':
                print '<td scope="row">';
                print '<strong>' . $langs->trans('TPDEDUPLICATION_DATE_MODIF') . '</strong>';
                print '</td>';
                print '<td>';
                print dol_print_date($soc1->$v);
                print '</td>';
                print '<td>';
                print dol_print_date($soc2->$v);
                print '</td>';
                break;
            case 'date_fact':
                print '<td scope="row">';
                print '<strong>' . $langs->trans('TPDEDUPLICATION_DATE_LAST_INVOICE') . '</strong>';
                print '</td>';
                print '<td>';
                $sql = "SELECT tms FROM " . MAIN_DB_PREFIX . "facture WHERE tms IS NOT NULL AND fk_soc = " . $soc1->id . " ORDER BY tms DESC LIMIT 1";
                $results = $db->query($sql);
                $tab = array();
                if ($results) {
                    $obj = $db->fetch_object($results);
                    print $obj->tms;
                }
                print '</td>';
                print '<td>';
                $sql = "SELECT tms FROM " . MAIN_DB_PREFIX . "facture WHERE tms IS NOT NULL AND fk_soc = " . $soc2->id . " ORDER BY tms DESC LIMIT 1";
                $results = $db->query($sql);
                $tab = array();
                if ($results) {
                    $obj = $db->fetch_object($results);
                    print $obj->tms;
                }
                print '</td>';
                break;
            case 'date_dev':
                print '<td scope="row">';
                print '<strong>' . $langs->trans('TPDEDUPLICATION_DATE_LAST_PROPOSAL') . '</strong>';
                print '</td>';
                print '<td>';
                $sql = "SELECT tms FROM " . MAIN_DB_PREFIX . "propal WHERE tms IS NOT NULL AND fk_soc = " . $soc1->id . " ORDER BY tms DESC LIMIT 1";
                $results = $db->query($sql);
                $tab = array();
                if ($results) {
                    $obj = $db->fetch_object($results);
                    print $obj->tms;
                }
                print '</td>';
                print '<td>';
                $sql = "SELECT tms FROM " . MAIN_DB_PREFIX . "propal WHERE tms IS NOT NULL AND fk_soc = " . $soc2->id . " ORDER BY tms DESC LIMIT 1";
                $results = $db->query($sql);
                $tab = array();
                if ($results) {
                    $obj = $db->fetch_object($results);
                    print $obj->tms;
                }
                print '</td>';
                break;
        }
        print '</tr>';
    }
    print '<tr>';

    //Button Ignore
    print '<td>';
    print '<div class="ignoreThirdParty" style="text-align: left;">';
    print '<form name="ignoreButton" action="' . $_SERVER['REQUEST_URI'] . '" method="post">';
    print '<input type="hidden" name="token" value="' . $_SESSION['newtoken'] . '">';
    print '<input type="hidden" name="action" value="ignore">';
    print '<input type="hidden" name="socidtokeep1" value="' . $soc2->id . '">';
    print '<input type="hidden" name="socidtokeep2" value="' . $soc1->id . '">';
    print '<button class="btn btn-warning" type="submit" style="color: #fff; background-color: #f0ad4e; border-color: #eea236; cursor: pointer; padding: 0.5rem 1rem; border-radius: 4px; border: 1px solid transparent; font-size: 1.1em;">' . $langs->trans('TPDEDUPLICATION_THIRDPARTY_TO_IGNORE') . '</button>';
    print '</form>';
    print '</div>';
    print '</td>';

    //Keep button thirdparty A
    print '<td>';
    print '<div style="text-align:left;">';
    print '<form name="keepButton" action="' . $_SERVER['REQUEST_URI'] . '" method="post">';
    print '<input type="hidden" name="token" value="' . $_SESSION['newtoken'] . '">';
    print '<input type="hidden" name="action" value="keep">';
    print '<input type="hidden" name="socidtokeep" value="' . $soc1->id . '">';
    print '<input type="hidden" name="socidtodelete" value="' . $soc2->id . '">';
    print '<button class="btn btn-primary" type="submit" style="color: #fff; background-color: #337ab7; border-color: #2e6da4; cursor: pointer; padding: 0.5rem 1rem; border-radius: 4px; border: 1px solid transparent; font-size: 1.1em;" >' . $langs->trans('TPDEDUPLICATION_THIRDPARTY_TO_KEEP') . '</button>';
    print '</form>';
    print '</div>';
    print '</td>';

    //Keep button thirdparty B
    print '<td>';
    print '<div style="text-align:left;">';
    print '<form name="keepButton" action="' . $_SERVER['REQUEST_URI'] . '" method="post">';
    print '<input type="hidden" name="token" value="' . $_SESSION['newtoken'] . '">';
    print '<input type="hidden" name="action" value="keep">';
    print '<input type="hidden" name="socidtokeep" value="' . $soc2->id . '">';
    print '<input type="hidden" name="socidtodelete" value="' . $soc1->id . '">';
    print '<button class="btn btn-primary" type="submit" style="color: #fff; background-color: #337ab7; border-color: #2e6da4; cursor: pointer; padding: 0.5rem 1rem; border-radius: 4px; border: 1px solid transparent; font-size: 1.1em;">' . $langs->trans('TPDEDUPLICATION_THIRDPARTY_TO_KEEP') . '</button>';
    print '</form>';
    print '</div>';
    print '</td>';
    print '</tr>';

    print '</table>';
    print '</div>';

    print '</div>';
}
print '</div></div></div>';

// End of page
llxFooter();
$db->close();
