<?php
ini_set('memory_limit', '4096M');
set_time_limit(0);
require 'config.php';
require_once DOL_DOCUMENT_ROOT.'/core/lib/functions.lib.php';
require_once DOL_DOCUMENT_ROOT.'/core/class/html.form.class.php';
dol_include_once('/importatm/class/importatm.class.php');
dol_include_once('/importatm/lib/importatm.lib.php');

$max_input_vars = ini_get('max_input_vars');

if(empty($user->rights->importatm)) accessforbidden();

$langs->load('importatm@importatm');
$langs->load('admin');
$langs->load('companies');
$langs->load('multicurrency');
$langs->load('bills');

$action = GETPOST('action');
$id = GETPOST('id'); // For debug
$type_object = GETPOST('type_object');

$PDOdb = new TPDOdb;
$object = TImportatm::getInstance($type_object);
// For debug
if (!empty($id)) $object->load($PDOdb, $id);

$hookmanager->initHooks(array('importatmcard', 'globalcard'));

/*
 * Actions
 */

		
$parameters = array('id' => $id, 'type_object' => $type_object);
$reshook = $hookmanager->executeHooks('doActions', $parameters, $object, $action); // Note that $action and $object may have been modified by some
if ($reshook < 0) setEventMessages($hookmanager->error, $hookmanager->errors, 'errors');

// Si vide alors le comportement n'est pas remplacé
if (empty($reshook))
{
	$error = 0;
	switch ($action) {
		
		case 'init_import':
			
			$object->readFile($_FILES['file']);
			
			if ($object->nb_lines > 0) _showCard($object, $action, $type_object, $_FILES['file']['name']);
			elseif ($object->nb_lines == 0)
			{
				$action='';
				setEventMessage($langs->trans('warning_file_is_empty'), 'warnings');
				_showCard($object, $action);
			}
			else
			{
				$action='';
				setEventMessages('', $object->errors, 'errors');
				_showCard($object, $action);
			}
			
			break;
		
		case 'simulate_import':
			
			$TUserId = explode(',', GETPOST('TUserId'));
			$object->initData(GETPOST('data_brut'));
			$TLineToImport = GETPOST('TLineToImport', 'array');
			
			if (empty($TLineToImport))
			{
				setEventMessage($langs->trans('Importatm_warning_no_line_selected'), 'warnings');
				_showCard($object, 'init_import', $type_object, GETPOST('filename'));
				break;
			}
			
			$fk_importatm_schema = GETPOST('fk_importatm_schema');
			$TCol = GETPOST('TCol', 'array');
			$TKey_to_update = GETPOST('TKey_to_update', 'array');
			
			// TODO give key_to_fetch
			$object->importData($PDOdb, $TLineToImport, $TCol, GETPOST('filename'), $type_object, $TKey_to_update, $fk_importatm_schema, $TUserId);
			
			if (!empty($object->errors)) setEventMessages('', $object->errors, 'errors');
			
			_showCard($object, $action, $type_object, GETPOST('filename'), $fk_importatm_schema, $TCol, $TKey_to_update);
			
			break;
		
		case 'confirm_import':
			
			$TUserId = explode(',', GETPOST('TUserId'));
			$object->initData(GETPOST('data_brut'));
			
			$TCol = unserialize(base64_decode(GETPOST('TCol')));
			$TKey_to_update = unserialize(base64_decode(GETPOST('TKey_to_update')));
			
			// TODO give key_to_fetch
			$object->importData($PDOdb, array_keys($object->TData), $TCol, GETPOST('filename'), $type_object, $TKey_to_update, GETPOST('fk_importatm_schema'), $TUserId, false);
			
			setEventMessage($langs->trans('Importatm_nb_lines_inserted', $object->nb_create));
			setEventMessage($langs->trans('Importatm_nb_lines_updated', $object->nb_update));
			if ($object->nb_errors > 0) setEventMessage($langs->trans('Importatm_nb_lines_errored', $object->nb_errors), 'errors');
			
			header('Location: '.dol_buildpath('/importatm/card.php', 1));
			exit;
			break;
		
		default:
			
			_showCard($object, $action);
			
			break;
	}
}


/**
 * View
 */
function _showCard($object, $action, $type_object='', $filename='', $fk_importatm_schema=0, $TCol=array(), $TKey_to_update=array())
{
	global $langs,$db;
	
	$form = new Form($db);
	$formcore = new TFormCore;
	$TTypeObjectToImport = TImportatm::getTypeObjectToImport();
	
	if ($action == 'init_import') $next_action = 'simulate_import'; // step 2
	elseif ($action == 'simulate_import') $next_action = 'confirm_import'; // step 3
	else $next_action = 'init_import'; // step 1
	
	$title=$langs->trans("Importatm");
	llxHeader('',$title);

	$head = importatm_prepare_head($object);
	$picto = 'generic';
	dol_fiche_head($head, 'card', $langs->trans("Importatm"), 0, $picto);
	
	print $formcore->begin_form(null, 'importatm_form', 'POST', true);
	
	print $formcore->hidden('action', $next_action);
	print $formcore->hidden('token', $_SESSION['newtoken']);
	if (!empty($type_object)) print $formcore->hidden('type_object', $type_object);
	if (!empty($object->TData)) print '<textarea style="display:none;" name="data_brut">'.base64_encode(serialize($object->TData)).'</textarea>';
	if (!empty($filename)) print $formcore->hidden('filename', $filename);
	if (!empty($fk_importatm_schema)) print $formcore->hidden('fk_importatm_schema', $fk_importatm_schema);
	if (!empty($TCol)) print $formcore->hidden('TCol', base64_encode(serialize($TCol)));
	if (!empty($TKey_to_update)) print $formcore->hidden('TKey_to_update', base64_encode(serialize($TKey_to_update)));
	
	if (empty($action))
	{
		_step1($form, $TTypeObjectToImport);
	}
	elseif ($action == 'init_import')
	{
		_step2($object, $form, $TTypeObjectToImport, $type_object, $filename);
	}
	elseif ($action == 'simulate_import')
	{
		_step3($object, $form, $TTypeObjectToImport, $type_object, $filename, $TCol);
	}
	
	print $formcore->end_form();

	llxFooter();
}


function _step1(&$form, $TTypeObjectToImport)
{
	global $langs,$bc;
	
	print '<table id="importatm_table" class="noborder" width="100%">';
	
	$var=false;
	print '<tr '.$bc[$var].'>';
	print '<td class="fieldrequired" width="15%">'.$langs->trans('Importatm_type_import').'</td>';
	print '<td>'.$form->selectarray('type_object', $TTypeObjectToImport, 'type_object').'</td>';
	print '</tr>';
	$var=!$var;
	print '<tr '.$bc[$var].'>';
	print '<td class="fieldrequired" width="15%">'.$langs->trans('Importatm_file').'</td>';
	print '<td>'.$form->textwithpicto('<input type="file" name="file" value="" />', $langs->trans('Importatm_tooltip_csv')).'</td>';
	print '</tr>';
	
	print '</table>';
	
	print '<div class="center"><input class="button" value="'.$langs->trans('Importatm_btinit_import').'" type="submit"></div>';
}

function _step2(&$object, &$form, $TTypeObjectToImport, $type_object, $filename)
{	
	_showParams($object, $TTypeObjectToImport, $type_object, $filename);
	_showFormSchema($object, $type_object);
	_showContent($object, $form, 'before_simulation');
}

function _step3(&$object, &$form, $TTypeObjectToImport, $type_object, $filename, $TCol)
{
	_showParams($object, $TTypeObjectToImport, $type_object, $filename);
	_showStats($object);
	_showContent($object, $form, 'after_simulation', $TCol);
}


function _showParams(&$object, $TTypeObjectToImport, $type_object, $filename)
{
	global $langs,$bc;
	
	print '<table id="importatm_table" class="noborder" width="100%">';
	
	print '<tr class="liste_titre">';
	print '<th colspan="2">'.$langs->trans('Importatm_Parameters').'</th>';
	print '</tr>';

	$var=false;
	print '<tr '.$bc[$var].'>';
	print '<td width="15%">'.$langs->trans('Importatm_type_import').'</td><td>'.$TTypeObjectToImport[$type_object].'</td>';
	print '</tr>';
	$var=!$var;
	print '<tr '.$bc[$var].'>';
	print '<td width="15%">'.$langs->trans('Importatm_file').'</td><td>'.$filename.'</td>';
	print '</tr>';
	$var=!$var;
	print '<tr '.$bc[$var].'>';
	print '<td width="15%">'.$langs->trans('Importatm_nb_lines').'</td><td>'.$object->nb_lines.'</td>';
	print '</tr>';
	
	print '</table><br />';
}

function _showFormSchema($object, $type_object)
{
	global $langs, $form;
	
	$TUsableKeys = $object->getTUsableKeyToUpdate();
	
	print '<table class="border">';
	
	print '<tr>';
	print '<td width="250px"><b>'.$langs->trans('Importatm_select_schema').'</b></td>';
	print '<td width="250px"><select id="importatm_schema" name="fk_importatm_schema" class="minwidth100"></select></td>';
	print '<td width="200px"><input type="button" id="importatm_schema_delete" class="button" value="'.$langs->trans('Delete').'" /></td>';
	print '<td width="250px" rowspan="2" class="center"><input class="butAction" name="simulate" value="'.$langs->trans('Importatm_btsimulate_import').'" type="submit"></td>';
	print '</tr>';
	
	print '<tr>';
	print '<td width="250px"><b>'.$langs->trans('Importatm_save_schema').'</b></td>';
	print '<td width="250px"><input type="text" id="importatm_schema_name" value="" /></td>';
	print '<td width="200px"><input type="button" id="importatm_schema_save" class="button" value="'.$langs->trans('Save').'" /></td>';
	print '</tr>';
	
	print '<tr>';
	print '<td width="250px"><b>'.$langs->trans('Importatm_select_key_to_update').'</b></td>';
	print '<td colspan="3">';
	print $form->multiselectarray('TKey_to_update', $TUsableKeys, GETPOST('TKey_to_update'), 0, 0, 'minwidth200');
	print '</td>';
	print '</tr>';
	
	print '</table>';

	?>
	<script type="text/javascript">
		$(function() {
			$('#importatm_schema_save').click(function() {
				if ($('#importatm_schema_name').val() == '') $.jnotify("<?php echo $langs->trans('Importatm_warning_name_schema_empty'); ?>", 'warning');
				else {
					var data = {put:'schema', name:$('#importatm_schema_name').val(), type_object:'<?php echo $type_object; ?>', json:1, TField:[]};
					$('select.TFieldToImport').each(function(i, item) {
						data.TField.push($(item).val());
					});
					
					$.post("<?php echo dol_buildpath('/importatm/script/interface.php', 1); ?>", data, function(response) {
						importatm_load_schema();
					});
				}
			});
			
			importatm_load_schema = function(first_load) {
				var data = {get:'TSchema', type_object:'<?php echo $type_object; ?>', json:1};
				$.post("<?php echo dol_buildpath('/importatm/script/interface.php', 1); ?>", data, function(response) {
					console.log(response);
					if (response.TSchema.length > 0)
					{
						$('#importatm_schema').empty();
						$('#importatm_schema').append('<option value=""></option>');
						
						for (let i = 0; i < response.TSchema.length; i++) {
							var option = $('<option>');
							$(option).val(response.TSchema[i].rowid);
							$(option).text(response.TSchema[i].name);
							$(option).data('schema', response.TSchema[i].schema_import);

							$('#importatm_schema').append(option);
						}
						
						if (first_load === true) {
							$('#importatm_schema').val('<?php echo GETPOST('fk_importatm_schema'); ?>').trigger('change');
						}
					}
				}, 'json');
			};
			
			importatm_load_schema(true);
			
			$('#importatm_schema').change(function(event) {
				$('#importatm_schema_name').val('');
				var TField = $('option:selected', this).data('schema');
				if (typeof TField == 'undefined') $('select.TFieldToImport').val('').trigger('change');
				else {
					$('select.TFieldToImport').each(function(i, item) {
						$(item).val(TField[i]).trigger('change');
					});
				}
			});
			
			
			$('#importatm_schema_delete').click(function() {
				if ($('#importatm_schema').val() != '') {
					var data = {put:'deleteSchema', json:1, id:$('#importatm_schema').val()};
					$.post("<?php echo dol_buildpath('/importatm/script/interface.php', 1); ?>", data, function(response) {
						importatm_load_schema();
					});
				}
			});
		});
	</script>
	<?php
}

function _showStats(&$object)
{
	global $langs;
	
	print '<div class="info ok">'.$langs->trans('Importatm_nb_lines_inserted', $object->nb_create).'<br />'.$langs->trans('Importatm_nb_lines_updated', $object->nb_update).'</div>';
	if ($object->nb_errors > 0) print '<div class="warning error">'.$langs->trans('Importatm_nb_lines_errored', $object->nb_errors).'</div>';
}

function _showContent(&$object, $form, $mode='before_simulation', $TCol=array())
{
	global $langs;
	if ($mode === 'after_simulation') print '<div class="tabBar"><input class="butAction" value="'.$langs->trans('Importatm_btconfirm_import').'" type="submit"></div>';
	
	if ($mode === 'before_simulation')
	{
		print '<script type="text/javascript"> $(function() { $(".TFieldToImport").select2({width:"resolve"}); }); </script>';
	}
	
	$more = '';
	if (in_array(get_class($object), array('TImportatmSociete', 'TImportatmActionComm')))
	{
		$more .= 'listwithfilterbefore';
		// TODO affectation du commercial ou du propriétaire de l'event
		print '<div class="liste_titre liste_titre_bydiv centpercent">';
		if ($mode === 'before_simulation')
		{
			// TODO à modifier pour avoir un multiselect
			print '<div class="divsearchfield">'.$langs->trans('Importatm_default_user').' '.$form->select_dolusers(GETPOST('TUserId'), 'TUserId', 1, null, 0, '', '', 0, 0, 0, '', 0, '', '', 1).'</div>';
		}
		else
		{
			print '<div class="divsearchfield">';
			print $langs->trans('Importatm_default_user').' '.$form->select_dolusers(GETPOST('TUserId'), 'TUserIdDisabled', 1, null, 1, '', '', 0, 0, 0, '', 0, '', '', 1);
			// TODO quand ça sera un multiselect il faudra faire un implode(',')
			print '<input type="hidden" name="TUserId" value="'.GETPOST('TUserId').'" />';
			print '</div>';
		}
		print '</div>';
	}
	
	
	print '<table id="importatm_table_data" class="noborder '.$more.'" width="100%">';
	
	$TField = $object->getTField();
	
	print '<tr class="liste_titre">';
	if ($mode === 'before_simulation')
	{
		print '<td><input type="checkbox" name="checkall" value="1" onclick="$(\'.TLineToImport\').prop(\'checked\', $(this).prop(\'checked\'));" /></td>';
		
		$options = array();
		foreach ($TField as $key => &$Tab)
		{
			$options[$key] = $Tab['label'].' '.$Tab['info'];
		}

		foreach ($object->TData[1] as $i => &$Tab)
		{
			print '<td>'.$form->selectarray('TCol['.$i.']', $options, '', 0, 0, 0, '', 0, 0, 0, '', 'TFieldToImport maxwidth200', 1).'</td>';
		}
	}
	else
	{
		// Après la simulation (donc juste avant la confirmation de l'import) je n'affiche que les lignes et clonnes à importer
		foreach ($TCol as $attr)
		{
			if (!empty($attr) && $attr != -1) print '<td>'.$TField[$attr]['label'].'</td>';
		}
	}
	print '</tr>';
	
	foreach ($object->TData as $line_index => &$Tab)
	{
		print '<tr id="row-'.$line_index.'">';
		
		// Avant de lancer une simulation, l'utilisateur a le choix cocher les lignes à importer
		if ($mode === 'before_simulation') print '<td><input type="checkbox" class="TLineToImport" name="TLineToImport['.$line_index.']" value="'.$line_index.'" /></td>';
		
		foreach ($Tab as $data_index => &$val)
		{
			if ($mode === 'before_simulation') print '<td>'.$val.'</td>';
			else
			{
				if (!empty($TCol[$data_index]) && $TCol[$data_index] != -1) print '<td>'.$val.'</td>';
			}
		}
		
		print '</tr>';
	}
	
	print '</table>';
}
