<?php

/**
 * Copyright © 2015 Marcos García de La Fuente <hola@marcosgdf.com>
 *
 * This file is part of Multismtp.
 *
 * Multismtp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Multismtp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Multismtp.  If not, see <http://www.gnu.org/licenses/>.
 */

class Multismtp
{
	/**
	 * User
	 * @var int
	 */
	private $fk_user;

	/**
	 * SMTP server
	 * @var string
	 */
	public $smtp_server = null;

	/**
	 * SMTP port
	 * @var string
	 */
	public $smtp_port = null;

	/**
	 * SMTP TLS
	 * @var bool
	 */
	public $smtp_tls = null;

	/**
	 * SMTP username
	 * @var string
	 */
	public $smtp_id;

	/**
	 * SMTP password
	 * @var string
	 */
	public $smtp_pw;

	/**
	 * Database handler
	 * @var DoliDB
	 */
	private $db;

	/**
	 * Multismtp constructor.
	 * Cannot use typehinting because of 3.4 compatibility
	 *
	 * @param DoliDB $db Database handler
	 */
	public function __construct($db)
	{
		$this->db = $db;
	}

	/**
	 * Retrieves SMTP configuration for the given user
	 *
	 * @param User $user User to retrieve data from
	 * @return bool true if everything was OK, false if no records found
	 * @throws Exception When DB error
	 */
	public function fetch(User $user)
	{
		$this->fk_user = $user->id;

		$sql = 'SELECT * FROM '.MAIN_DB_PREFIX.'user2smtp WHERE fk_user = '.(int)$this->fk_user;

		$query = $this->db->query($sql);

		if (!$query) {
			throw new Exception($this->db->error());
		}

		if (!$this->db->num_rows($query)) {
			return false;
		}

		$resql = $this->db->fetch_object($query);

		$this->smtp_id = $resql->smtp_id;
		$this->smtp_pw = $resql->smtp_pw;
		$this->smtp_server = $resql->smtp_server;
		$this->smtp_port = $resql->smtp_port;
		$this->smtp_tls = (bool) $resql->smtp_tls;

		return true;
	}

	/**
	 * Updates the data
	 *
	 * @return bool
	 * @throws Exception When a DB error happens
	 * @throws BadMethodCallException When fk_user is not set
	 */
	public function update()
	{
		if (!$this->fk_user) {
			throw new BadMethodCallException();
		}

		$smtp_port = 'NULL';
		$smtp_tls = 'NULL';

		if ($this->smtp_port !== null) {
			$smtp_port = (int)$this->smtp_port;
		}
		if ($this->smtp_tls !== null) {
			$smtp_tls = (int)$this->smtp_tls;
		}

		$sql = "INSERT INTO ".MAIN_DB_PREFIX."user2smtp (smtp_id, smtp_pw, smtp_server, smtp_port, smtp_tls, fk_user)
			VALUES ('".$this->db->escape($this->smtp_id)."',
			'".$this->db->escape($this->smtp_pw)."',
			'".$this->db->escape($this->smtp_server)."',
			".$smtp_port.",
			".$smtp_tls.",
			".(int)$this->fk_user.")";

		$query = $this->db->query($sql);

		if (!$query) {

			$sql = "UPDATE ".MAIN_DB_PREFIX."user2smtp SET
		smtp_id = '".$this->db->escape($this->smtp_id)."',
		smtp_pw = '".$this->db->escape($this->smtp_pw)."',
		smtp_server = '".$this->db->escape($this->smtp_server)."',
		smtp_port = ".$smtp_port.",
		smtp_tls = ".$smtp_tls."
		WHERE fk_user = ".(int)$this->fk_user;

			$subquery = $this->db->query($sql);

			if (!$subquery) {
				throw new Exception($this->db->error());
			}

			return true;
		}

		return true;
	}

}