<?php

/**
 * Copyright © 2015 Marcos García de La Fuente <hola@marcosgdf.com>
 *
 * This file is part of Multismtp.
 *
 * Multismtp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Multismtp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Multismtp.  If not, see <http://www.gnu.org/licenses/>.
 */

//Handling custom/not custom directory

if (file_exists('../main.inc.php')) {
        require '../main.inc.php';
} 
if (file_exists('../../main.inc.php')) {
        require '../../main.inc.php';
} 
 else {
	require '../../../main.inc.php';
}

require_once dirname(__FILE__).'/class/Multismtp.class.php';
require_once DOL_DOCUMENT_ROOT.'/core/lib/usergroups.lib.php';

// Defini si peux lire/modifier permisssions
$canreaduser=($user->admin || $user->rights->user->user->lire);

$action = GETPOST('action');
$id = GETPOST('id','int');

$result = restrictedArea($user, 'user', $id, '&user', $feature2);
if ($user->id <> $id && ! $canreaduser) accessforbidden();

// Charge utilisateur edite
$fuser = new User($db);
if ($fuser->fetch($id) < 0) {
	return dol_print_error($db, 'User not found');
}

$multismtp = new Multismtp($db);

$multismtp->fetch($fuser);

if ($action == 'update' && empty($_POST["cancel"])) {

	$main_mail_smtps_pw = GETPOST('MAIN_MAIL_SMTPS_PW');
	$mail_mail_smtps_tls = GETPOST('MAIN_MAIL_EMAIL_TLS', 'int');
	$mail_mail_smtps_server = GETPOST('MAIN_MAIL_SMTP_SERVER');
	$mail_mail_smtps_port = GETPOST('MAIN_MAIL_SMTP_PORT');

	if ($conf->global->MULTISMTP_ALLOW_CHANGESERVER == 1) {

		$multismtp->smtp_tls = (bool) $mail_mail_smtps_tls;

		if (!empty($mail_mail_smtps_server)) {
			$multismtp->smtp_server = $mail_mail_smtps_server;
		}

		if (!empty($mail_mail_smtps_port)) {
			$multismtp->smtp_port = $mail_mail_smtps_port;
		}
	}

	$multismtp->smtp_id = GETPOST('MAIN_MAIL_SMTPS_ID');

	//Users which do not enter password do not want to change it
	if ($main_mail_smtps_pw) {
		$multismtp->smtp_pw = $main_mail_smtps_pw;
	}

	try {
		$multismtp->update();

		$langs->load('admin');
		setEventMessage('SetupSaved');
	} catch (Exception $e) {
		setEventMessage($e->getMessage(), 'errors');
	}
}

/*
 * View
 */

$langs->load("admin");

$form = new Form($db);

llxHeader('', $langs->trans('Email'));

$head = user_prepare_head($fuser);

$title = $langs->trans("User");

dol_fiche_head($head, 'email', $title, 0, 'user');

if ($action == 'edit') {

	print '<form method="POST" action="'.$_SERVER['PHP_SELF'].'?id='.$id.'">';
	print '<input type="hidden" name="action" value="update">';
}

print '<table class="border" width="100%">';

// Ref
print '<tr><td width="25%" valign="top">'.$langs->trans("Ref").'</td>';
print '<td colspan="2">';
print $form->showrefnav($fuser,'id','',$user->rights->user->user->lire || $user->admin);
print '</td>';
print '</tr>';

// LastName
print '<tr><td width="25%" valign="top">'.$langs->trans("LastName").'</td>';
print '<td colspan="2">'.$fuser->lastname.'</td>';
print "</tr>\n";

// FirstName
print '<tr><td width="25%" valign="top">'.$langs->trans("FirstName").'</td>';
print '<td colspan="2">'.$fuser->firstname.'</td>';
print "</tr>\n";

// Email
print '<tr><td width="25%" valign="top">'.$langs->trans("Email").'</td>';
print '<td colspan="2">'.$fuser->email.'</td>';
print "</tr>\n";

$smtp_iniserver = ini_get('SMTP') ? ini_get('SMTP') : $langs->transnoentities("Undefined");
$smtp_iniport = ini_get('smtp_port') ? ini_get('smtp_port') : $langs->transnoentities("Undefined");

if (!$multismtp->smtp_server) {
	$smtp_server = $conf->global->MAIN_MAIL_SMTP_SERVER;
} else {
	$smtp_server = $multismtp->smtp_server;
}

if ($multismtp->smtp_tls === null) {
	$smtp_tls = $conf->global->MAIN_MAIL_EMAIL_TLS;
} else {
	$smtp_tls = $multismtp->smtp_tls;
}

if (!$multismtp->smtp_port) {
	$smtp_port = $conf->global->MAIN_MAIL_SMTP_PORT;
} else {
	$smtp_port = $multismtp->smtp_port;
}

if ($action == 'edit') {

	// Server
	print '<tr><td>'.$langs->trans("MAIN_MAIL_SMTP_SERVER", $smtp_iniserver).'</td><td>';

	if ($conf->global->MULTISMTP_ALLOW_CHANGESERVER == 1) {
		print '<input type="text" class="flat" size="32" name="MAIN_MAIL_SMTP_SERVER" value="'.$smtp_server.'">';
	}
	else {
		print $conf->global->MAIN_MAIL_SMTP_SERVER;
	}
	print '</td></tr>';

	// Port
	print '<tr><td>'.$langs->trans("MAIN_MAIL_SMTP_PORT", $smtp_iniport).'</td><td>';
	if ($conf->global->MULTISMTP_ALLOW_CHANGESERVER == 1) {
		print '<input type="text" class="flat" size="3" name="MAIN_MAIL_SMTP_PORT" value="'.$smtp_port.'">';
	} else {
		print $conf->global->MAIN_MAIL_SMTP_PORT;
	}
	print '</td></tr>';

	// TLS
	print '<tr><td>'.$langs->trans("MAIN_MAIL_EMAIL_TLS").'</td><td>';
	if (function_exists('openssl_open')) {
		if ($conf->global->MULTISMTP_ALLOW_CHANGESERVER == 1) {
			print $form->selectyesno('MAIN_MAIL_EMAIL_TLS', $smtp_tls,1);
		} else {
			print yn($smtp_tls);
		}
	} else {
		print yn(0).' ('.$langs->trans("YourPHPDoesNotHaveSSLSupport").')';
	}
	print '</td></tr>';

	// SMTPS ID
	print '<tr><td>'.$langs->trans("MAIN_MAIL_SMTPS_ID").'</td><td><input type="text" class="flat" size="32" name="MAIN_MAIL_SMTPS_ID" value="'.$multismtp->smtp_id.'"></td></tr>';

	// SMTPS PW
	print '<tr><td>'.$langs->trans("MAIN_MAIL_SMTPS_PW").'</td><td><input type="password" class="flat" size="32" name="MAIN_MAIL_SMTPS_PW" value=""></td></tr>';

	print '</table><br>';

	print '<center>';
	print '<input class="button" type="submit" name="save" value="'.$langs->trans("Save").'">';
	print ' &nbsp; &nbsp; ';
	print '<input class="button" type="submit" name="cancel" value="'.$langs->trans("Cancel").'">';
	print '</center></form>';
} else {

	// Server
	print '<tr><td>'.$langs->trans("MAIN_MAIL_SMTP_SERVER", $smtp_iniserver).'</td><td>'.$smtp_server.'</td></tr>';

	// Port
	print '<tr><td>'.$langs->trans("MAIN_MAIL_SMTP_PORT", $smtp_iniport).'</td><td>'.$smtp_port.'</td></tr>';

	// TLS
	print '<tr><td>'.$langs->trans("MAIN_MAIL_EMAIL_TLS").'</td><td>';
	if (function_exists('openssl_open')) {
		print yn($smtp_tls);
	} else {
		print yn(0).' ('.$langs->trans("YourPHPDoesNotHaveSSLSupport").')';
	}
	print '</td></tr>';

	// SMTPS ID
	print '<tr><td>'.$langs->trans("MAIN_MAIL_SMTPS_ID").'</td><td>'.$multismtp->smtp_id.'</td></tr>';

	// SMTPS PW
	print '<tr><td>'.$langs->trans("MAIN_MAIL_SMTPS_PW").'</td><td>'.preg_replace('/./', '*',
			$multismtp->smtp_pw).'</td></tr>';

	print '</table><br>';
}

dol_fiche_end();

if ($action != 'edit') {
// Boutons actions
	print '<div class="tabsAction">';

	print '<a class="butAction" href="'.$_SERVER["PHP_SELF"].'?id='.$id.'&action=edit">'.$langs->trans("Modify").'</a>';

	print '</div>';
}

llxFooter();
